@@ -232,6 +232,9 @@ func TestParseLevel(t *testing.T) {
 	assert.Equal(t, zapcore.PanicLevel, *parseLevel("PAnIC"))
 	assert.Equal(t, zapcore.FatalLevel, *parseLevel("faTal"))
 	assert.Assert(t, parseLevel("x") == nil, "parse error")
+
+	assert.Assert(t, parseLevel("-129") == nil, "Values outside int8 range (-128 to 127)")
+	assert.Assert(t, parseLevel("128") == nil, "Values outside int8 range (-128 to 127)")
 }
 
 func TestParentLogger(t *testing.T) {
