@@ -19,9 +19,9 @@
 package scheduler
 
 import (
+	"errors"
 	"fmt"
 	"math"
-	"strconv"
 	"time"
 
 	"go.uber.org/zap"
@@ -131,22 +131,22 @@ func (cc *ClusterContext) schedule() bool {
 		}
 		// try reservations first
 		schedulingStart := time.Now()
-		alloc := psc.tryReservedAllocate()
-		if alloc == nil {
+		result := psc.tryReservedAllocate()
+		if result == nil {
 			// placeholder replacement second
-			alloc = psc.tryPlaceholderAllocate()
+			result = psc.tryPlaceholderAllocate()
 			// nothing reserved that can be allocated try normal allocate
-			if alloc == nil {
-				alloc = psc.tryAllocate()
+			if result == nil {
+				result = psc.tryAllocate()
 			}
 		}
-		if alloc != nil {
+		if result != nil {
 			metrics.GetSchedulerMetrics().ObserveSchedulingLatency(schedulingStart)
-			if alloc.GetResult() == objects.Replaced {
+			if result.ResultType == objects.Replaced {
 				// communicate the removal to the RM
-				cc.notifyRMAllocationReleased(psc.RmID, alloc.GetReleasesClone(), si.TerminationType_PLACEHOLDER_REPLACED, "replacing allocationID: "+alloc.GetAllocationID())
+				cc.notifyRMAllocationReleased(psc.RmID, psc.Name, []*objects.Allocation{result.Request.GetRelease()}, si.TerminationType_PLACEHOLDER_REPLACED, "replacing allocationKey: "+result.Request.GetAllocationKey())
 			} else {
-				cc.notifyRMNewAllocation(psc.RmID, alloc)
+				cc.notifyRMNewAllocation(psc.RmID, result.Request)
 			}
 			activity = true
 		}
@@ -555,9 +555,6 @@ func (cc *ClusterContext) handleRMUpdateApplicationEvent(event *rmevent.RMUpdate
 	}
 	// Update metrics with removed applications
 	if len(request.Remove) > 0 {
-		metrics.GetSchedulerMetrics().SubTotalApplicationsRunning(len(request.Remove))
-		// ToDO: need to improve this once we have state in YuniKorn for apps.
-		metrics.GetSchedulerMetrics().AddTotalApplicationsCompleted(len(request.Remove))
 		for _, app := range request.Remove {
 			partition := cc.GetPartition(app.PartitionName)
 			if partition == nil {
@@ -565,7 +562,7 @@ func (cc *ClusterContext) handleRMUpdateApplicationEvent(event *rmevent.RMUpdate
 			}
 			allocations := partition.removeApplication(app.ApplicationID)
 			if len(allocations) > 0 {
-				cc.notifyRMAllocationReleased(partition.RmID, allocations, si.TerminationType_STOPPED_BY_RM,
+				cc.notifyRMAllocationReleased(partition.RmID, partition.Name, allocations, si.TerminationType_STOPPED_BY_RM,
 					fmt.Sprintf("Application %s Removed", app.ApplicationID))
 			}
 			log.Log(log.SchedContext).Info("Application removed from partition",
@@ -608,11 +605,10 @@ func (cc *ClusterContext) addNode(nodeInfo *si.NodeInfo, schedulable bool) error
 		return err
 	}
 
-	existingAllocations := cc.convertAllocations(nodeInfo.ExistingAllocations)
-	err := partition.AddNode(sn, existingAllocations)
+	err := partition.AddNode(sn)
 	sn.SendNodeAddedEvent()
 	if err != nil {
-		wrapped := fmt.Errorf("failure while adding new node, node rejected with error: %w", err)
+		wrapped := errors.Join(errors.New("failure while adding new node, node rejected with error: "), err)
 		log.Log(log.SchedContext).Error("Failed to add node to partition (rejected)",
 			zap.String("nodeID", sn.NodeID),
 			zap.String("partitionName", sn.Partition),
@@ -620,9 +616,6 @@ func (cc *ClusterContext) addNode(nodeInfo *si.NodeInfo, schedulable bool) error
 		return wrapped
 	}
 
-	if !sn.IsReady() {
-		metrics.GetSchedulerMetrics().IncUnhealthyNodes()
-	}
 	if !sn.IsSchedulable() {
 		metrics.GetSchedulerMetrics().IncDrainingNodes()
 	}
@@ -665,24 +658,6 @@ func (cc *ClusterContext) updateNode(nodeInfo *si.NodeInfo) {
 
 	switch nodeInfo.Action {
 	case si.NodeInfo_UPDATE:
-		var newReadyStatus bool
-		var err error
-		if newReadyStatus, err = strconv.ParseBool(nodeInfo.Attributes[siCommon.NodeReadyAttribute]); err != nil {
-			log.Log(log.SchedContext).Error("Could not parse ready attribute, assuming true", zap.Any("attributes", nodeInfo.Attributes))
-			newReadyStatus = true
-		}
-
-		if node.IsReady() && !newReadyStatus {
-			log.Log(log.SchedContext).Info("Node has become unhealthy", zap.String("Node ID", node.NodeID))
-			metrics.GetSchedulerMetrics().IncUnhealthyNodes()
-			node.SetReady(newReadyStatus)
-		}
-		if !node.IsReady() && newReadyStatus {
-			log.Log(log.SchedContext).Info("Node has become healthy", zap.String("Node ID", node.NodeID))
-			metrics.GetSchedulerMetrics().DecUnhealthyNodes()
-			node.SetReady(newReadyStatus)
-		}
-
 		if sr := nodeInfo.SchedulableResource; sr != nil {
 			partition.updatePartitionResource(node.SetCapacity(resources.NewResourceFromProto(sr)))
 		}
@@ -705,9 +680,6 @@ func (cc *ClusterContext) updateNode(nodeInfo *si.NodeInfo) {
 		if !node.IsSchedulable() {
 			metrics.GetSchedulerMetrics().DecDrainingNodes()
 		}
-		if !node.IsReady() {
-			metrics.GetSchedulerMetrics().DecUnhealthyNodes()
-		}
 		metrics.GetSchedulerMetrics().IncTotalDecommissionedNodes()
 		// set the state to not schedulable then tell the partition to clean up
 		node.SetSchedulable(false)
@@ -715,7 +687,7 @@ func (cc *ClusterContext) updateNode(nodeInfo *si.NodeInfo) {
 		node.SendNodeRemovedEvent()
 		// notify the shim allocations have been released from node
 		if len(released) != 0 {
-			cc.notifyRMAllocationReleased(partition.RmID, released, si.TerminationType_STOPPED_BY_RM,
+			cc.notifyRMAllocationReleased(partition.RmID, partition.Name, released, si.TerminationType_STOPPED_BY_RM,
 				fmt.Sprintf("Node %s Removed", node.NodeID))
 		}
 		for _, confirm := range confirmed {
@@ -739,13 +711,7 @@ func (cc *ClusterContext) handleRMUpdateAllocationEvent(event *rmevent.RMUpdateA
 	if len(request.Allocations) != 0 {
 		cc.processAllocations(request)
 	}
-	if len(request.Asks) != 0 {
-		cc.processAsks(request)
-	}
 	if request.Releases != nil {
-		if len(request.Releases.AllocationAsksToRelease) > 0 {
-			cc.processAskReleases(request.Releases.AllocationAsksToRelease)
-		}
 		if len(request.Releases.AllocationsToRelease) > 0 {
 			cc.processAllocationReleases(request.Releases.AllocationsToRelease, request.RmID)
 		}
@@ -776,13 +742,15 @@ func (cc *ClusterContext) processAllocations(request *si.AllocationRequest) {
 		}
 
 		alloc := objects.NewAllocationFromSI(siAlloc)
-		if err := partition.addAllocation(alloc); err != nil {
+
+		_, newAlloc, err := partition.UpdateAllocation(alloc)
+		if err != nil {
 			rejectedAllocs = append(rejectedAllocs, &si.RejectedAllocation{
 				AllocationKey: siAlloc.AllocationKey,
 				ApplicationID: siAlloc.ApplicationID,
 				Reason:        err.Error(),
 			})
-			log.Log(log.SchedContext).Error("Invalid allocation add requested by shim",
+			log.Log(log.SchedContext).Error("Invalid allocation update requested by shim",
 				zap.String("partition", siAlloc.PartitionName),
 				zap.String("nodeID", siAlloc.NodeID),
 				zap.String("applicationID", siAlloc.ApplicationID),
@@ -790,7 +758,10 @@ func (cc *ClusterContext) processAllocations(request *si.AllocationRequest) {
 				zap.Error(err))
 			continue
 		}
-		cc.notifyRMNewAllocation(request.RmID, alloc)
+		// at some point, we may need to handle new requests as well
+		if newAlloc {
+			cc.notifyRMNewAllocation(request.RmID, alloc)
+		}
 	}
 
 	// Reject allocs returned to RM proxy for the apps and partitions not found
@@ -802,67 +773,6 @@ func (cc *ClusterContext) processAllocations(request *si.AllocationRequest) {
 	}
 }
 
-func (cc *ClusterContext) processAsks(request *si.AllocationRequest) {
-	// Send rejects back to RM
-	rejectedAsks := make([]*si.RejectedAllocationAsk, 0)
-
-	// Send to scheduler
-	for _, siAsk := range request.Asks {
-		// try to get ApplicationInfo
-		partition := cc.GetPartition(siAsk.PartitionName)
-		if partition == nil {
-			msg := fmt.Sprintf("Failed to find partition %s, for application %s and allocation %s", siAsk.PartitionName, siAsk.ApplicationID, siAsk.AllocationKey)
-			log.Log(log.SchedContext).Error("Invalid ask add requested by shim, partition not found",
-				zap.String("partition", siAsk.PartitionName),
-				zap.String("applicationID", siAsk.ApplicationID),
-				zap.String("askKey", siAsk.AllocationKey))
-			rejectedAsks = append(rejectedAsks, &si.RejectedAllocationAsk{
-				AllocationKey: siAsk.AllocationKey,
-				ApplicationID: siAsk.ApplicationID,
-				Reason:        msg,
-			})
-			continue
-		}
-
-		// try adding to app
-		if err := partition.addAllocationAsk(siAsk); err != nil {
-			rejectedAsks = append(rejectedAsks,
-				&si.RejectedAllocationAsk{
-					AllocationKey: siAsk.AllocationKey,
-					ApplicationID: siAsk.ApplicationID,
-					Reason:        err.Error(),
-				})
-			log.Log(log.SchedContext).Error("Invalid ask add requested by shim",
-				zap.String("partition", siAsk.PartitionName),
-				zap.String("applicationID", siAsk.ApplicationID),
-				zap.String("askKey", siAsk.AllocationKey),
-				zap.Error(err))
-		}
-	}
-
-	// Reject asks returned to RM Proxy for the apps and partitions not found
-	if len(rejectedAsks) > 0 {
-		cc.rmEventHandler.HandleEvent(&rmevent.RMRejectedAllocationAskEvent{
-			RmID:                   request.RmID,
-			RejectedAllocationAsks: rejectedAsks,
-		})
-	}
-}
-
-func (cc *ClusterContext) processAskReleases(releases []*si.AllocationAskRelease) {
-	for _, toRelease := range releases {
-		partition := cc.GetPartition(toRelease.PartitionName)
-		if partition == nil {
-			log.Log(log.SchedContext).Error("Invalid ask release requested by shim, partition not found",
-				zap.String("partition", toRelease.PartitionName),
-				zap.String("applicationID", toRelease.ApplicationID),
-				zap.String("askKey", toRelease.AllocationKey))
-			continue
-		}
-		partition.removeAllocationAsk(toRelease)
-	}
-}
-
 func (cc *ClusterContext) processAllocationReleases(releases []*si.AllocationRelease, rmID string) {
 	for _, toRelease := range releases {
 		partition := cc.GetPartition(toRelease.PartitionName)
@@ -870,7 +780,7 @@ func (cc *ClusterContext) processAllocationReleases(releases []*si.AllocationRel
 			allocs, confirmed := partition.removeAllocation(toRelease)
 			// notify the RM of the exact released allocations
 			if len(allocs) > 0 {
-				cc.notifyRMAllocationReleased(rmID, allocs, si.TerminationType_STOPPED_BY_RM, "allocation remove as per RM request")
+				cc.notifyRMAllocationReleased(rmID, partition.Name, allocs, si.TerminationType_STOPPED_BY_RM, "allocation remove as per RM request")
 			}
 			// notify the RM of the confirmed allocations (placeholder swap & preemption)
 			if confirmed != nil {
@@ -880,16 +790,6 @@ func (cc *ClusterContext) processAllocationReleases(releases []*si.AllocationRel
 	}
 }
 
-// Convert the si allocation to a proposal to add to the node
-func (cc *ClusterContext) convertAllocations(allocations []*si.Allocation) []*objects.Allocation {
-	convert := make([]*objects.Allocation, len(allocations))
-	for current, allocation := range allocations {
-		convert[current] = objects.NewAllocationFromSI(allocation)
-	}
-
-	return convert
-}
-
 // Create a RM update event to notify RM of new allocations
 // Lock free call, all updates occur via events.
 func (cc *ClusterContext) notifyRMNewAllocation(rmID string, alloc *objects.Allocation) {
@@ -912,7 +812,7 @@ func (cc *ClusterContext) notifyRMNewAllocation(rmID string, alloc *objects.Allo
 
 // Create a RM update event to notify RM of released allocations
 // Lock free call, all updates occur via events.
-func (cc *ClusterContext) notifyRMAllocationReleased(rmID string, released []*objects.Allocation, terminationType si.TerminationType, message string) {
+func (cc *ClusterContext) notifyRMAllocationReleased(rmID string, partitionName string, released []*objects.Allocation, terminationType si.TerminationType, message string) {
 	c := make(chan *rmevent.Result)
 	releaseEvent := &rmevent.RMReleaseAllocationEvent{
 		ReleasedAllocations: make([]*si.AllocationRelease, 0),
@@ -922,8 +822,7 @@ func (cc *ClusterContext) notifyRMAllocationReleased(rmID string, released []*ob
 	for _, alloc := range released {
 		releaseEvent.ReleasedAllocations = append(releaseEvent.ReleasedAllocations, &si.AllocationRelease{
 			ApplicationID:   alloc.GetApplicationID(),
-			PartitionName:   alloc.GetPartitionName(),
-			AllocationID:    alloc.GetAllocationID(),
+			PartitionName:   partitionName,
 			TerminationType: terminationType,
 			Message:         message,
 			AllocationKey:   alloc.GetAllocationKey(),
