@@ -120,9 +120,12 @@ func TestConfigUpdate(t *testing.T) {
 
 	// update config to run every 100ms and wait for refresh
 	configs.SetConfigMap(map[string]string{configs.HealthCheckInterval: "100ms"})
-	err = common.WaitForCondition(func() bool {
-		return healthChecker.GetPeriod() == 100*time.Millisecond
-	}, 10*time.Millisecond, 5*time.Second)
+	err = common.WaitForCondition(10*time.Millisecond,
+		5*time.Second,
+		func() bool {
+			return healthChecker.GetPeriod() == 100*time.Millisecond
+		},
+	)
 	assert.NilError(t, err, "timed out waiting for config refresh")
 
 	// clear results
@@ -134,9 +137,11 @@ func TestConfigUpdate(t *testing.T) {
 
 	// disable and wait for refresh
 	configs.SetConfigMap(map[string]string{configs.HealthCheckInterval: "0"})
-	err = common.WaitForCondition(func() bool {
-		return !healthChecker.IsEnabled()
-	}, 10*time.Millisecond, 5*time.Second)
+	err = common.WaitForCondition(10*time.Millisecond,
+		5*time.Second, func() bool {
+			return !healthChecker.IsEnabled()
+		},
+	)
 	assert.NilError(t, err, "timed out waiting for config refresh")
 
 	// clear results
@@ -193,18 +198,18 @@ func TestGetSchedulerHealthStatusContext(t *testing.T) {
 		SchedulableResource: &si.Resource{
 			Resources: map[string]*si.Quantity{"memory": {Value: -10}},
 		},
-	}), []*objects.Allocation{})
+	}))
 	assert.NilError(t, err, "Unexpected error while adding a new node")
 	healthInfo = GetSchedulerHealthStatus(schedulerMetrics, schedulerContext)
 	assert.Assert(t, !healthInfo.Healthy, "Scheduler should not be healthy")
 
 	// add orphan allocation to a node
 	node := schedulerContext.partitions[partName].nodes.GetNode("node")
-	alloc := objects.NewAllocation("node", newAllocationAsk("key", "appID", resources.NewResource()))
+	alloc := newAllocation("key", "appID", "node", resources.NewResource())
 	node.AddAllocation(alloc)
 	healthInfo = GetSchedulerHealthStatus(schedulerMetrics, schedulerContext)
 	assert.Assert(t, !healthInfo.Healthy, "Scheduler should not be healthy")
-	assert.Assert(t, !healthInfo.HealthChecks[9].Succeeded, "The orphan allocation check on the node should not be successful")
+	assert.Assert(t, !healthInfo.HealthChecks[7].Succeeded, "The orphan allocation check on the node should not be successful")
 
 	// add the allocation to the app as well
 	part := schedulerContext.partitions[partName]
@@ -213,13 +218,13 @@ func TestGetSchedulerHealthStatusContext(t *testing.T) {
 	err = part.AddApplication(app)
 	assert.NilError(t, err, "Could not add application")
 	healthInfo = GetSchedulerHealthStatus(schedulerMetrics, schedulerContext)
-	assert.Assert(t, healthInfo.HealthChecks[9].Succeeded, "The orphan allocation check on the node should be successful")
+	assert.Assert(t, healthInfo.HealthChecks[7].Succeeded, "The orphan allocation check on the node should be successful")
 
 	// remove the allocation from the node, so we will have an orphan allocation assigned to the app
-	node.RemoveAllocation("key-0")
+	node.RemoveAllocation("key")
 	healthInfo = GetSchedulerHealthStatus(schedulerMetrics, schedulerContext)
-	assert.Assert(t, healthInfo.HealthChecks[9].Succeeded, "The orphan allocation check on the node should be successful")
-	assert.Assert(t, !healthInfo.HealthChecks[10].Succeeded, "The orphan allocation check on the app should not be successful")
+	assert.Assert(t, healthInfo.HealthChecks[7].Succeeded, "The orphan allocation check on the node should be successful")
+	assert.Assert(t, !healthInfo.HealthChecks[8].Succeeded, "The orphan allocation check on the app should not be successful")
 }
 
 func TestGetSchedulerHealthStatusMetrics(t *testing.T) {
