@@ -48,16 +48,14 @@ type Allocation struct {
 	allowPreemptOther bool
 	originator        bool
 	tags              map[string]string
-	resKeyWithoutNode string // the reservation key without node
 
 	// Mutable fields which need protection
 	allocated            bool
 	allocLog             map[string]*AllocationLogEntry
 	preemptionTriggered  bool
 	preemptCheckTime     time.Time
-	schedulingAttempted  bool              // whether scheduler core has tried to schedule this allocation
-	scaleUpTriggered     bool              // whether this aloocation has triggered autoscaling or not
-	resKeyPerNode        map[string]string // reservation key for a given node
+	schedulingAttempted  bool // whether scheduler core has tried to schedule this allocation
+	scaleUpTriggered     bool // whether this allocation has triggered autoscaling or not
 	allocatedResource    *resources.Resource
 	askEvents            *schedEvt.AskEvents
 	userQuotaCheckFailed bool
@@ -129,8 +127,6 @@ func NewAllocationFromSI(alloc *si.Allocation) *Allocation {
 		allowPreemptOther: alloc.PreemptionPolicy.GetAllowPreemptOther(),
 		originator:        alloc.Originator,
 		allocLog:          make(map[string]*AllocationLogEntry),
-		resKeyPerNode:     make(map[string]string),
-		resKeyWithoutNode: reservationKeyWithoutNode(alloc.ApplicationID, alloc.AllocationKey),
 		askEvents:         schedEvt.NewAskEvents(events.GetEventSystem()),
 		allocated:         allocated,
 		nodeID:            nodeID,
@@ -457,6 +453,11 @@ func (a *Allocation) SendPredicatesFailedEvent(predicateErrors map[string]int) {
 	a.askEvents.SendPredicatesFailed(a.allocationKey, a.applicationID, predicateErrors, a.GetAllocatedResource())
 }
 
+// SendRequiredNodePreemptionFailedEvent updates the event system with required node preemption failed event.
+func (a *Allocation) SendRequiredNodePreemptionFailedEvent(node string) {
+	a.askEvents.SendRequiredNodePreemptionFailed(a.allocationKey, a.applicationID, node, a.GetAllocatedResource())
+}
+
 // GetAllocationLog returns a list of log entries corresponding to allocation preconditions not being met.
 func (a *Allocation) GetAllocationLog() []*AllocationLogEntry {
 	a.RLock()
@@ -526,18 +527,6 @@ func (a *Allocation) HasTriggeredScaleUp() bool {
 	return a.scaleUpTriggered
 }
 
-func (a *Allocation) setReservationKeyForNode(node, resKey string) {
-	a.Lock()
-	defer a.Unlock()
-	a.resKeyPerNode[node] = resKey
-}
-
-func (a *Allocation) getReservationKeyForNode(node string) string {
-	a.RLock()
-	defer a.RUnlock()
-	return a.resKeyPerNode[node]
-}
-
 func (a *Allocation) setHeadroomCheckFailed(headroom *resources.Resource, queue string) {
 	a.Lock()
 	defer a.Unlock()
