@@ -20,11 +20,11 @@ package objects
 
 import (
 	"fmt"
-	"sync"
 
 	"github.com/google/btree"
 	"go.uber.org/zap"
 
+	"github.com/apache/yunikorn-core/pkg/locking"
 	"github.com/apache/yunikorn-core/pkg/log"
 	"github.com/apache/yunikorn-core/pkg/scheduler/policies"
 )
@@ -37,6 +37,8 @@ var acceptAll = func(node *Node) bool {
 	return true
 }
 
+// NodeCollection represents a collection of nodes for a partition.
+// Implementations of this interface must be internally synchronized to avoid data races.
 type NodeCollection interface {
 	AddNode(node *Node) error
 	RemoveNode(nodeID string) *Node
@@ -79,7 +81,7 @@ type baseNodeCollection struct {
 	unreservedIterator *treeIterator
 	fullIterator       *treeIterator
 
-	sync.RWMutex
+	locking.RWMutex
 }
 
 func (nc *baseNodeCollection) scoreNode(node *Node) float64 {
