@@ -30,13 +30,19 @@ import (
 	"github.com/apache/yunikorn-core/pkg/scheduler/policies"
 )
 
-// verify queue ordering is working
-func TestSortQueues(t *testing.T) {
+// verify queue ordering is working when explicity guarantees are provided
+func TestSortQueuesWithGuarantees(t *testing.T) {
 	root, err := createRootQueue(nil)
 	assert.NilError(t, err, "queue create failed")
 
 	var q0, q1, q2, q3 *Queue
 	var queues []*Queue
+	fairMaxResources := []*resources.Resource{resources.NewResourceFromMap(map[string]resources.Quantity{}),
+		resources.NewResourceFromMap(map[string]resources.Quantity{}),
+		resources.NewResourceFromMap(map[string]resources.Quantity{}),
+		resources.NewResourceFromMap(map[string]resources.Quantity{}),
+	}
+
 	q0, err = createManagedQueue(root, "q0", false, nil)
 	assert.NilError(t, err, "failed to create leaf queue")
 	q0.guaranteedResource = resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 500, "vcore": 500})
@@ -63,54 +69,64 @@ func TestSortQueues(t *testing.T) {
 
 	// fifo
 	queues = []*Queue{q0, q1, q2, q3}
-	sortQueue(queues, policies.FifoSortPolicy, false)
+
+	sortQueue(queues, fairMaxResources, policies.FifoSortPolicy, false)
 	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q0, q1, q2, q3}), "fifo first")
 
 	queues = []*Queue{q0, q1, q2, q3}
-	sortQueue(queues, policies.FifoSortPolicy, true)
+	sortQueue(queues, fairMaxResources, policies.FifoSortPolicy, true)
 	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q0, q1, q2}), "fifo first - priority")
 
 	// fifo - different starting order
 	queues = []*Queue{q1, q3, q0, q2}
-	sortQueue(queues, policies.FifoSortPolicy, false)
+	sortQueue(queues, fairMaxResources, policies.FifoSortPolicy, false)
 	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q1, q3, q0, q2}), "fifo second")
 
 	queues = []*Queue{q1, q3, q0, q2}
-	sortQueue(queues, policies.FifoSortPolicy, true)
+	sortQueue(queues, fairMaxResources, policies.FifoSortPolicy, true)
 	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q1, q0, q2}), "fifo second - priority")
 
 	// fairness ratios: q0:300/500=0.6, q1:200/300=0.67, q2:100/200=0.5, q3:100/200=0.5
 	queues = []*Queue{q0, q1, q2, q3}
-	sortQueue(queues, policies.FairSortPolicy, false)
+	sortQueue(queues, fairMaxResources, policies.FairSortPolicy, false)
 	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q2, q0, q1}), "fair first")
 
 	queues = []*Queue{q0, q1, q2, q3}
-	sortQueue(queues, policies.FairSortPolicy, true)
+	sortQueue(queues, fairMaxResources, policies.FairSortPolicy, true)
 	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q2, q0, q1}), "fair first - priority")
 
 	// fairness ratios: q0:200/500=0.4, q1:300/300=1, q2:100/200=0.5, q3:100/200=0.5
 	q0.allocatedResource = resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 200, "vcore": 200})
 	q1.allocatedResource = resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 300, "vcore": 300})
 	queues = []*Queue{q0, q1, q2, q3}
-	sortQueue(queues, policies.FairSortPolicy, false)
+	sortQueue(queues, fairMaxResources, policies.FairSortPolicy, false)
 	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q0, q3, q2, q1}), "fair second")
 	queues = []*Queue{q0, q1, q2, q3}
-	sortQueue(queues, policies.FairSortPolicy, true)
+	sortQueue(queues, fairMaxResources, policies.FairSortPolicy, true)
 	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q0, q2, q1}), "fair second - priority")
 
 	// fairness ratios: q0:150/500=0.3, q1:120/300=0.4, q2:100/200=0.5, q3:100/200=0.5
 	q0.allocatedResource = resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 150, "vcore": 150})
 	q1.allocatedResource = resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 120, "vcore": 120})
 	queues = []*Queue{q0, q1, q2, q3}
-	sortQueue(queues, policies.FairSortPolicy, false)
+	sortQueue(queues, fairMaxResources, policies.FairSortPolicy, false)
 	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q0, q1, q3, q2}), "fair third")
 	queues = []*Queue{q0, q1, q2, q3}
-	sortQueue(queues, policies.FairSortPolicy, true)
+	sortQueue(queues, fairMaxResources, policies.FairSortPolicy, true)
 	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q0, q1, q2}), "fair third - priority")
+
+	// fairness ratios: q0:400/800=0.5, q1:200/400= 0.5, q2:100/200=0.5, q3:100/200=0.5
+	q0.guaranteedResource = resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 800, "vcore": 400})
+	q0.allocatedResource = resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 400, "vcore": 200})
+	q1.guaranteedResource = resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 400, "vcore": 300})
+	q1.allocatedResource = resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 200, "vcore": 150})
+	queues = []*Queue{q0, q1, q2, q3}
+	sortQueue(queues, fairMaxResources, policies.FairSortPolicy, false)
+	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q0, q1, q2}), "fair - pending resource")
 }
 
 // queue guaranteed resource is not set (same as a zero resource)
-func TestNoQueueLimits(t *testing.T) {
+func TestSortQueuesNoGuarantees(t *testing.T) {
 	root, err := createRootQueue(nil)
 	assert.NilError(t, err, "queue create failed")
 
@@ -136,16 +152,24 @@ func TestNoQueueLimits(t *testing.T) {
 	q3.currentPriority = 3
 
 	queues := []*Queue{q0, q1, q2, q3}
-	sortQueue(queues, policies.FairSortPolicy, false)
-	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q2, q1, q0}), "fair no limit first")
-	sortQueue(queues, policies.FairSortPolicy, true)
-	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q2, q0, q1}), "fair no limit first - priority")
+	fairMaxResources := []*resources.Resource{resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 1000, "vcore": 1000}),
+		resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 1000, "vcore": 1000}),
+		resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 1000, "vcore": 1000}),
+		resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 1000, "vcore": 1000}),
+	}
+	sortQueue(queues, fairMaxResources, policies.FairSortPolicy, false)
+	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q2, q1, q0}), "fair no gaurantees first")
+
+	sortQueue(queues, fairMaxResources, policies.FairSortPolicy, true)
+	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q2, q0, q1}), "fair no gaurantees first - priority")
 
 	q0.allocatedResource = resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 200, "vcore": 200})
 	q1.allocatedResource = resources.NewResourceFromMap(map[string]resources.Quantity{"memory": 300, "vcore": 300})
-	sortQueue(queues, policies.FairSortPolicy, false)
-	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q2, q0, q1}), "fair no limit second")
-	sortQueue(queues, policies.FairSortPolicy, true)
+
+	sortQueue(queues, fairMaxResources, policies.FairSortPolicy, false)
+	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q2, q0, q1}), "fair no gaurantees second")
+
+	sortQueue(queues, fairMaxResources, policies.FairSortPolicy, true)
 	assert.Equal(t, queueNames(queues), queueNames([]*Queue{q3, q2, q0, q1}), "fair no limit second - priority")
 }
 
@@ -174,11 +198,6 @@ func TestSortAppsNoPending(t *testing.T) {
 	list = sortApplications(input, policies.FifoSortPolicy, true, nil)
 	assertAppListLength(t, list, []string{}, "fifo no pending - priority")
 
-	list = sortApplications(input, policies.StateAwarePolicy, false, nil)
-	assertAppListLength(t, list, []string{}, "state no pending")
-	list = sortApplications(input, policies.StateAwarePolicy, true, nil)
-	assertAppListLength(t, list, []string{}, "state no pending - priority")
-
 	// set one app with pending
 	appID := "app-1"
 	input[appID].pending = res
@@ -191,11 +210,6 @@ func TestSortAppsNoPending(t *testing.T) {
 	assertAppListLength(t, list, []string{appID}, "fifo one pending")
 	list = sortApplications(input, policies.FifoSortPolicy, true, nil)
 	assertAppListLength(t, list, []string{appID}, "fifo one pending - priority")
-
-	list = sortApplications(input, policies.StateAwarePolicy, false, nil)
-	assertAppListLength(t, list, []string{appID}, "state one pending")
-	list = sortApplications(input, policies.StateAwarePolicy, true, nil)
-	assertAppListLength(t, list, []string{appID}, "state one pending - priority")
 }
 
 func TestSortAppsFifo(t *testing.T) {
@@ -219,6 +233,18 @@ func TestSortAppsFifo(t *testing.T) {
 	// fifo - apps should come back in order created 0, 1, 2, 3
 	list = sortApplications(input, policies.FifoSortPolicy, false, nil)
 	assertAppList(t, list, []int{0, 1, 2, 3}, "fifo simple")
+
+	input["app-1"].askMaxPriority = 3
+	input["app-3"].askMaxPriority = 5
+	input["app-2"].SubmissionTime = input["app-3"].SubmissionTime
+	input["app-1"].SubmissionTime = input["app-3"].SubmissionTime
+	list = sortApplications(input, policies.FifoSortPolicy, false, nil)
+	/*
+	* apps order: 0, 3, 1, 2
+	* the resultType of app index is [0, 2, 3, 1]
+	* app0 with index 0, app1 with index 2, app2 with index 3 and app3 with index 1
+	 */
+	assertAppList(t, list, []int{0, 2, 3, 1}, "fifo first, priority second")
 }
 
 func TestSortAppsPriorityFifo(t *testing.T) {
@@ -286,6 +312,19 @@ func TestSortAppsFair(t *testing.T) {
 	// apps should come back in order: 3, 0, 2, 1
 	list = sortApplications(input, policies.FairSortPolicy, false, resources.Multiply(res, 5))
 	assertAppList(t, list, []int{1, 3, 2, 0}, "app-1 & app-3 allocated")
+
+	// update allocated resource for app-3 & app-1 where priority of app-3 is higher
+	input["app-1"].allocatedResource = resources.Multiply(res, 10)
+	input["app-1"].askMaxPriority = 2
+	input["app-3"].allocatedResource = resources.Multiply(res, 10)
+	input["app-3"].askMaxPriority = 3
+	list = sortApplications(input, policies.FairSortPolicy, false, resources.Multiply(res, 5))
+	/*
+	*  expected apps order: 0, 2, 3, 1 means
+	*  So resultType of apps indexs is [0, 3, 1, 2]
+	*  app0 in 0, app1 in 3, app2 in 1, app3 in 2:
+	 */
+	assertAppList(t, list, []int{0, 3, 1, 2}, "app-1 & app-3 allocated, app-3 high priority")
 }
 
 func TestSortAppsPriorityFair(t *testing.T) {
@@ -332,61 +371,6 @@ func TestSortAppsPriorityFair(t *testing.T) {
 	assertAppList(t, list, []int{2, 0, 3, 1}, "app-1 & app-3 allocated")
 }
 
-func TestSortAppsStateAware(t *testing.T) {
-	// stable sort is used so equal values stay where they were
-	res := resources.NewResourceFromMap(map[string]resources.Quantity{
-		"vcore": resources.Quantity(100)})
-	// setup all apps with pending resources, all accepted state
-	input := make(map[string]*Application, 4)
-	for i := 0; i < 4; i++ {
-		num := strconv.Itoa(i)
-		appID := "app-" + num
-		app := newApplication(appID, "partition", "queue")
-		app.pending = res
-		input[appID] = app
-		err := app.HandleApplicationEvent(RunApplication)
-		assert.NilError(t, err, "state change failed for app %v", appID)
-		// make sure the time stamps differ at least a bit (tracking in nano seconds)
-		time.Sleep(time.Nanosecond * 5)
-	}
-	// only first app should be returned (all in accepted)
-	list := sortApplications(input, policies.StateAwarePolicy, false, nil)
-	appID0 := "app-0"
-	assertAppListLength(t, list, []string{appID0}, "state all accepted")
-
-	// set first app pending to zero, should get 2nd app back
-	input[appID0].pending = resources.NewResource()
-	list = sortApplications(input, policies.StateAwarePolicy, false, nil)
-	appID1 := "app-1"
-	assertAppListLength(t, list, []string{appID1}, "state no pending")
-
-	// move the first app to starting no pending resource should get nothing
-	err := input[appID0].HandleApplicationEvent(RunApplication)
-	assert.NilError(t, err, "state change failed for app-0")
-	list = sortApplications(input, policies.StateAwarePolicy, false, nil)
-	assertAppListLength(t, list, []string{}, "state starting no pending")
-
-	// move first app to running (no pending resource) and 4th app to starting should get starting app
-	err = input[appID0].HandleApplicationEvent(RunApplication)
-	assert.NilError(t, err, "state change failed for app-0")
-	appID3 := "app-3"
-	err = input[appID3].HandleApplicationEvent(RunApplication)
-	assert.NilError(t, err, "state change failed for app-3")
-	list = sortApplications(input, policies.StateAwarePolicy, false, nil)
-	assertAppListLength(t, list, []string{appID3}, "state starting")
-
-	// set pending for first app, should get back 1st and 4th in that order
-	input[appID0].pending = res
-	list = sortApplications(input, policies.StateAwarePolicy, false, nil)
-	assertAppListLength(t, list, []string{appID0, appID3}, "state first pending")
-
-	// move 4th to running should get back: 1st, 2nd and 4th in that order
-	err = input[appID3].HandleApplicationEvent(RunApplication)
-	assert.NilError(t, err, "state change failed for app-3")
-	list = sortApplications(input, policies.StateAwarePolicy, false, nil)
-	assertAppListLength(t, list, []string{appID0, appID1, appID3}, "state not app-2")
-}
-
 func queueNames(list []*Queue) string {
 	result := make([]string, 0)
 	for _, v := range list {
