@@ -19,7 +19,7 @@
 package placement
 
 import (
-	"fmt"
+	"errors"
 	"strings"
 
 	"go.uber.org/zap"
@@ -30,8 +30,12 @@ import (
 	"github.com/apache/yunikorn-core/pkg/log"
 	"github.com/apache/yunikorn-core/pkg/scheduler/objects"
 	"github.com/apache/yunikorn-core/pkg/scheduler/placement/types"
+	"github.com/apache/yunikorn-core/pkg/webservice/dao"
 )
 
+// RejectedError is the standard error returned if placement has failed
+var RejectedError = errors.New("application rejected: no placement rule matched")
+
 type AppPlacementManager struct {
 	rules   []rule
 	queueFn func(string) *objects.Queue
@@ -49,10 +53,19 @@ func NewPlacementManager(rules []configs.PlacementRule, queueFunc func(string) *
 	return m
 }
 
-// Update the rules for an active placement manager
-// Note that this will only be called when the manager is created earlier and the config is updated.
+// GetRulesDAO returns a list of RuleDAO objects of the configured rules
+func (m *AppPlacementManager) GetRulesDAO() []*dao.RuleDAO {
+	m.RLock()
+	defer m.RUnlock()
+	info := make([]*dao.RuleDAO, len(m.rules))
+	for i, r := range m.rules {
+		info[i] = r.ruleDAO()
+	}
+	return info
+}
+
+// UpdateRules sets the rules for an active placement manager
 func (m *AppPlacementManager) UpdateRules(rules []configs.PlacementRule) error {
-	log.Log(log.Config).Info("Building new rule list for placement manager")
 	if err := m.initialise(rules); err != nil {
 		log.Log(log.Config).Info("Placement manager rules not reloaded", zap.Error(err))
 		return err
@@ -60,11 +73,11 @@ func (m *AppPlacementManager) UpdateRules(rules []configs.PlacementRule) error {
 	return nil
 }
 
-// Initialise the rules from a parsed config.
+// initialise the rules from a parsed config.
 func (m *AppPlacementManager) initialise(rules []configs.PlacementRule) error {
 	log.Log(log.Config).Info("Building new rule list for placement manager")
 	// build temp list from new config
-	tempRules, err := m.buildRules(rules)
+	tempRules, err := buildRules(rules)
 	if err != nil {
 		return err
 	}
@@ -82,10 +95,116 @@ func (m *AppPlacementManager) initialise(rules []configs.PlacementRule) error {
 	return nil
 }
 
-// Build the rule set based on the config.
+// PlaceApplication executes the rules for the passed in application.
+// On success the queueName of the application is set to the queue the application wil run in.
+// On failure the queueName is set to "" and an error is returned.
+func (m *AppPlacementManager) PlaceApplication(app *objects.Application) error {
+	m.RLock()
+	defer m.RUnlock()
+
+	var queueName string
+	var err error
+	var remainingRules = len(m.rules)
+	for _, checkRule := range m.rules {
+		remainingRules--
+		log.Log(log.SchedApplication).Debug("Executing rule for placing application",
+			zap.String("ruleName", checkRule.getName()),
+			zap.String("application", app.ApplicationID))
+		queueName, err = checkRule.placeApplication(app, m.queueFn)
+		if err != nil {
+			log.Log(log.SchedApplication).Error("rule execution failed",
+				zap.String("ruleName", checkRule.getName()),
+				zap.Error(err))
+			app.SetQueuePath("")
+			return err
+		}
+		// if no queue found even after the last rule, try to place in the default queue
+		if remainingRules == 0 && queueName == "" {
+			log.Log(log.Config).Info("No rule matched, placing application in default queue",
+				zap.String("application", app.ApplicationID),
+				zap.String("defaultQueue", common.DefaultPlacementQueue))
+			// get the queue object
+			queue := m.queueFn(common.DefaultPlacementQueue)
+			if queue != nil {
+				// default queue exist
+				queueName = common.DefaultPlacementQueue
+			}
+		}
+		// no queue name next rule
+		if queueName == "" {
+			continue
+		}
+		// We have the recovery queue bail out: only if we are doing forced placement
+		// Recovery rule is last in the list. Recovery queue cannot be returned by other rules.
+		// We do not want to trigger any checks for this queue.
+		if queueName == common.RecoveryQueueFull && app.IsCreateForced() {
+			log.Log(log.SchedApplication).Info("Placing application in recovery queue",
+				zap.String("application", app.ApplicationID))
+			break
+		}
+		// queueName returned make sure ACL allows access and set the queueName in the app
+		queue := m.queueFn(queueName)
+		// walk up the tree if the queue does not exist
+		if queue == nil {
+			current := queueName
+			for queue == nil {
+				current = current[0:strings.LastIndex(current, configs.DOT)]
+				// check if the queue exist
+				queue = m.queueFn(current)
+			}
+			// Check if the user is allowed to submit to this queueName, if not next rule
+			if !queue.CheckSubmitAccess(app.GetUser()) {
+				log.Log(log.SchedApplication).Debug("Submit access denied on queue",
+					zap.String("queueName", queue.GetQueuePath()),
+					zap.String("ruleName", checkRule.getName()),
+					zap.String("application", app.ApplicationID))
+				// reset the queue name for the last rule in the chain
+				queueName = ""
+				continue
+			}
+		} else {
+			// Check if this final queue is a leaf queue, if not next rule
+			if !queue.IsLeafQueue() {
+				log.Log(log.SchedApplication).Debug("Rule returned parent queue",
+					zap.String("queueName", queueName),
+					zap.String("ruleName", checkRule.getName()),
+					zap.String("application", app.ApplicationID))
+				// reset the queue name for the last rule in the chain
+				queueName = ""
+				continue
+			}
+			// Check if the user is allowed to submit to this queueName, if not next rule
+			if !queue.CheckSubmitAccess(app.GetUser()) {
+				log.Log(log.SchedApplication).Debug("Submit access denied on queue",
+					zap.String("queueName", queueName),
+					zap.String("ruleName", checkRule.getName()),
+					zap.String("application", app.ApplicationID))
+				// reset the queue name for the last rule in the chain
+				queueName = ""
+				continue
+			}
+		}
+		// we have a queue that allows submitting and can be created: app placed
+		log.Log(log.SchedApplication).Info("Rule result for placing application",
+			zap.String("application", app.ApplicationID),
+			zap.String("ruleName", checkRule.getName()),
+			zap.String("queueName", queueName))
+		break
+	}
+	// no more rules to check no queueName found reject placement
+	if queueName == "" {
+		app.SetQueuePath("")
+		return RejectedError
+	}
+	// Add the queue into the application, overriding what was submitted
+	app.SetQueuePath(queueName)
+	return nil
+}
+
+// buildRules builds a new rule set based on the config.
 // If the rule set is correct and can be used the new set is returned.
 // If any error is encountered a nil array is returned and the error set
-func (m *AppPlacementManager) buildRules(rules []configs.PlacementRule) ([]rule, error) {
+func buildRules(rules []configs.PlacementRule) ([]rule, error) {
 	// empty list should result in a single "provided" rule
 	if len(rules) == 0 {
 		log.Log(log.Config).Info("Placement manager configured without rules: using implicit provided rule")
@@ -108,97 +227,3 @@ func (m *AppPlacementManager) buildRules(rules []configs.PlacementRule) ([]rule,
 
 	return newRules, nil
 }
-
-func (m *AppPlacementManager) PlaceApplication(app *objects.Application) error {
-	m.RLock()
-	defer m.RUnlock()
-
-	var queueName string
-	var aclCheck bool
-	var err error
-	var remainingRules = len(m.rules)
-	for _, checkRule := range m.rules {
-		remainingRules--
-		log.Log(log.Config).Debug("Executing rule for placing application",
-			zap.String("ruleName", checkRule.getName()),
-			zap.String("application", app.ApplicationID))
-		queueName, aclCheck, err = checkRule.placeApplication(app, m.queueFn)
-		if err != nil {
-			log.Log(log.Config).Error("rule execution failed",
-				zap.String("ruleName", checkRule.getName()),
-				zap.Error(err))
-			app.SetQueuePath("")
-			return err
-		}
-		// if no queue found even after the last rule, try to place in the default queue
-		if remainingRules == 0 && queueName == "" {
-			log.Log(log.Config).Info("No rule matched, placing application in default queue",
-				zap.String("application", app.ApplicationID),
-				zap.String("defaultQueue", common.DefaultPlacementQueue))
-			// get the queue object
-			queue := m.queueFn(common.DefaultPlacementQueue)
-			if queue != nil {
-				// default queue exist
-				queueName = common.DefaultPlacementQueue
-			}
-		}
-		// queueName returned make sure ACL allows access and create the queueName if not exist
-		if queueName != "" {
-			// get the queue object
-			queue := m.queueFn(queueName)
-			// walk up the tree if the queue does not exist
-			if queue == nil {
-				current := queueName
-				for queue == nil {
-					current = current[0:strings.LastIndex(current, configs.DOT)]
-					// check if the queue exist
-					queue = m.queueFn(current)
-				}
-				// Check if the user is allowed to submit to this queueName, if not next rule
-				if aclCheck && !queue.CheckSubmitAccess(app.GetUser()) {
-					log.Log(log.Config).Debug("Submit access denied on queue",
-						zap.String("queueName", queue.GetQueuePath()),
-						zap.String("ruleName", checkRule.getName()),
-						zap.String("application", app.ApplicationID))
-					// reset the queue name for the last rule in the chain
-					queueName = ""
-					continue
-				}
-			} else {
-				// Check if this final queue is a leaf queue, if not next rule
-				if !queue.IsLeafQueue() {
-					log.Log(log.Config).Debug("Rule returned parent queue",
-						zap.String("queueName", queueName),
-						zap.String("ruleName", checkRule.getName()),
-						zap.String("application", app.ApplicationID))
-					// reset the queue name for the last rule in the chain
-					queueName = ""
-					continue
-				}
-				// Check if the user is allowed to submit to this queueName, if not next rule
-				if aclCheck && !queue.CheckSubmitAccess(app.GetUser()) {
-					log.Log(log.Config).Debug("Submit access denied on queue",
-						zap.String("queueName", queueName),
-						zap.String("ruleName", checkRule.getName()),
-						zap.String("application", app.ApplicationID))
-					// reset the queue name for the last rule in the chain
-					queueName = ""
-					continue
-				}
-			}
-			// we have a queue that allows submitting and can be created: app placed
-			break
-		}
-	}
-	log.Log(log.Config).Debug("Rule result for placing application",
-		zap.String("application", app.ApplicationID),
-		zap.String("queueName", queueName))
-	// no more rules to check no queueName found reject placement
-	if queueName == "" {
-		app.SetQueuePath("")
-		return fmt.Errorf("application rejected: no placement rule matched")
-	}
-	// Add the queue into the application, overriding what was submitted
-	app.SetQueuePath(queueName)
-	return nil
-}
