@@ -19,6 +19,7 @@
 package placement
 
 import (
+	"errors"
 	"testing"
 
 	"gotest.tools/v3/assert"
@@ -34,21 +35,34 @@ import (
 func TestManagerNew(t *testing.T) {
 	// basic info without rules, manager should not init
 	man := NewPlacementManager(nil, queueFunc)
-	assert.Equal(t, 2, len(man.rules), "wrong rule count for empty placement manager")
+	assert.Equal(t, 2, len(man.rules), "wrong rule count for new placement manager, no config")
 	assert.Equal(t, types.Provided, man.rules[0].getName(), "wrong name for implicit provided rule")
 	assert.Equal(t, types.Recovery, man.rules[1].getName(), "wrong name for implicit recovery rule")
+
+	// fail update rules with unknown rule
+	rules := []configs.PlacementRule{{Name: "unknown", Create: true}}
+	man = NewPlacementManager(rules, queueFunc)
+	assert.Equal(t, 0, len(man.rules), "wrong rule count for new placement manager, no config")
 }
 
 func TestManagerInit(t *testing.T) {
 	// basic info without rules, manager should implicitly init
 	man := NewPlacementManager(nil, queueFunc)
-	assert.Equal(t, 2, len(man.rules), "wrong rule count for nil placement manager")
+	assert.Equal(t, 2, len(man.rules), "wrong rule count for nil rules config")
+	ruleDAOs := man.GetRulesDAO()
+	assert.Equal(t, 2, len(ruleDAOs), "wrong DAO count for nil rules config")
+	assert.Equal(t, ruleDAOs[0].Name, types.Provided, "expected provided rule as first rule")
+	assert.Equal(t, ruleDAOs[1].Name, types.Recovery, "expected recovery rule as second rule")
 
 	// try to init with empty list should do the same
 	var rules []configs.PlacementRule
 	err := man.initialise(rules)
 	assert.NilError(t, err, "Failed to initialize empty placement rules")
-	assert.Equal(t, 2, len(man.rules), "wrong rule count for empty placement manager")
+	assert.Equal(t, 2, len(man.rules), "wrong rule count for empty rules config")
+	ruleDAOs = man.GetRulesDAO()
+	assert.Equal(t, 2, len(ruleDAOs), "wrong DAO count for empty rules config")
+	assert.Equal(t, ruleDAOs[0].Name, types.Provided, "expected provided rule as first rule")
+	assert.Equal(t, ruleDAOs[1].Name, types.Recovery, "expected recovery rule as second rule")
 
 	rules = []configs.PlacementRule{
 		{Name: "unknown"},
@@ -57,6 +71,11 @@ func TestManagerInit(t *testing.T) {
 	if err == nil {
 		t.Error("initialise with 'unknown' rule list should have failed")
 	}
+	// rules should not have changed
+	ruleDAOs = man.GetRulesDAO()
+	assert.Equal(t, 2, len(ruleDAOs), "unexpected change: wrong DAO count for failed reinit")
+	assert.Equal(t, ruleDAOs[0].Name, types.Provided, "expected provided rule as first rule")
+	assert.Equal(t, ruleDAOs[1].Name, types.Recovery, "expected recovery rule as second rule")
 
 	// init the manager with one rule
 	rules = []configs.PlacementRule{
@@ -64,17 +83,47 @@ func TestManagerInit(t *testing.T) {
 	}
 	err = man.initialise(rules)
 	assert.NilError(t, err, "failed to init existing manager")
+	ruleDAOs = man.GetRulesDAO()
+	assert.Equal(t, 2, len(ruleDAOs), "wrong DAO count for manager with test rule")
+	assert.Equal(t, ruleDAOs[0].Name, types.Test, "expected test rule as first rule")
+	assert.Equal(t, ruleDAOs[1].Name, types.Recovery, "expected recovery rule as second rule")
 
 	// update the manager: remove rules implicit state is reverted
 	rules = []configs.PlacementRule{}
 	err = man.initialise(rules)
 	assert.NilError(t, err, "Failed to re-initialize empty placement rules")
-	assert.Equal(t, 2, len(man.rules), "wrong rule count for newly empty placement manager")
+	assert.Equal(t, 2, len(man.rules), "wrong rule count for empty rules config")
+	ruleDAOs = man.GetRulesDAO()
+	assert.Equal(t, 2, len(ruleDAOs), "wrong DAO count for empty rules list")
+	assert.Equal(t, ruleDAOs[0].Name, types.Provided, "expected provided rule as first rule")
+	assert.Equal(t, ruleDAOs[1].Name, types.Recovery, "expected recovery rule as second rule")
 
 	// check if we handle a nil list
 	err = man.initialise(nil)
 	assert.NilError(t, err, "Failed to re-initialize nil placement rules")
-	assert.Equal(t, 2, len(man.rules), "wrong rule count for nil placement manager")
+	assert.Equal(t, 2, len(man.rules), "wrong rule count for nil rules config")
+	ruleDAOs = man.GetRulesDAO()
+	assert.Equal(t, 2, len(ruleDAOs), "wrong DAO count for nil rules config")
+	assert.Equal(t, ruleDAOs[0].Name, types.Provided, "expected provided rule as first rule")
+	assert.Equal(t, ruleDAOs[1].Name, types.Recovery, "expected recovery rule as second rule")
+
+	// init the manager with one rule which has parent
+	rules = []configs.PlacementRule{
+		{
+			Name:   "test",
+			Create: true,
+			Parent: &configs.PlacementRule{
+				Name: "test",
+			},
+		},
+	}
+	err = man.initialise(rules)
+	assert.NilError(t, err, "Failed to re-initialize nil placement rules")
+	assert.Equal(t, 2, len(man.rules), "wrong rule count for nil rules config")
+	ruleDAOs = man.GetRulesDAO()
+	assert.Equal(t, 2, len(ruleDAOs), "wrong DAO count for nil rules config")
+	assert.Equal(t, ruleDAOs[0].Name, types.Test, "expected test rule as first rule")
+	assert.Equal(t, ruleDAOs[1].Name, types.Recovery, "expected recovery rule as second rule")
 }
 
 func TestManagerUpdate(t *testing.T) {
@@ -86,26 +135,44 @@ func TestManagerUpdate(t *testing.T) {
 	}
 	err := man.UpdateRules(rules)
 	assert.NilError(t, err, "failed to update existing manager")
+	ruleDAOs := man.GetRulesDAO()
+	assert.Equal(t, 2, len(ruleDAOs), "wrong DAO count for manager with test rule")
+	assert.Equal(t, ruleDAOs[0].Name, types.Test, "expected test rule as first rule")
+	assert.Equal(t, ruleDAOs[1].Name, types.Recovery, "expected recovery rule as second rule")
 
 	// update the manager: remove rules init state is reverted
 	rules = []configs.PlacementRule{}
 	err = man.UpdateRules(rules)
 	assert.NilError(t, err, "Failed to re-initialize empty placement rules")
-	assert.Equal(t, 2, len(man.rules), "wrong rule count for newly empty placement manager")
+	assert.Equal(t, 2, len(man.rules), "wrong rule count for empty rules config")
+	ruleDAOs = man.GetRulesDAO()
+	assert.Equal(t, 2, len(ruleDAOs), "wrong DAO count for empty rules config")
+	assert.Equal(t, ruleDAOs[0].Name, types.Provided, "expected provided rule as first rule")
+	assert.Equal(t, ruleDAOs[1].Name, types.Recovery, "expected recovery rule as second rule")
 
 	// check if we handle a nil list
 	err = man.UpdateRules(nil)
 	assert.NilError(t, err, "Failed to re-initialize nil placement rules")
-	assert.Equal(t, 2, len(man.rules), "wrong rule count for nil placement manager")
+	assert.Equal(t, 2, len(man.rules), "wrong rule count for nil rules config")
+	ruleDAOs = man.GetRulesDAO()
+	assert.Equal(t, 2, len(ruleDAOs), "wrong DAO count for nil rules config")
+	assert.Equal(t, ruleDAOs[0].Name, types.Provided, "expected provided rule as first rule")
+	assert.Equal(t, ruleDAOs[1].Name, types.Recovery, "expected recovery rule as second rule")
+
+	// fail to update rules with unknown rule
+	rules = []configs.PlacementRule{{Name: "unknown", Create: true}}
+	err = man.UpdateRules(rules)
+	if err == nil {
+		t.Errorf("unknown rule should fail to update the rule.")
+	}
 }
 
 func TestManagerBuildRule(t *testing.T) {
 	// basic with 1 rule
-	man := NewPlacementManager(nil, queueFunc)
 	rules := []configs.PlacementRule{
 		{Name: "test"},
 	}
-	ruleObjs, err := man.buildRules(rules)
+	ruleObjs, err := buildRules(rules)
 	if err != nil {
 		t.Errorf("test rule build should not have failed, err: %v", err)
 	}
@@ -121,12 +188,12 @@ func TestManagerBuildRule(t *testing.T) {
 			},
 		},
 	}
-	ruleObjs, err = man.buildRules(rules)
+	ruleObjs, err = buildRules(rules)
 	if err != nil || len(ruleObjs) != 2 {
 		t.Errorf("test rule build should not have failed and created 2 top level rule, err: %v, rules: %v", err, ruleObjs)
 	} else {
 		parent := ruleObjs[0].getParent()
-		if parent == nil || parent.getName() != "test" {
+		if parent == nil || parent.getName() != rules[0].Name {
 			t.Error("test rule build should have created 2 rules: parent not found")
 		}
 	}
@@ -136,10 +203,10 @@ func TestManagerBuildRule(t *testing.T) {
 		{Name: "user"},
 		{Name: "test"},
 	}
-	ruleObjs, err = man.buildRules(rules)
+	ruleObjs, err = buildRules(rules)
 	if err != nil || len(ruleObjs) != 3 {
 		t.Errorf("rule build should not have failed and created 3 rules, err: %v, rules: %v", err, ruleObjs)
-	} else if ruleObjs[0].getName() != "user" || ruleObjs[1].getName() != "test" || ruleObjs[2].getName() != "recovery" {
+	} else if ruleObjs[0].getName() != rules[0].Name || ruleObjs[1].getName() != rules[1].Name || ruleObjs[2].getName() != "recovery" {
 		t.Errorf("rule build order is not preserved: %v", ruleObjs)
 	}
 }
@@ -193,9 +260,8 @@ partitions:
 	// user rule existing queue, acl allowed
 	err = man.PlaceApplication(app)
 	queueName := app.GetQueuePath()
-	if err != nil || queueName != "root.testparent.testchild" {
-		t.Errorf("leaf exist: app should have been placed in user queue, queue: '%s', error: %v", queueName, err)
-	}
+	assert.NilError(t, err)
+	assert.Equal(t, "root.testparent.testchild", queueName)
 	user = security.UserGroup{
 		User:   "other-user",
 		Groups: []string{},
@@ -259,6 +325,7 @@ partitions:
 	}
 }
 
+//nolint:funlen
 func TestForcePlaceApp(t *testing.T) {
 	const (
 		provided       = "provided"
@@ -337,7 +404,7 @@ partitions:
 			app := newApplication("app1", "default", tt.queue, tt.user, tt.tags, nil, "")
 			err = man.PlaceApplication(app)
 			if tt.placed == "" {
-				assert.ErrorContains(t, err, "rejected", "unexpected error or no error returned")
+				assert.Assert(t, errors.Is(err, RejectedError), "unexpected error or no error returned")
 			} else {
 				assert.NilError(t, err, "unexpected placement failure")
 				assert.Equal(t, tt.placed, app.GetQueuePath(), "incorrect queue set")
@@ -395,7 +462,7 @@ partitions:
 			app := newApplication("app1", "default", tt.queue, tt.user, tt.tags, nil, "")
 			err = man.PlaceApplication(app)
 			if tt.placed == "" {
-				assert.ErrorContains(t, err, "rejected", "unexpected error or no error returned")
+				assert.Assert(t, errors.Is(err, RejectedError), "unexpected error or no error returned")
 			} else {
 				assert.NilError(t, err, "unexpected placement failure")
 				assert.Equal(t, tt.placed, app.GetQueuePath(), "incorrect queue set")
@@ -464,7 +531,7 @@ partitions:
 			app := newApplication("app1", "default", tt.queue, tt.user, tt.tags, nil, "")
 			err = man1.PlaceApplication(app)
 			if tt.placed == "" {
-				assert.ErrorContains(t, err, "rejected", "unexpected error or no error returned")
+				assert.Assert(t, errors.Is(err, RejectedError), "unexpected error or no error returned")
 			} else {
 				assert.NilError(t, err, "unexpected placement failure")
 				assert.Equal(t, tt.placed, app.GetQueuePath(), "incorrect queue set")
