@@ -47,10 +47,7 @@ func TestGTIncreaseTrackedResource(t *testing.T) {
 	}
 
 	manager.Headroom(queuePath1, TestApp1, *user)
-	result := groupTracker.increaseTrackedResource(path1, TestApp1, usage1, user.User)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", path1, TestApp1, usage1)
-	}
+	groupTracker.increaseTrackedResource(path1, TestApp1, usage1, user.User)
 	assert.Equal(t, 1, len(eventSystem.Events))
 	assert.Equal(t, si.EventRecord_UG_GROUP_RESOURCE, eventSystem.Events[0].EventChangeDetail)
 	assert.Equal(t, si.EventRecord_ADD, eventSystem.Events[0].EventChangeType)
@@ -59,28 +56,19 @@ func TestGTIncreaseTrackedResource(t *testing.T) {
 	if err != nil {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage2)
 	}
-	result = groupTracker.increaseTrackedResource(path2, TestApp2, usage2, user.User)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", path2, TestApp2, usage2)
-	}
+	groupTracker.increaseTrackedResource(path2, TestApp2, usage2, user.User)
 
 	usage3, err := resources.NewResourceFromConf(map[string]string{"mem": "30M", "vcore": "30"})
 	if err != nil {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage3)
 	}
-	result = groupTracker.increaseTrackedResource(path3, TestApp3, usage3, user.User)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", path3, TestApp3, usage3)
-	}
+	groupTracker.increaseTrackedResource(path3, TestApp3, usage3, user.User)
 
 	usage4, err := resources.NewResourceFromConf(map[string]string{"mem": "20M", "vcore": "20"})
 	if err != nil {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage3)
 	}
-	result = groupTracker.increaseTrackedResource(path4, TestApp4, usage4, user.User)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", path4, TestApp4, usage4)
-	}
+	groupTracker.increaseTrackedResource(path4, TestApp4, usage4, user.User)
 	actualResources := getGroupResource(groupTracker)
 
 	assert.Equal(t, "map[mem:80000000 vcore:80000]", actualResources["root"].String(), "wrong resource")
@@ -90,6 +78,9 @@ func TestGTIncreaseTrackedResource(t *testing.T) {
 	assert.Equal(t, "map[mem:20000000 vcore:20000]", actualResources["root.parent.child2"].String(), "wrong resource")
 	assert.Equal(t, "map[mem:20000000 vcore:20000]", actualResources["root.parent.child12"].String(), "wrong resource")
 	assert.Equal(t, 4, len(groupTracker.getTrackedApplications()))
+
+	groupTracker = nil
+	groupTracker.increaseTrackedResource(path1, TestApp1, usage1, user.User) // no panic
 }
 
 func TestGTDecreaseTrackedResource(t *testing.T) {
@@ -105,20 +96,14 @@ func TestGTDecreaseTrackedResource(t *testing.T) {
 	if err != nil {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage1)
 	}
-	result := groupTracker.increaseTrackedResource(path1, TestApp1, usage1, user.User)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", path1, TestApp1, usage1)
-	}
+	groupTracker.increaseTrackedResource(path1, TestApp1, usage1, user.User)
 	assert.Equal(t, 1, len(groupTracker.getTrackedApplications()))
 
 	usage2, err := resources.NewResourceFromConf(map[string]string{"mem": "20M", "vcore": "20"})
 	if err != nil {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage2)
 	}
-	result = groupTracker.increaseTrackedResource(path2, TestApp2, usage2, user.User)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", path2, TestApp2, usage2)
-	}
+	groupTracker.increaseTrackedResource(path2, TestApp2, usage2, user.User)
 	actualResources := getGroupResource(groupTracker)
 
 	assert.Equal(t, 2, len(groupTracker.getTrackedApplications()))
@@ -132,19 +117,13 @@ func TestGTDecreaseTrackedResource(t *testing.T) {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage3)
 	}
 	eventSystem.Reset()
-	removeQT, decreased := groupTracker.decreaseTrackedResource(path1, TestApp1, usage3, false)
-	if !decreased {
-		t.Fatalf("unable to decrease tracked resource: queuepath %s, app %s, res %v, error %t", path1, TestApp1, usage3, err)
-	}
+	removeQT := groupTracker.decreaseTrackedResource(path1, TestApp1, usage3, false)
 	assert.Equal(t, removeQT, false, "wrong remove queue tracker value")
 	assert.Equal(t, 1, len(eventSystem.Events))
 	assert.Equal(t, si.EventRecord_UG_GROUP_RESOURCE, eventSystem.Events[0].EventChangeDetail)
 	assert.Equal(t, si.EventRecord_REMOVE, eventSystem.Events[0].EventChangeType)
 
-	removeQT, decreased = groupTracker.decreaseTrackedResource(path2, TestApp2, usage3, false)
-	if !decreased {
-		t.Fatalf("unable to decrease tracked resource: queuepath %s, app %s, res %v, error %t", path2, TestApp2, usage3, err)
-	}
+	removeQT = groupTracker.decreaseTrackedResource(path2, TestApp2, usage3, false)
 	assert.Equal(t, removeQT, false, "wrong remove queue tracker value")
 
 	actualResources1 := getGroupResource(groupTracker)
@@ -159,10 +138,7 @@ func TestGTDecreaseTrackedResource(t *testing.T) {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage3)
 	}
 
-	removeQT, decreased = groupTracker.decreaseTrackedResource(path1, TestApp1, usage4, true)
-	if !decreased {
-		t.Fatalf("unable to decrease tracked resource: queuepath %s, app %s, res %v, error %t", path1, TestApp1, usage1, err)
-	}
+	removeQT = groupTracker.decreaseTrackedResource(path1, TestApp1, usage4, true)
 	assert.Equal(t, 1, len(groupTracker.getTrackedApplications()))
 	assert.Equal(t, removeQT, false, "wrong remove queue tracker value")
 
@@ -171,12 +147,35 @@ func TestGTDecreaseTrackedResource(t *testing.T) {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage3)
 	}
 
-	removeQT, decreased = groupTracker.decreaseTrackedResource(path2, TestApp2, usage5, true)
-	if !decreased {
-		t.Fatalf("unable to decrease tracked resource: queuepath %s, app %s, res %v, error %t", path2, TestApp2, usage2, err)
-	}
+	removeQT = groupTracker.decreaseTrackedResource(path2, TestApp2, usage5, true)
 	assert.Equal(t, 0, len(groupTracker.getTrackedApplications()))
 	assert.Equal(t, removeQT, true, "wrong remove queue tracker value")
+
+	groupTracker = nil
+	removeQT = groupTracker.decreaseTrackedResource(path1, TestApp1, usage1, true) // no panic
+	assert.Assert(t, !removeQT)
+}
+
+func TestDecreaseAllTrackedResourceUsage(t *testing.T) {
+	GetUserManager()
+	testUser := &security.UserGroup{User: "test", Groups: []string{"test"}}
+	eventSystem := mock.NewEventSystem()
+	groupTracker := newGroupTracker(testUser.User, newUGMEvents(eventSystem))
+
+	usage1, err := resources.NewResourceFromConf(map[string]string{"mem": "10M", "vcore": "10"})
+	assert.NilError(t, err)
+	groupTracker.increaseTrackedResource(path1, TestApp1, usage1, testUser.User)
+	groupTracker.increaseTrackedResource(path1, TestApp2, usage1, testUser.User)
+	groupTracker.queueTracker.increaseTrackedResource(strings.Split(path1, configs.DOT), TestApp3, user, usage1)
+
+	apps := groupTracker.decreaseAllTrackedResourceUsage(strings.Split(path1, configs.DOT))
+	assert.Equal(t, 2, len(apps))
+	assert.Equal(t, testUser.User, apps[TestApp1])
+	assert.Equal(t, testUser.User, apps[TestApp2])
+
+	groupTracker = nil
+	apps = groupTracker.decreaseAllTrackedResourceUsage(strings.Split(path1, configs.DOT)) // no panic
+	assert.Assert(t, apps == nil)
 }
 
 func TestGTSetAndClearMaxLimits(t *testing.T) {
@@ -192,10 +191,7 @@ func TestGTSetAndClearMaxLimits(t *testing.T) {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage1)
 	}
 
-	result := groupTracker.increaseTrackedResource(path1, TestApp1, usage1, user.User)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", path1, TestApp1, usage1)
-	}
+	groupTracker.increaseTrackedResource(path1, TestApp1, usage1, user.User)
 
 	// higher limits - apps can run
 	eventSystem.Reset()
@@ -208,14 +204,8 @@ func TestGTSetAndClearMaxLimits(t *testing.T) {
 	assert.Equal(t, si.EventRecord_UG_GROUP_LIMIT, eventSystem.Events[1].EventChangeDetail)
 	assert.Equal(t, si.EventRecord_SET, eventSystem.Events[1].EventChangeType)
 	assert.Equal(t, path5, eventSystem.Events[1].ReferenceID)
-	result = groupTracker.increaseTrackedResource(path1, TestApp2, usage1, user.User)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", path1, TestApp2, usage1)
-	}
-	result = groupTracker.increaseTrackedResource(path1, TestApp3, usage1, user.User)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", path1, TestApp3, usage1)
-	}
+	groupTracker.increaseTrackedResource(path1, TestApp2, usage1, user.User)
+	groupTracker.increaseTrackedResource(path1, TestApp3, usage1, user.User)
 	path1expectedHeadroom := resources.NewResourceFromMap(map[string]resources.Quantity{
 		"mem":   20000000,
 		"vcore": 20000,
@@ -265,6 +255,34 @@ func TestGTSetAndClearMaxLimits(t *testing.T) {
 	assert.Equal(t, si.EventRecord_UG_GROUP_LIMIT, eventSystem.Events[1].EventChangeDetail)
 }
 
+func TestGTCanRunApp(t *testing.T) {
+	manager := GetUserManager()
+	defer manager.ClearConfigLimits()
+	user := security.UserGroup{User: "test", Groups: []string{"test"}}
+	groupTracker := newGroupTracker(user.User, newUGMEvents(mock.NewEventSystemDisabled()))
+	manager.userWildCardLimitsConfig["root.parent"] = &LimitConfig{
+		maxResources: resources.NewResourceFromMap(map[string]resources.Quantity{
+			"cpu": 1000,
+		}),
+		maxApplications: 3,
+	} // manipulate map directly to avoid hierarchy creation
+
+	hierarchy1 := strings.Split(path1, configs.DOT)
+	assert.Assert(t, groupTracker.canRunApp(hierarchy1, TestApp1))
+	// make sure wildcard limits are not applied due to the tracker type
+	assert.Equal(t, uint64(0), groupTracker.queueTracker.childQueueTrackers["parent"].maxRunningApps)
+	assert.Assert(t, groupTracker.queueTracker.childQueueTrackers["parent"].maxResources == nil)
+	assert.Assert(t, !groupTracker.queueTracker.childQueueTrackers["parent"].useWildCard)
+
+	// maxApps limit hit
+	groupTracker.setLimits(path1, nil, 1)
+	groupTracker.increaseTrackedResource(path1, TestApp1, resources.NewResourceFromMap(map[string]resources.Quantity{
+		"cpu": 1000,
+	}), user.User)
+	assert.Assert(t, groupTracker.canRunApp(hierarchy1, TestApp1))
+	assert.Assert(t, !groupTracker.canRunApp(hierarchy1, TestApp2))
+}
+
 func getGroupResource(gt *GroupTracker) map[string]*resources.Resource {
 	resources := make(map[string]*resources.Resource)
 	usage := gt.GetGroupResourceUsageDAOInfo()
