@@ -26,6 +26,7 @@ import (
 
 	"github.com/apache/yunikorn-core/pkg/common/configs"
 	"github.com/apache/yunikorn-core/pkg/common/resources"
+	"github.com/apache/yunikorn-core/pkg/webservice/dao"
 )
 
 func TestQTIncreaseTrackedResource(t *testing.T) {
@@ -40,37 +41,25 @@ func TestQTIncreaseTrackedResource(t *testing.T) {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage1)
 	}
 
-	result := queueTracker.increaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, user, usage1)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", queuePath1, TestApp1, usage1)
-	}
+	queueTracker.increaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, user, usage1)
 
 	usage2, err := resources.NewResourceFromConf(map[string]string{"mem": "20M", "vcore": "20"})
 	if err != nil {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage2)
 	}
-	result = queueTracker.increaseTrackedResource(strings.Split(queuePath2, configs.DOT), TestApp2, user, usage2)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", queuePath2, TestApp2, usage2)
-	}
+	queueTracker.increaseTrackedResource(strings.Split(queuePath2, configs.DOT), TestApp2, user, usage2)
 
 	usage3, err := resources.NewResourceFromConf(map[string]string{"mem": "30M", "vcore": "30"})
 	if err != nil {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage3)
 	}
-	result = queueTracker.increaseTrackedResource(strings.Split(queuePath3, configs.DOT), TestApp3, user, usage3)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", queuePath3, TestApp3, usage3)
-	}
+	queueTracker.increaseTrackedResource(strings.Split(queuePath3, configs.DOT), TestApp3, user, usage3)
 
 	usage4, err := resources.NewResourceFromConf(map[string]string{"mem": "20M", "vcore": "20"})
 	if err != nil {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage3)
 	}
-	result = queueTracker.increaseTrackedResource(strings.Split(queuePath4, configs.DOT), TestApp4, user, usage4)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", queuePath4, TestApp4, usage4)
-	}
+	queueTracker.increaseTrackedResource(strings.Split(queuePath4, configs.DOT), TestApp4, user, usage4)
 	actualResources := getQTResource(queueTracker)
 
 	assert.Equal(t, "map[mem:80000000 vcore:80000]", actualResources["root"].String(), "wrong resource")
@@ -93,20 +82,14 @@ func TestQTDecreaseTrackedResource(t *testing.T) {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage1)
 	}
 
-	result := queueTracker.increaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, user, usage1)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", queuePath1, TestApp1, usage1)
-	}
+	queueTracker.increaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, user, usage1)
 	assert.Equal(t, 1, len(queueTracker.runningApplications))
 
 	usage2, err := resources.NewResourceFromConf(map[string]string{"mem": "20M", "vcore": "20"})
 	if err != nil {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage2)
 	}
-	result = queueTracker.increaseTrackedResource(strings.Split(queuePath2, configs.DOT), TestApp2, user, usage2)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", queuePath2, TestApp2, usage2)
-	}
+	queueTracker.increaseTrackedResource(strings.Split(queuePath2, configs.DOT), TestApp2, user, usage2)
 	actualResources := getQTResource(queueTracker)
 
 	assert.Equal(t, 2, len(queueTracker.runningApplications))
@@ -120,16 +103,10 @@ func TestQTDecreaseTrackedResource(t *testing.T) {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage3)
 	}
 
-	removeQT, decreased := queueTracker.decreaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, usage3, false)
-	if !decreased {
-		t.Fatalf("unable to decrease tracked resource: queuepath %s, app %s, res %v, error %t", queuePath1, TestApp1, usage3, err)
-	}
+	removeQT := queueTracker.decreaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, usage3, false)
 	assert.Equal(t, removeQT, false, "wrong remove queue tracker value")
 
-	removeQT, decreased = queueTracker.decreaseTrackedResource(strings.Split(queuePath2, configs.DOT), TestApp2, usage3, false)
-	if !decreased {
-		t.Fatalf("unable to decrease tracked resource: queuepath %s, app %s, res %v, error %t", queuePath2, TestApp2, usage3, err)
-	}
+	removeQT = queueTracker.decreaseTrackedResource(strings.Split(queuePath2, configs.DOT), TestApp2, usage3, false)
 	actualResources1 := getQTResource(queueTracker)
 
 	assert.Equal(t, removeQT, false, "wrong remove queue tracker value")
@@ -143,10 +120,7 @@ func TestQTDecreaseTrackedResource(t *testing.T) {
 	if err != nil {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage3)
 	}
-	removeQT, decreased = queueTracker.decreaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, usage4, true)
-	if !decreased {
-		t.Fatalf("unable to decrease tracked resource: queuepath %s, app %s, res %v, error %t", queuePath1, TestApp1, usage1, err)
-	}
+	removeQT = queueTracker.decreaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, usage4, true)
 	assert.Equal(t, 1, len(queueTracker.runningApplications))
 	assert.Equal(t, removeQT, false, "wrong remove queue tracker value")
 	// Make sure childQueueTracker cleaned
@@ -156,30 +130,25 @@ func TestQTDecreaseTrackedResource(t *testing.T) {
 	if err != nil {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage5)
 	}
-	removeQT, decreased = queueTracker.decreaseTrackedResource(strings.Split(queuePath2, configs.DOT), TestApp2, usage5, true)
-	if !decreased {
-		t.Fatalf("unable to decrease tracked resource: queuepath %s, app %s, res %v, error %t", queuePath2, TestApp2, usage2, err)
-	}
+	removeQT = queueTracker.decreaseTrackedResource(strings.Split(queuePath2, configs.DOT), TestApp2, usage5, true)
 	assert.Equal(t, 0, len(queueTracker.runningApplications))
 	// Make sure all childQueueTracker cleaned
 	assert.Equal(t, len(queueTracker.childQueueTrackers), 0)
 	assert.Equal(t, removeQT, true, "wrong remove queue tracker value")
 
 	// Test parent queueTracker has not zero usage, but child queueTrackers has all deleted
-	result = queueTracker.increaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, user, usage1)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", queuePath1, TestApp1, usage1)
-	}
+	queueTracker.increaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, user, usage1)
 	assert.Equal(t, 1, len(queueTracker.runningApplications))
 
 	usage2, err = resources.NewResourceFromConf(map[string]string{"mem": "20M", "vcore": "20"})
 	if err != nil {
 		t.Errorf("new resource create returned error or wrong resource: error %t, res %v", err, usage2)
 	}
-	result = queueTracker.increaseTrackedResource([]string{"root", "parent"}, TestApp2, user, usage2)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", "root.parent", TestApp2, usage2)
-	}
+	queueTracker.increaseTrackedResource([]string{"root", "parent"}, TestApp2, user, usage2)
+
+	// no child queue
+	removeQT = queueTracker.decreaseTrackedResource([]string{"root", "nonexisting"}, TestApp2, usage3, false)
+	assert.Assert(t, !removeQT)
 }
 
 func TestQTQuotaEnforcement(t *testing.T) {
@@ -216,56 +185,32 @@ func TestQTQuotaEnforcement(t *testing.T) {
 	child2QueueTracker.maxRunningApps = 2
 	parentQueueTracker.childQueueTrackers["child2"] = child2QueueTracker
 
-	result := queueTracker.increaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, user, usage1)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", queuePath1, TestApp1, usage1)
-	}
-
-	result = queueTracker.increaseTrackedResource(strings.Split(queuePath2, configs.DOT), TestApp2, user, usage1)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", queuePath2, TestApp2, usage1)
-	}
-
-	result = queueTracker.increaseTrackedResource(strings.Split(queuePath2, configs.DOT), TestApp2, user, usage1)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", queuePath2, TestApp2, usage1)
-	}
-
+	queueTracker.increaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, user, usage1)
+	queueTracker.increaseTrackedResource(strings.Split(queuePath2, configs.DOT), TestApp2, user, usage1)
+	queueTracker.increaseTrackedResource(strings.Split(queuePath2, configs.DOT), TestApp2, user, usage1)
 	headroom := queueTracker.headroom(strings.Split(queuePath2, configs.DOT), user)
 	assert.Equal(t, headroom.FitInMaxUndef(usage1), false)
-
-	result = queueTracker.increaseTrackedResource(strings.Split(queuePath3, configs.DOT), TestApp3, user, usage1)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", queuePath3, TestApp3, usage1)
-	}
-
-	result = queueTracker.increaseTrackedResource(strings.Split(queuePath4, configs.DOT), TestApp4, user, usage1)
-	if !result {
-		t.Fatalf("unable to increase tracked resource: queuepath %s, app %s, res %v", queuePath4, TestApp4, usage1)
-	}
-
+	queueTracker.increaseTrackedResource(strings.Split(queuePath3, configs.DOT), TestApp3, user, usage1)
+	queueTracker.increaseTrackedResource(strings.Split(queuePath4, configs.DOT), TestApp4, user, usage1)
 	headroom = queueTracker.headroom(strings.Split(queuePath4, configs.DOT), user)
 	assert.Equal(t, headroom.FitInMaxUndef(usage1), false)
 }
 
 func TestHeadroom(t *testing.T) {
+	GetUserManager()
 	var nilResource *resources.Resource
 	path := "root.parent.leaf"
 	hierarchy := strings.Split(path, configs.DOT)
 
-	// nothing exists make sure the hierarchy gets created
+	// validate that the hierarchy gets created
 	root := newRootQueueTracker(user)
-	root.childQueueTrackers["parent"] = newQueueTracker("root", "parent", user)
+	headroom := root.headroom(hierarchy, user)
+	assert.Equal(t, headroom, nilResource, "auto create: expected nil resource")
 	parent := root.childQueueTrackers["parent"]
 	assert.Assert(t, parent != nil, "parent queue tracker should have been created")
-	parent.childQueueTrackers["leaf"] = newQueueTracker("root.parent", "leaf", user)
 	leaf := parent.childQueueTrackers["leaf"]
 	assert.Assert(t, leaf != nil, "leaf queue tracker should have been created")
 
-	// auto created trackers no max resource set
-	headroom := root.headroom(hierarchy, none)
-	assert.Equal(t, headroom, nilResource, "auto create: expected nil resource")
-
 	// prep resources to set as usage and max
 	usage, err := resources.NewResourceFromConf(map[string]string{"mem": "10M", "vcore": "10"})
 	assert.NilError(t, err, "usage: new resource create returned error")
@@ -313,6 +258,38 @@ func TestHeadroom(t *testing.T) {
 	assert.Assert(t, resources.Equals(headroom, combined), "headroom should be same as combined")
 }
 
+func TestQTCanRunApp(t *testing.T) {
+	GetUserManager()
+
+	// validate that the hierarchy gets created
+	root := newRootQueueTracker(user)
+	hierarchy := strings.Split("root.parent.leaf", configs.DOT)
+	assert.Assert(t, root.canRunApp(hierarchy, TestApp1, user))
+	parent := root.childQueueTrackers["parent"]
+	assert.Assert(t, parent != nil, "parent queue tracker should have been created")
+	leaf := parent.childQueueTrackers["leaf"]
+	assert.Assert(t, leaf != nil, "leaf queue tracker should have been created")
+
+	// limit in the leaf queue
+	leaf.maxRunningApps = 2
+	leaf.runningApplications[TestApp1] = true
+	leaf.runningApplications[TestApp2] = true
+	parent.runningApplications[TestApp1] = true
+	parent.runningApplications[TestApp2] = true
+	root.runningApplications[TestApp1] = true
+	root.runningApplications[TestApp2] = true
+	assert.Assert(t, root.canRunApp(hierarchy, TestApp1, user))
+	assert.Assert(t, root.canRunApp(hierarchy, TestApp2, user))
+	assert.Assert(t, !root.canRunApp(hierarchy, TestApp3, user))
+
+	// limit in the parent queue
+	leaf.maxRunningApps = 0
+	parent.maxRunningApps = 2
+	assert.Assert(t, root.canRunApp(hierarchy, TestApp1, user))
+	assert.Assert(t, root.canRunApp(hierarchy, TestApp2, user))
+	assert.Assert(t, !root.canRunApp(hierarchy, TestApp3, user))
+}
+
 func TestNewQueueTracker(t *testing.T) {
 	manager := GetUserManager()
 	defer manager.ClearConfigLimits()
@@ -346,6 +323,74 @@ func TestNewQueueTracker(t *testing.T) {
 	assert.Assert(t, resources.IsZero(parent.resourceUsage))
 }
 
+func TestCanBeRemoved(t *testing.T) {
+	GetUserManager()
+	root := newRootQueueTracker(user)
+	usage1, err := resources.NewResourceFromConf(map[string]string{"mem": "10M", "vcore": "10"})
+	assert.NilError(t, err)
+
+	// create tracker hierarchy
+	root.increaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, user, usage1)
+	parentQ := root.childQueueTrackers["parent"]
+	childQ := parentQ.childQueueTrackers["child1"]
+	assert.Assert(t, !root.canBeRemoved())
+	assert.Assert(t, !parentQ.canBeRemoved())
+	assert.Assert(t, !childQ.canBeRemoved())
+
+	removeQT := root.decreaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, usage1, true)
+	assert.Assert(t, removeQT)
+	assert.Assert(t, root.canBeRemoved())
+}
+
+func TestGetResourceUsageDAOInfo(t *testing.T) {
+	GetUserManager()
+	root := newRootQueueTracker(user)
+	usage1, err := resources.NewResourceFromConf(map[string]string{"mem": "10M", "vcore": "10"})
+	assert.NilError(t, err)
+
+	// create tracker hierarchy
+	root.increaseTrackedResource(strings.Split(queuePath1, configs.DOT), TestApp1, user, usage1)
+
+	// update settings on "parent" and "child1" directly
+	parentQ := root.childQueueTrackers["parent"]
+	childQ := parentQ.childQueueTrackers["child1"]
+	childQ.maxRunningApps = 2
+	maxRes := resources.NewResourceFromMap(map[string]resources.Quantity{
+		"first": 123,
+	})
+	childQ.maxResources = maxRes.Clone()
+	parentQ.maxRunningApps = 3
+
+	rootDao := root.getResourceUsageDAOInfo("")
+	assert.Assert(t, resources.Equals(usage1, rootDao.ResourceUsage))
+	assert.Equal(t, "root", rootDao.QueuePath)
+	assert.Equal(t, 1, len(rootDao.RunningApplications))
+	assert.Equal(t, TestApp1, rootDao.RunningApplications[0])
+	assert.Equal(t, 1, len(rootDao.Children))
+	assert.Equal(t, uint64(0), rootDao.MaxApplications)
+	assert.Assert(t, rootDao.MaxResources == nil)
+	parentDao := rootDao.Children[0]
+	assert.Assert(t, resources.Equals(usage1, parentDao.ResourceUsage))
+	assert.Equal(t, "root.parent", parentDao.QueuePath)
+	assert.Equal(t, 1, len(parentDao.RunningApplications))
+	assert.Equal(t, TestApp1, parentDao.RunningApplications[0])
+	assert.Equal(t, uint64(3), parentDao.MaxApplications)
+	assert.Assert(t, parentDao.MaxResources == nil)
+	assert.Equal(t, 1, len(parentDao.Children))
+	childDao := parentDao.Children[0]
+	assert.Assert(t, resources.Equals(usage1, childDao.ResourceUsage))
+	assert.Equal(t, "root.parent.child1", childDao.QueuePath)
+	assert.Equal(t, 1, len(childDao.RunningApplications))
+	assert.Equal(t, TestApp1, childDao.RunningApplications[0])
+	assert.Equal(t, uint64(2), childDao.MaxApplications)
+	assert.Assert(t, resources.Equals(maxRes, childDao.MaxResources))
+	assert.Equal(t, 0, len(childDao.Children))
+
+	root = nil
+	rootDao = root.getResourceUsageDAOInfo("")
+	assert.DeepEqual(t, rootDao, &dao.ResourceUsageDAOInfo{})
+}
+
 func TestSetLimit(t *testing.T) {
 	manager := GetUserManager()
 	defer manager.ClearConfigLimits()
