@@ -24,15 +24,13 @@ type AllocationDAOInfo struct {
 	RequestTime      int64             `json:"requestTime,omitempty"`     // Allocation ask's createTime if PlaceholderUsed is false, otherwise equivalent to placeholder allocation's createTime
 	AllocationTime   int64             `json:"allocationTime,omitempty"`  // Allocation's createTime
 	AllocationDelay  int64             `json:"allocationDelay,omitempty"` // Difference between AllocationTime and RequestTime
-	UUID             string            `json:"uuid,omitempty"`            // Deprecated. Need to remove this in next major release
-	AllocationID     string            `json:"allocationID,omitempty"`    // Added to replace UUID.
 	ResourcePerAlloc map[string]int64  `json:"resource,omitempty"`
 	Priority         string            `json:"priority,omitempty"`
 	NodeID           string            `json:"nodeId,omitempty"`
 	ApplicationID    string            `json:"applicationId,omitempty"`
-	Partition        string            `json:"partition,omitempty"`
 	Placeholder      bool              `json:"placeholder,omitempty"`
 	PlaceholderUsed  bool              `json:"placeholderUsed,omitempty"`
 	TaskGroupName    string            `json:"taskGroupName,omitempty"`
 	Preempted        bool              `json:"preempted,omitempty"`
+	Originator       bool              `json:"originator,omitempty"`
 }
