@@ -24,10 +24,12 @@ import (
 	"io"
 	"math"
 	"net/http"
+	"net/url"
 	"runtime"
 	"sort"
 	"strconv"
 	"strings"
+	"sync/atomic"
 	"time"
 
 	"github.com/julienschmidt/httprouter"
@@ -54,28 +56,32 @@ const (
 	PartitionDoesNotExists   = "Partition not found"
 	MissingParamsName        = "Missing parameters"
 	QueueDoesNotExists       = "Queue not found"
+	InvalidUserName          = "Invalid user name"
+	InvalidGroupName         = "Invalid group name"
 	UserDoesNotExists        = "User not found"
 	GroupDoesNotExists       = "Group not found"
-	UserNameMissing          = "User name is missing"
-	GroupNameMissing         = "Group name is missing"
 	ApplicationDoesNotExists = "Application not found"
 	NodeDoesNotExists        = "Node not found"
+
+	AppStateActive    = "active"
+	AppStateRejected  = "rejected"
+	AppStateCompleted = "completed"
 )
 
 var allowedActiveStatusMsg string
 var allowedAppActiveStatuses map[string]bool
 var streamingLimiter *StreamingLimiter
+var maxRESTResponseSize atomic.Uint64
 
 func init() {
 	allowedAppActiveStatuses = make(map[string]bool)
 
-	allowedAppActiveStatuses["new"] = true
-	allowedAppActiveStatuses["accepted"] = true
-	allowedAppActiveStatuses["starting"] = true
-	allowedAppActiveStatuses["running"] = true
-	allowedAppActiveStatuses["completing"] = true
-	allowedAppActiveStatuses["failing"] = true
-	allowedAppActiveStatuses["resuming"] = true
+	allowedAppActiveStatuses[strings.ToLower(objects.New.String())] = true
+	allowedAppActiveStatuses[strings.ToLower(objects.Accepted.String())] = true
+	allowedAppActiveStatuses[strings.ToLower(objects.Running.String())] = true
+	allowedAppActiveStatuses[strings.ToLower(objects.Completing.String())] = true
+	allowedAppActiveStatuses[strings.ToLower(objects.Failing.String())] = true
+	allowedAppActiveStatuses[strings.ToLower(objects.Resuming.String())] = true
 
 	var activeStatuses []string
 	for k := range allowedAppActiveStatuses {
@@ -84,6 +90,22 @@ func init() {
 	allowedActiveStatusMsg = fmt.Sprintf("Only following active statuses are allowed: %s", strings.Join(activeStatuses, ","))
 
 	streamingLimiter = NewStreamingLimiter()
+
+	configs.AddConfigMapCallback("rest-response-size", func() {
+		newSize := common.GetConfigurationUint(configs.GetConfigMap(), configs.CMRESTResponseSize, configs.DefaultRESTResponseSize)
+		if newSize == 0 {
+			log.Log(log.REST).Warn("Illegal value `0` for config key, using default",
+				zap.String("key", configs.CMRESTResponseSize),
+				zap.Uint64("default", configs.DefaultRESTResponseSize))
+			newSize = configs.DefaultRESTResponseSize
+		}
+
+		log.Log(log.REST).Info("Reloading max REST event response size setting",
+			zap.Uint64("current", maxRESTResponseSize.Load()),
+			zap.Uint64("new", newSize))
+		maxRESTResponseSize.Store(newSize)
+	})
+	maxRESTResponseSize.Store(configs.DefaultRESTResponseSize)
 }
 
 func getStackInfo(w http.ResponseWriter, r *http.Request) {
@@ -107,7 +129,7 @@ func getStackInfo(w http.ResponseWriter, r *http.Request) {
 func getClusterInfo(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
 
-	lists := schedulerContext.GetPartitionMapClone()
+	lists := schedulerContext.Load().GetPartitionMapClone()
 	clustersInfo := getClusterDAO(lists)
 	if err := json.NewEncoder(w).Encode(clustersInfo); err != nil {
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
@@ -118,10 +140,8 @@ func validateQueue(queuePath string) error {
 	if queuePath != "" {
 		queueNameArr := strings.Split(queuePath, ".")
 		for _, name := range queueNameArr {
-			if !configs.QueueNameRegExp.MatchString(name) {
-				return fmt.Errorf("problem in queue query parameter parsing as queue param "+
-					"%s contains invalid queue name %s. Queue name must only have "+
-					"alphanumeric characters, - or _, and be no longer than 64 characters", queuePath, name)
+			if err := configs.IsQueueNameValid(name); err != nil {
+				return err
 			}
 		}
 	}
@@ -164,8 +184,9 @@ func buildJSONErrorResponse(w http.ResponseWriter, detail string, code int) {
 
 func getClusterJSON(partition *scheduler.PartitionContext) *dao.ClusterDAOInfo {
 	clusterInfo := &dao.ClusterDAOInfo{}
-	clusterInfo.StartTime = schedulerContext.GetStartTime().UnixNano()
-	rmInfo := schedulerContext.GetRMInfoMapClone()
+	ctx := schedulerContext.Load()
+	clusterInfo.StartTime = ctx.GetStartTime().UnixNano()
+	rmInfo := ctx.GetRMInfoMapClone()
 	clusterInfo.RMBuildInformation = getRMBuildInformation(rmInfo)
 	clusterInfo.PartitionName = common.GetPartitionNameWithoutClusterID(partition.Name)
 	clusterInfo.ClusterName = "kubernetes"
@@ -211,7 +232,7 @@ func getAllocationDAO(alloc *objects.Allocation) *dao.AllocationDAOInfo {
 	if alloc.IsPlaceholderUsed() {
 		requestTime = alloc.GetPlaceholderCreateTime().UnixNano()
 	} else {
-		requestTime = alloc.GetAsk().GetCreateTime().UnixNano()
+		requestTime = alloc.GetCreateTime().UnixNano()
 	}
 	allocTime := alloc.GetCreateTime().UnixNano()
 	allocDAO := &dao.AllocationDAOInfo{
@@ -220,8 +241,6 @@ func getAllocationDAO(alloc *objects.Allocation) *dao.AllocationDAOInfo {
 		RequestTime:      requestTime,
 		AllocationTime:   allocTime,
 		AllocationDelay:  allocTime - requestTime,
-		UUID:             alloc.GetAllocationID(),
-		AllocationID:     alloc.GetAllocationID(),
 		ResourcePerAlloc: alloc.GetAllocatedResource().DAOMap(),
 		PlaceholderUsed:  alloc.IsPlaceholderUsed(),
 		Placeholder:      alloc.IsPlaceholder(),
@@ -229,8 +248,8 @@ func getAllocationDAO(alloc *objects.Allocation) *dao.AllocationDAOInfo {
 		Priority:         strconv.Itoa(int(alloc.GetPriority())),
 		NodeID:           alloc.GetNodeID(),
 		ApplicationID:    alloc.GetApplicationID(),
-		Partition:        alloc.GetPartitionName(),
 		Preempted:        alloc.IsPreempted(),
+		Originator:       alloc.IsOriginator(),
 	}
 	return allocDAO
 }
@@ -278,31 +297,35 @@ func getStatesDAO(entries []*objects.StateLogEntry) []*dao.StateDAOInfo {
 	return statesDAO
 }
 
-func getApplicationDAO(app *objects.Application) *dao.ApplicationDAOInfo {
+func getApplicationDAO(app *objects.Application, summary *objects.ApplicationSummary) *dao.ApplicationDAOInfo {
 	if app == nil {
 		return &dao.ApplicationDAOInfo{}
 	}
 
 	return &dao.ApplicationDAOInfo{
-		ApplicationID:      app.ApplicationID,
-		UsedResource:       app.GetAllocatedResource().DAOMap(),
-		MaxUsedResource:    app.GetMaxAllocatedResource().DAOMap(),
-		PendingResource:    app.GetPendingResource().DAOMap(),
-		Partition:          common.GetPartitionNameWithoutClusterID(app.Partition),
-		QueueName:          app.GetQueuePath(),
-		SubmissionTime:     app.SubmissionTime.UnixNano(),
-		FinishedTime:       common.ZeroTimeInUnixNano(app.FinishedTime()),
-		Requests:           getAllocationAsksDAO(app.GetAllRequests()),
-		Allocations:        getAllocationsDAO(app.GetAllAllocations()),
-		State:              app.CurrentState(),
-		User:               app.GetUser().User,
-		Groups:             app.GetUser().Groups,
-		RejectedMessage:    app.GetRejectedMessage(),
-		PlaceholderData:    getPlaceholdersDAO(app.GetAllPlaceholderData()),
-		StateLog:           getStatesDAO(app.GetStateLog()),
-		HasReserved:        app.HasReserved(),
-		Reservations:       app.GetReservations(),
-		MaxRequestPriority: app.GetAskMaxPriority(),
+		ApplicationID:       app.ApplicationID,
+		UsedResource:        app.GetAllocatedResource().DAOMap(),
+		MaxUsedResource:     app.GetMaxAllocatedResource().DAOMap(),
+		PendingResource:     app.GetPendingResource().DAOMap(),
+		Partition:           common.GetPartitionNameWithoutClusterID(app.Partition),
+		QueueName:           app.GetQueuePath(),
+		SubmissionTime:      app.SubmissionTime.UnixNano(),
+		FinishedTime:        common.ZeroTimeInUnixNano(app.FinishedTime()),
+		Requests:            getAllocationAsksDAO(app.GetAllRequests()),
+		Allocations:         getAllocationsDAO(app.GetAllAllocations()),
+		State:               app.CurrentState(),
+		User:                app.GetUser().User,
+		Groups:              app.GetUser().Groups,
+		RejectedMessage:     app.GetRejectedMessage(),
+		PlaceholderData:     getPlaceholdersDAO(app.GetAllPlaceholderData()),
+		StateLog:            getStatesDAO(app.GetStateLog()),
+		HasReserved:         app.HasReserved(),
+		Reservations:        app.GetReservations(),
+		MaxRequestPriority:  app.GetAskMaxPriority(),
+		StartTime:           app.StartTime().UnixMilli(),
+		ResourceUsage:       summary.ResourceUsage,
+		PreemptedResource:   summary.PreemptedResource,
+		PlaceholderResource: summary.PlaceholderResource,
 	}
 }
 
@@ -321,19 +344,16 @@ func getAllocationLogsDAO(logEntries []*objects.AllocationLogEntry) []*dao.Alloc
 	return logsDAO
 }
 
-func getAllocationAskDAO(ask *objects.AllocationAsk) *dao.AllocationAskDAOInfo {
+func getAllocationAskDAO(ask *objects.Allocation) *dao.AllocationAskDAOInfo {
 	return &dao.AllocationAskDAOInfo{
 		AllocationKey:       ask.GetAllocationKey(),
 		AllocationTags:      ask.GetTagsClone(),
 		RequestTime:         ask.GetCreateTime().UnixNano(),
 		ResourcePerAlloc:    ask.GetAllocatedResource().DAOMap(),
-		PendingCount:        ask.GetPendingAskRepeat(),
 		Priority:            strconv.Itoa(int(ask.GetPriority())),
 		RequiredNodeID:      ask.GetRequiredNode(),
 		ApplicationID:       ask.GetApplicationID(),
-		Partition:           common.GetPartitionNameWithoutClusterID(ask.GetPartitionName()),
 		Placeholder:         ask.IsPlaceholder(),
-		PlaceholderTimeout:  ask.GetTimeout().Nanoseconds(),
 		TaskGroupName:       ask.GetTaskGroup(),
 		AllocationLog:       getAllocationLogsDAO(ask.GetAllocationLog()),
 		TriggeredPreemption: ask.HasTriggeredPreemption(),
@@ -343,10 +363,10 @@ func getAllocationAskDAO(ask *objects.AllocationAsk) *dao.AllocationAskDAOInfo {
 	}
 }
 
-func getAllocationAsksDAO(asks []*objects.AllocationAsk) []*dao.AllocationAskDAOInfo {
+func getAllocationAsksDAO(asks []*objects.Allocation) []*dao.AllocationAskDAOInfo {
 	asksDAO := make([]*dao.AllocationAskDAOInfo, 0, len(asks))
 	for _, ask := range asks {
-		if ask.GetPendingAskRepeat() > 0 {
+		if !ask.IsAllocated() {
 			asksDAO = append(asksDAO, getAllocationAskDAO(ask))
 		}
 	}
@@ -386,7 +406,7 @@ func getNodesDAO(entries []*objects.Node) []*dao.NodeDAOInfo {
 // Deprecated - To be removed in next major release. Replaced with getNodesUtilisations
 func getNodeUtilisation(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
-	partitionContext := schedulerContext.GetPartitionWithoutClusterID(configs.DefaultPartition)
+	partitionContext := schedulerContext.Load().GetPartitionWithoutClusterID(configs.DefaultPartition)
 	if partitionContext == nil {
 		buildJSONErrorResponse(w, PartitionDoesNotExists, http.StatusInternalServerError)
 		return
@@ -451,7 +471,7 @@ func getNodesUtilJSON(partition *scheduler.PartitionContext, name string) *dao.N
 func getNodeUtilisations(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
 	var result []*dao.PartitionNodesUtilDAOInfo
-	for _, part := range schedulerContext.GetPartitionMapClone() {
+	for _, part := range schedulerContext.Load().GetPartitionMapClone() {
 		result = append(result, getPartitionNodesUtilJSON(part))
 	}
 
@@ -582,7 +602,7 @@ func getClusterConfig(w http.ResponseWriter, r *http.Request) {
 func getClusterConfigDAO() *dao.ConfigDAOInfo {
 	// merge core config with extra config
 	conf := dao.ConfigDAOInfo{
-		SchedulerConfig:          configs.ConfigContext.Get(schedulerContext.GetPolicyGroup()),
+		SchedulerConfig:          configs.ConfigContext.Get(schedulerContext.Load().GetPolicyGroup()),
 		Extra:                    configs.GetConfigMap(),
 		DeadlockDetectionEnabled: locking.IsTrackingEnabled(),
 		DeadlockTimeoutSeconds:   locking.GetDeadlockTimeoutSeconds(),
@@ -595,7 +615,7 @@ func checkHealthStatus(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
 
 	// Fetch last healthCheck result
-	result := schedulerContext.GetLastHealthCheckResult()
+	result := schedulerContext.Load().GetLastHealthCheckResult()
 	if result != nil {
 		if !result.Healthy {
 			log.Log(log.SchedHealth).Error("Scheduler is not healthy", zap.Any("health check info", *result))
@@ -617,7 +637,7 @@ func checkHealthStatus(w http.ResponseWriter, r *http.Request) {
 func getPartitions(w http.ResponseWriter, _ *http.Request) {
 	writeHeaders(w)
 
-	lists := schedulerContext.GetPartitionMapClone()
+	lists := schedulerContext.Load().GetPartitionMapClone()
 	partitionsInfo := getPartitionInfoDAO(lists)
 	if err := json.NewEncoder(w).Encode(partitionsInfo); err != nil {
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
@@ -633,7 +653,7 @@ func getPartitionQueues(w http.ResponseWriter, r *http.Request) {
 	}
 	partitionName := vars.ByName("partition")
 	var partitionQueuesDAOInfo dao.PartitionQueueDAOInfo
-	var partition = schedulerContext.GetPartitionWithoutClusterID(partitionName)
+	var partition = schedulerContext.Load().GetPartitionWithoutClusterID(partitionName)
 	if partition != nil {
 		partitionQueuesDAOInfo = partition.GetPartitionQueues()
 	} else {
@@ -653,18 +673,23 @@ func getPartitionQueue(w http.ResponseWriter, r *http.Request) {
 		return
 	}
 	partition := vars.ByName("partition")
-	partitionContext := schedulerContext.GetPartitionWithoutClusterID(partition)
+	partitionContext := schedulerContext.Load().GetPartitionWithoutClusterID(partition)
 	if partitionContext == nil {
 		buildJSONErrorResponse(w, PartitionDoesNotExists, http.StatusNotFound)
 		return
 	}
 	queueName := vars.ByName("queue")
-	queueErr := validateQueue(queueName)
+	unescapedQueueName, err := url.QueryUnescape(queueName)
+	if err != nil {
+		buildJSONErrorResponse(w, err.Error(), http.StatusBadRequest)
+		return
+	}
+	queueErr := validateQueue(unescapedQueueName)
 	if queueErr != nil {
 		buildJSONErrorResponse(w, queueErr.Error(), http.StatusBadRequest)
 		return
 	}
-	queue := partitionContext.GetQueue(queueName)
+	queue := partitionContext.GetQueue(unescapedQueueName)
 	if queue == nil {
 		buildJSONErrorResponse(w, QueueDoesNotExists, http.StatusNotFound)
 		return
@@ -683,7 +708,7 @@ func getPartitionNodes(w http.ResponseWriter, r *http.Request) {
 		return
 	}
 	partition := vars.ByName("partition")
-	partitionContext := schedulerContext.GetPartitionWithoutClusterID(partition)
+	partitionContext := schedulerContext.Load().GetPartitionWithoutClusterID(partition)
 	if partitionContext != nil {
 		nodesDao := getNodesDAO(partitionContext.GetNodes())
 		if err := json.NewEncoder(w).Encode(nodesDao); err != nil {
@@ -702,7 +727,7 @@ func getPartitionNode(w http.ResponseWriter, r *http.Request) {
 		return
 	}
 	partition := vars.ByName("partition")
-	partitionContext := schedulerContext.GetPartitionWithoutClusterID(partition)
+	partitionContext := schedulerContext.Load().GetPartitionWithoutClusterID(partition)
 	if partitionContext != nil {
 		nodeID := vars.ByName("node")
 		node := partitionContext.GetNode(nodeID)
@@ -728,17 +753,22 @@ func getQueueApplications(w http.ResponseWriter, r *http.Request) {
 	}
 	partition := vars.ByName("partition")
 	queueName := vars.ByName("queue")
-	queueErr := validateQueue(queueName)
+	unescapedQueueName, err := url.QueryUnescape(queueName)
+	if err != nil {
+		buildJSONErrorResponse(w, err.Error(), http.StatusBadRequest)
+		return
+	}
+	queueErr := validateQueue(unescapedQueueName)
 	if queueErr != nil {
 		buildJSONErrorResponse(w, queueErr.Error(), http.StatusBadRequest)
 		return
 	}
-	partitionContext := schedulerContext.GetPartitionWithoutClusterID(partition)
+	partitionContext := schedulerContext.Load().GetPartitionWithoutClusterID(partition)
 	if partitionContext == nil {
 		buildJSONErrorResponse(w, PartitionDoesNotExists, http.StatusNotFound)
 		return
 	}
-	queue := partitionContext.GetQueue(queueName)
+	queue := partitionContext.GetQueue(unescapedQueueName)
 	if queue == nil {
 		buildJSONErrorResponse(w, QueueDoesNotExists, http.StatusNotFound)
 		return
@@ -746,7 +776,8 @@ func getQueueApplications(w http.ResponseWriter, r *http.Request) {
 
 	appsDao := make([]*dao.ApplicationDAOInfo, 0)
 	for _, app := range queue.GetCopyOfApps() {
-		appsDao = append(appsDao, getApplicationDAO(app))
+		summary := app.GetApplicationSummary(partitionContext.RmID)
+		appsDao = append(appsDao, getApplicationDAO(app, summary))
 	}
 
 	if err := json.NewEncoder(w).Encode(appsDao); err != nil {
@@ -764,14 +795,14 @@ func getPartitionApplicationsByState(w http.ResponseWriter, r *http.Request) {
 	partition := vars.ByName("partition")
 	appState := strings.ToLower(vars.ByName("state"))
 
-	partitionContext := schedulerContext.GetPartitionWithoutClusterID(partition)
+	partitionContext := schedulerContext.Load().GetPartitionWithoutClusterID(partition)
 	if partitionContext == nil {
 		buildJSONErrorResponse(w, PartitionDoesNotExists, http.StatusNotFound)
 		return
 	}
 	var appList []*objects.Application
 	switch appState {
-	case "active":
+	case AppStateActive:
 		if status := strings.ToLower(r.URL.Query().Get("status")); status != "" {
 			if !allowedAppActiveStatuses[status] {
 				buildJSONErrorResponse(w, allowedActiveStatusMsg, http.StatusBadRequest)
@@ -785,17 +816,18 @@ func getPartitionApplicationsByState(w http.ResponseWriter, r *http.Request) {
 		} else {
 			appList = partitionContext.GetApplications()
 		}
-	case "rejected":
+	case AppStateRejected:
 		appList = partitionContext.GetRejectedApplications()
-	case "completed":
+	case AppStateCompleted:
 		appList = partitionContext.GetCompletedApplications()
 	default:
-		buildJSONErrorResponse(w, "Only following application states are allowed: active, rejected, completed", http.StatusBadRequest)
+		buildJSONErrorResponse(w, fmt.Sprintf("Only following application states are allowed: %s, %s, %s", AppStateActive, AppStateRejected, AppStateCompleted), http.StatusBadRequest)
 		return
 	}
 	appsDao := make([]*dao.ApplicationDAOInfo, 0, len(appList))
 	for _, app := range appList {
-		appsDao = append(appsDao, getApplicationDAO(app))
+		summary := app.GetApplicationSummary(partitionContext.RmID)
+		appsDao = append(appsDao, getApplicationDAO(app, summary))
 	}
 	if err := json.NewEncoder(w).Encode(appsDao); err != nil {
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
@@ -811,22 +843,27 @@ func getApplication(w http.ResponseWriter, r *http.Request) {
 	}
 	partition := vars.ByName("partition")
 	queueName := vars.ByName("queue")
+	unescapedQueueName, err := url.QueryUnescape(queueName)
+	if err != nil {
+		buildJSONErrorResponse(w, err.Error(), http.StatusBadRequest)
+		return
+	}
 	application := vars.ByName("application")
-	partitionContext := schedulerContext.GetPartitionWithoutClusterID(partition)
+	partitionContext := schedulerContext.Load().GetPartitionWithoutClusterID(partition)
 	if partitionContext == nil {
 		buildJSONErrorResponse(w, PartitionDoesNotExists, http.StatusNotFound)
 		return
 	}
 	var app *objects.Application
-	if len(queueName) == 0 {
+	if len(unescapedQueueName) == 0 {
 		app = partitionContext.GetApplication(application)
 	} else {
-		queueErr := validateQueue(queueName)
+		queueErr := validateQueue(unescapedQueueName)
 		if queueErr != nil {
 			buildJSONErrorResponse(w, queueErr.Error(), http.StatusBadRequest)
 			return
 		}
-		queue := partitionContext.GetQueue(queueName)
+		queue := partitionContext.GetQueue(unescapedQueueName)
 		if queue == nil {
 			buildJSONErrorResponse(w, QueueDoesNotExists, http.StatusNotFound)
 			return
@@ -837,12 +874,88 @@ func getApplication(w http.ResponseWriter, r *http.Request) {
 		buildJSONErrorResponse(w, ApplicationDoesNotExists, http.StatusNotFound)
 		return
 	}
-	appDao := getApplicationDAO(app)
+
+	summary := app.GetApplicationSummary(partitionContext.RmID)
+	appDao := getApplicationDAO(app, summary)
 	if err := json.NewEncoder(w).Encode(appDao); err != nil {
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 	}
 }
 
+func getPartitionRules(w http.ResponseWriter, r *http.Request) {
+	writeHeaders(w)
+	vars := httprouter.ParamsFromContext(r.Context())
+	if vars == nil {
+		buildJSONErrorResponse(w, MissingParamsName, http.StatusBadRequest)
+		return
+	}
+	partition := vars.ByName("partition")
+	partitionContext := schedulerContext.Load().GetPartitionWithoutClusterID(partition)
+	if partitionContext == nil {
+		buildJSONErrorResponse(w, PartitionDoesNotExists, http.StatusNotFound)
+		return
+	}
+	rulesDao := partitionContext.GetPlacementRules()
+	if err := json.NewEncoder(w).Encode(rulesDao); err != nil {
+		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
+	}
+}
+
+func getQueueApplicationsByState(w http.ResponseWriter, r *http.Request) {
+	writeHeaders(w)
+	vars := httprouter.ParamsFromContext(r.Context())
+	if vars == nil {
+		buildJSONErrorResponse(w, MissingParamsName, http.StatusBadRequest)
+		return
+	}
+
+	partition := vars.ByName("partition")
+	queueName := vars.ByName("queue")
+	appState := strings.ToLower(vars.ByName("state"))
+	status := strings.ToLower(r.URL.Query().Get("status"))
+
+	partitionContext := schedulerContext.Load().GetPartitionWithoutClusterID(partition)
+	if partitionContext == nil {
+		buildJSONErrorResponse(w, PartitionDoesNotExists, http.StatusNotFound)
+		return
+	}
+	unescapedQueueName, err := url.QueryUnescape(queueName)
+	if err != nil {
+		buildJSONErrorResponse(w, err.Error(), http.StatusBadRequest)
+		return
+	}
+	queueErr := validateQueue(unescapedQueueName)
+	if queueErr != nil {
+		buildJSONErrorResponse(w, queueErr.Error(), http.StatusBadRequest)
+		return
+	}
+	queue := partitionContext.GetQueue(unescapedQueueName)
+	if queue == nil {
+		buildJSONErrorResponse(w, QueueDoesNotExists, http.StatusNotFound)
+		return
+	}
+	if appState != AppStateActive {
+		buildJSONErrorResponse(w, fmt.Sprintf("Only following application states are allowed: %s", AppStateActive), http.StatusBadRequest)
+		return
+	}
+	if status != "" && !allowedAppActiveStatuses[status] {
+		buildJSONErrorResponse(w, allowedActiveStatusMsg, http.StatusBadRequest)
+		return
+	}
+
+	appsDao := make([]*dao.ApplicationDAOInfo, 0)
+	for _, app := range queue.GetCopyOfApps() {
+		if status == "" || strings.ToLower(app.CurrentState()) == status {
+			summary := app.GetApplicationSummary(partitionContext.RmID)
+			appsDao = append(appsDao, getApplicationDAO(app, summary))
+		}
+	}
+
+	if err := json.NewEncoder(w).Encode(appsDao); err != nil {
+		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
+	}
+}
+
 func getPartitionInfoDAO(lists map[string]*scheduler.PartitionContext) []*dao.PartitionInfo {
 	result := make([]*dao.PartitionInfo, 0, len(lists))
 
@@ -942,13 +1055,27 @@ func getApplicationsDAO(lists map[string]*scheduler.PartitionContext) []*dao.App
 		appList = append(appList, partition.GetRejectedApplications()...)
 
 		for _, app := range appList {
-			result = append(result, getApplicationDAO(app))
+			summary := app.GetApplicationSummary(partition.RmID)
+			result = append(result, getApplicationDAO(app, summary))
 		}
 	}
 
 	return result
 }
 
+func getPlacementRulesDAO(lists map[string]*scheduler.PartitionContext) []*dao.RuleDAOInfo {
+	result := make([]*dao.RuleDAOInfo, 0, len(lists))
+
+	for _, partition := range lists {
+		result = append(result, &dao.RuleDAOInfo{
+			Partition: common.GetPartitionNameWithoutClusterID(partition.Name),
+			Rules:     partition.GetPlacementRules(),
+		})
+	}
+
+	return result
+}
+
 func getPartitionQueuesDAO(lists map[string]*scheduler.PartitionContext) []dao.PartitionQueueDAOInfo {
 	result := make([]dao.PartitionQueueDAOInfo, 0, len(lists))
 
@@ -1033,11 +1160,16 @@ func getUserResourceUsage(w http.ResponseWriter, r *http.Request) {
 		return
 	}
 	user := vars.ByName("user")
-	if user == "" {
-		buildJSONErrorResponse(w, UserNameMissing, http.StatusBadRequest)
+	unescapedUser, err := url.QueryUnescape(user)
+	if err != nil {
+		buildJSONErrorResponse(w, err.Error(), http.StatusBadRequest)
 		return
 	}
-	userTracker := ugm.GetUserManager().GetUserTracker(user)
+	if !configs.UserRegExp.MatchString(unescapedUser) {
+		buildJSONErrorResponse(w, InvalidUserName, http.StatusBadRequest)
+		return
+	}
+	userTracker := ugm.GetUserManager().GetUserTracker(unescapedUser)
 	if userTracker == nil {
 		buildJSONErrorResponse(w, UserDoesNotExists, http.StatusNotFound)
 		return
@@ -1069,11 +1201,16 @@ func getGroupResourceUsage(w http.ResponseWriter, r *http.Request) {
 		return
 	}
 	group := vars.ByName("group")
-	if group == "" {
-		buildJSONErrorResponse(w, GroupNameMissing, http.StatusBadRequest)
+	unescapedGroupName, err := url.QueryUnescape(group)
+	if err != nil {
+		buildJSONErrorResponse(w, err.Error(), http.StatusBadRequest)
 		return
 	}
-	groupTracker := ugm.GetUserManager().GetGroupTracker(group)
+	if !configs.GroupRegExp.MatchString(unescapedGroupName) {
+		buildJSONErrorResponse(w, InvalidGroupName, http.StatusBadRequest)
+		return
+	}
+	groupTracker := ugm.GetUserManager().GetGroupTracker(unescapedGroupName)
 	if groupTracker == nil {
 		buildJSONErrorResponse(w, GroupDoesNotExists, http.StatusNotFound)
 		return
@@ -1092,9 +1229,8 @@ func getEvents(w http.ResponseWriter, r *http.Request) {
 		return
 	}
 
-	count := uint64(10000)
-	var start uint64
-
+	maxCount := maxRESTResponseSize.Load()
+	count := maxCount
 	if countStr := r.URL.Query().Get("count"); countStr != "" {
 		var err error
 		count, err = strconv.ParseUint(countStr, 10, 64)
@@ -1102,12 +1238,16 @@ func getEvents(w http.ResponseWriter, r *http.Request) {
 			buildJSONErrorResponse(w, err.Error(), http.StatusBadRequest)
 			return
 		}
+		if count > maxCount {
+			count = maxCount
+		}
 		if count == 0 {
 			buildJSONErrorResponse(w, `0 is not a valid value for "count"`, http.StatusBadRequest)
 			return
 		}
 	}
 
+	var start uint64
 	if startStr := r.URL.Query().Get("start"); startStr != "" {
 		var err error
 		start, err = strconv.ParseUint(startStr, 10, 64)
@@ -1119,7 +1259,7 @@ func getEvents(w http.ResponseWriter, r *http.Request) {
 
 	records, lowestID, highestID := eventSystem.GetEventsFromID(start, count)
 	eventDao := dao.EventRecordDAO{
-		InstanceUUID: schedulerContext.GetUUID(),
+		InstanceUUID: schedulerContext.Load().GetUUID(),
 		LowestID:     lowestID,
 		HighestID:    highestID,
 		EventRecords: records,
@@ -1175,6 +1315,14 @@ func getStream(w http.ResponseWriter, r *http.Request) {
 	stream := eventSystem.CreateEventStream(r.Host, count)
 	defer eventSystem.RemoveStream(stream)
 
+	if err := enc.Encode(dao.YunikornID{
+		InstanceUUID: schedulerContext.Load().GetUUID(),
+	}); err != nil {
+		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
+		return
+	}
+	f.Flush()
+
 	// Reading events in an infinite loop until either the client disconnects or Yunikorn closes the channel.
 	// This results in a persistent HTTP connection where the message body is never closed.
 	// Write deadline is adjusted before sending data to the client.
@@ -1189,7 +1337,7 @@ func getStream(w http.ResponseWriter, r *http.Request) {
 			if err != nil {
 				// should not fail at this point
 				log.Log(log.REST).Error("Cannot set write deadline", zap.Error(err))
-				buildJSONErrorResponse(w, fmt.Sprintf("Cannot set write deadline: %v", err), http.StatusInternalServerError)
+				buildJSONErrorResponse(w, fmt.Sprintf("Cannot set write deadline: %v", err), http.StatusOK) // status code is already 200 at this point
 				return
 			}
 
