@@ -30,7 +30,6 @@ usage() {
 case "$1" in
    --help)
        usage
-       exit 0
        ;;
 esac
 
@@ -101,14 +100,14 @@ if [ -f "$SPARK_BINARY_FILE_PATH" ]; then
   echo "The binary file $SPARK_BINARY_FILE_NAME has been cached!"
 else
   echo "The binary file $SPARK_BINARY_FILE_NAME did not exist, try to download."
-  wget https://archive.apache.org/dist/spark/spark-${SPARK_VERSION}/${SPARK_BINARY_FILE_NAME} -O "${SPARK_BINARY_FILE_PATH}"
+  wget "https://archive.apache.org/dist/spark/spark-${SPARK_VERSION}/${SPARK_BINARY_FILE_NAME}" -O "${SPARK_BINARY_FILE_PATH}"
 fi
 
 if [ -f "$SPARK_BINARY_FILE_CHECKSUM_FILE_PATH" ]; then
   echo "The binary checksum file $SPARK_BINARY_FILE_CHECKSUM_FILE_NAME has been cached!"
 else
   echo "The binary checksum file $SPARK_BINARY_FILE_CHECKSUM_FILE_NAME did not exist, try to download."
-  wget http://archive.apache.org/dist/spark/spark-${SPARK_VERSION}/${SPARK_BINARY_FILE_CHECKSUM_FILE_NAME} -O "${SPARK_BINARY_FILE_CHECKSUM_FILE_PATH}"
+  wget "http://archive.apache.org/dist/spark/spark-${SPARK_VERSION}/${SPARK_BINARY_FILE_CHECKSUM_FILE_NAME}" -O "${SPARK_BINARY_FILE_CHECKSUM_FILE_PATH}"
 fi
 
 if [[ "$SPARK_VERSION" < "3.2.1"  || "$SPARK_VERSION" == "3.2.1" ]]; then
@@ -123,7 +122,7 @@ if [[ "$SPARK_VERSION" < "3.2.1"  || "$SPARK_VERSION" == "3.2.1" ]]; then
   if [[ 'OK' == $(shasum -c -a 512 "$FORMATTED_SPARK_BINARY_FILE_CHECKSUM_FILE_PATH"  | awk '{print $2}') ]]; then
     echo "The checksum is matched!"
     echo "Try to remove the old unpacked dir and re-uncompress it"
-    rm -rf "$WORK_SPACE_ROOT"/spark-${SPARK_VERSION}-bin-hadoop${SPARK_HADOOP_VERSION}
+    rm -rf "$WORK_SPACE_ROOT"/spark-"${SPARK_VERSION}"-bin-hadoop"${SPARK_HADOOP_VERSION}"
     tar -xvzf "$SPARK_BINARY_FILE_PATH" -C "$WORK_SPACE_ROOT"
   else
     echo "The checksum is not matched, Removing the incompleted file, please download it again."
@@ -135,7 +134,7 @@ else
   if [[ $(shasum -a 512 "$SPARK_BINARY_FILE_PATH" | awk '{print $1}') == $(awk '{print $1}' < "$SPARK_BINARY_FILE_CHECKSUM_FILE_NAME") ]]; then
     echo "The checksum is matched!"
     echo "Try to remove the old unpacked dir and re-uncompress it"
-    rm -rf "$WORK_SPACE_ROOT"/spark-${SPARK_VERSION}-bin-hadoop${SPARK_HADOOP_VERSION}
+    rm -rf "$WORK_SPACE_ROOT"/spark-"${SPARK_VERSION}"-bin-hadoop"${SPARK_HADOOP_VERSION}"
     tar -xvzf "$SPARK_BINARY_FILE_PATH" -C "$WORK_SPACE_ROOT"
   else
     echo "The checksum is not matched, Removing the incompleted file, please download it again."
@@ -195,7 +194,7 @@ EOF
   --conf spark.executor.instances=${SPARK_EXECUTOR_NUM} \
   --conf spark.kubernetes.namespace=spark-test \
   --conf spark.kubernetes.executor.request.cores=1 \
-  --conf spark.kubernetes.container.image=${SPARK_DOCKER_IMAGE} \
+  --conf spark.kubernetes.container.image="${SPARK_DOCKER_IMAGE}" \
   --conf spark.kubernetes.authenticate.driver.serviceAccountName=spark \
   --conf spark.kubernetes.driver.podTemplateFile=../driver.yaml \
   --conf spark.kubernetes.executor.podTemplateFile=../executor.yaml \
