@@ -442,9 +442,9 @@ func TestReleaseAppAllocation(t *testing.T) {
 	app := NewApplication(appID, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	task := NewTask("task01", app, context, pod)
 	app.addTask(task)
-	task.allocationID = taskAllocationID
+	task.allocationKey = task.taskID
 	// app must be running states
-	err := app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, taskAllocationID))
+	err := app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, task.taskID))
 	if err == nil {
 		// this should give an error
 		t.Error("expecting error got 'nil'")
@@ -452,7 +452,7 @@ func TestReleaseAppAllocation(t *testing.T) {
 	// set app states to running, let event can be trigger
 	app.SetState(ApplicationStates().Running)
 	assertAppState(t, app, ApplicationStates().Running, 3*time.Second)
-	err = app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, taskAllocationID))
+	err = app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, task.taskID))
 	assert.NilError(t, err)
 	// after handle release event the states of app must be running
 	assertAppState(t, app, ApplicationStates().Running, 3*time.Second)
@@ -525,6 +525,7 @@ func assertAppState(t *testing.T, app *Application, expectedState string, durati
 	}
 }
 
+//nolint:funlen
 func TestGetNonTerminatedTaskAlias(t *testing.T) {
 	context := initContextForTest()
 	app := NewApplication(appID, "root.a", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
@@ -749,6 +750,7 @@ func TestTryReserve(t *testing.T) {
 	assert.NilError(t, err, "placeholders are not created")
 }
 
+//nolint:funlen
 func TestTryReservePostRestart(t *testing.T) {
 	context := initContextForTest()
 	dispatcher.RegisterEventHandler("TestAppHandler", dispatcher.EventTypeApp, context.ApplicationEventHandler())
@@ -823,7 +825,7 @@ func TestTryReservePostRestart(t *testing.T) {
 			Containers: containers,
 		},
 	})
-	task0.allocationID = string(task0.pod.UID)
+	task0.allocationKey = string(task0.pod.UID)
 	task0.nodeName = "fake-host"
 	task0.sm.SetState(TaskStates().Allocated)
 
@@ -1002,9 +1004,10 @@ func TestReleaseAppAllocationInFailingState(t *testing.T) {
 	app := NewApplication(appID, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	task := NewTask("task01", app, context, pod)
 	app.addTask(task)
-	task.allocationID = taskAllocationID
+	task.allocationKey = task.taskID
+
 	// app must be running states
-	err := app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, taskAllocationID))
+	err := app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, task.taskID))
 	if err == nil {
 		// this should give an error
 		t.Error("expecting error got 'nil'")
@@ -1012,12 +1015,12 @@ func TestReleaseAppAllocationInFailingState(t *testing.T) {
 	// set app states to running, let event can be trigger
 	app.SetState(ApplicationStates().Running)
 	assertAppState(t, app, ApplicationStates().Running, 3*time.Second)
-	err = app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, taskAllocationID))
+	err = app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, task.taskID))
 	assert.NilError(t, err)
 	// after handle release event the states of app must be running
 	assertAppState(t, app, ApplicationStates().Running, 3*time.Second)
 	app.SetState(ApplicationStates().Failing)
-	err = app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, taskAllocationID))
+	err = app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, task.taskID))
 	assert.NilError(t, err)
 	// after handle release event the states of app must be failing
 	assertAppState(t, app, ApplicationStates().Failing, 3*time.Second)
@@ -1057,7 +1060,7 @@ func TestResumingStateTransitions(t *testing.T) {
 	// Add tasks
 	app.addTask(task1)
 	app.addTask(task2)
-	task1.allocationID = taskAllocationID
+	task1.allocationKey = task1.taskID
 	context.addApplicationToContext(app)
 
 	// Set app state to "reserving"
@@ -1127,7 +1130,7 @@ func TestPlaceholderTimeoutEvents(t *testing.T) {
 		ObjectMeta: apis.ObjectMeta{
 			Name:      "pod00001",
 			Namespace: "default",
-			UID:       "UID-POD-00001",
+			UID:       "task01",
 			Labels: map[string]string{
 				"queue":         "root.a",
 				"applicationId": "app00001",
@@ -1150,7 +1153,7 @@ func TestPlaceholderTimeoutEvents(t *testing.T) {
 		ObjectMeta: apis.ObjectMeta{
 			Name:      "pod00002",
 			Namespace: "default",
-			UID:       "UID-POD-00002",
+			UID:       "task02",
 			Labels: map[string]string{
 				"queue":         "root.a",
 				"applicationId": "app00001",
@@ -1169,7 +1172,7 @@ func TestPlaceholderTimeoutEvents(t *testing.T) {
 	assert.Equal(t, len(app.GetNewTasks()), 1)
 
 	appID := "app00001"
-	allocationID := "UID-POD-00002"
+	allocationKey := "task02"
 
 	task1 := context.AddTask(&AddTaskRequest{
 		Metadata: TaskMetadata{
@@ -1181,21 +1184,19 @@ func TestPlaceholderTimeoutEvents(t *testing.T) {
 	})
 	assert.Assert(t, task1 != nil)
 	assert.Equal(t, task1.GetTaskID(), "task02")
+	assert.Assert(t, app.GetTask("task02") != nil, "Task should exist")
 
-	_, taskErr := app.GetTask("task02")
-	assert.NilError(t, taskErr, "Task should exist")
-
-	task1.allocationID = allocationID
+	task1.allocationKey = allocationKey
 
 	// set app states to running, let event can be trigger
 	app.SetState(ApplicationStates().Running)
 	assertAppState(t, app, ApplicationStates().Running, 3*time.Second)
-	err := app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, allocationID))
+	err := app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, allocationKey))
 	assert.NilError(t, err)
 	// after handle release event the states of app must be running
 	assertAppState(t, app, ApplicationStates().Running, 3*time.Second)
 
-	message := "Placeholder timed out"
+	message := "GangScheduling"
 	reason := "placeholder has been timed out"
 	// check that the event has been published
 	err = utils.WaitForCondition(func() bool {
@@ -1219,6 +1220,11 @@ func TestApplication_onReservationStateChange(t *testing.T) {
 	dispatcher.Start()
 	defer dispatcher.Stop()
 
+	recorder, ok := events.GetRecorder().(*k8sEvents.FakeRecorder)
+	if !ok {
+		t.Fatal("the EventRecorder is expected to be of type FakeRecorder")
+	}
+
 	app := NewApplication(appID, "root.a", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
 	context.addApplicationToContext(app)
 
@@ -1264,6 +1270,7 @@ func TestApplication_onReservationStateChange(t *testing.T) {
 	app.addTask(task1)
 	app.addTask(task2)
 	app.addTask(task3)
+	app.setOriginatingTask(task1)
 
 	// app stays in accepted with taskgroups defined none bound
 	app.onReservationStateChange()
@@ -1286,6 +1293,28 @@ func TestApplication_onReservationStateChange(t *testing.T) {
 	task3.setTaskGroupName("test-group-2")
 	app.onReservationStateChange()
 	assertAppState(t, app, ApplicationStates().Running, 1*time.Second)
+
+	message := "placeholder has been allocated"
+	reason := "GangScheduling"
+	counter := 0
+	// check that the event has been published
+	err := utils.WaitForCondition(func() bool {
+		for {
+			select {
+			case event := <-recorder.Events:
+				print(event)
+				if strings.Contains(event, reason) && strings.Contains(event, message) {
+					counter++
+					if counter == 4 {
+						return true
+					}
+				}
+			default:
+				return false
+			}
+		}
+	}, 5*time.Millisecond, time.Second)
+	assert.NilError(t, err, "event should have been emitted")
 }
 
 func TestTaskRemoval(t *testing.T) {
