@@ -40,24 +40,27 @@ import (
 )
 
 type Task struct {
-	taskID          string
-	alias           string
-	applicationID   string
-	application     *Application
-	allocationID    string
+	taskID        string
+	alias         string
+	applicationID string
+	application   *Application
+	podStatus     v1.PodStatus // pod status, maintained separately for efficiency reasons
+	context       *Context
+	createTime    time.Time
+	placeholder   bool
+	originator    bool
+	sm            *fsm.FSM
+
+	// mutable resources, require locking
+	allocationKey   string
+	nodeName        string
+	taskGroupName   string
+	terminationType string
+	schedulingState TaskSchedulingState
 	resource        *si.Resource
 	pod             *v1.Pod
-	podStatus       v1.PodStatus // pod status, maintained separately for efficiency reasons
-	context         *Context
-	nodeName        string
-	createTime      time.Time
-	taskGroupName   string
-	placeholder     bool
-	terminationType string
-	originator      bool
-	schedulingState TaskSchedulingState
-	sm              *fsm.FSM
-	lock            *locking.RWMutex
+
+	lock *locking.RWMutex
 }
 
 func NewTask(tid string, app *Application, ctx *Context, pod *v1.Pod) *Task {
@@ -135,14 +138,10 @@ func (task *Task) GetTaskPod() *v1.Pod {
 }
 
 func (task *Task) GetTaskID() string {
-	task.lock.RLock()
-	defer task.lock.RUnlock()
 	return task.taskID
 }
 
 func (task *Task) IsPlaceholder() bool {
-	task.lock.RLock()
-	defer task.lock.RUnlock()
 	return task.placeholder
 }
 
@@ -157,26 +156,32 @@ func (task *Task) setTaskGroupName(groupName string) {
 	task.taskGroupName = groupName
 }
 
-func (task *Task) setTaskTerminationType(terminationTyp string) {
+func (task *Task) setTaskTerminationType(terminationType string) {
 	task.lock.Lock()
 	defer task.lock.Unlock()
-	task.terminationType = terminationTyp
+	task.terminationType = terminationType
 }
 
-func (task *Task) getTaskGroupName() string {
+func (task *Task) GetTaskTerminationType() string {
+	task.lock.RLock()
+	defer task.lock.RUnlock()
+	return task.terminationType
+}
+
+func (task *Task) GetTaskGroupName() string {
 	task.lock.RLock()
 	defer task.lock.RUnlock()
 	return task.taskGroupName
 }
 
-func (task *Task) getTaskAllocationID() string {
+func (task *Task) GetNodeName() string {
 	task.lock.RLock()
 	defer task.lock.RUnlock()
-	return task.allocationID
+	return task.nodeName
 }
 
-func (task *Task) DeleteTaskPod(pod *v1.Pod) error {
-	return task.context.apiProvider.GetAPIs().KubeClient.Delete(task.pod)
+func (task *Task) DeleteTaskPod() error {
+	return task.context.apiProvider.GetAPIs().KubeClient.Delete(task.GetTaskPod())
 }
 
 func (task *Task) UpdateTaskPodStatus(pod *v1.Pod) (*v1.Pod, error) {
@@ -210,20 +215,18 @@ func (task *Task) initialize() {
 	// the resources were already released, instead of starting
 	// from New, directly set the task to Completed
 	if utils.IsPodTerminated(task.pod) {
-		task.allocationID = string(task.pod.UID)
+		task.allocationKey = string(task.pod.UID)
 		task.nodeName = task.pod.Spec.NodeName
 		task.sm.SetState(TaskStates().Completed)
 		log.Log(log.ShimCacheTask).Info("set task as Completed",
 			zap.String("appID", task.applicationID),
 			zap.String("taskID", task.taskID),
-			zap.String("allocationID", task.allocationID),
+			zap.String("allocationKey", task.allocationKey),
 			zap.String("nodeName", task.nodeName))
 	}
 }
 
 func (task *Task) IsOriginator() bool {
-	task.lock.RLock()
-	defer task.lock.RUnlock()
 	return task.originator
 }
 
@@ -260,12 +263,12 @@ func (task *Task) SetTaskSchedulingState(state TaskSchedulingState) {
 	task.schedulingState = state
 }
 
-func (task *Task) MarkPreviouslyAllocated(allocationID string, nodeID string) {
+func (task *Task) MarkPreviouslyAllocated(allocationKey string, nodeID string) {
 	task.sm.SetState(TaskStates().Bound)
 	task.lock.Lock()
 	defer task.lock.Unlock()
 	task.schedulingState = TaskSchedAllocated
-	task.allocationID = allocationID
+	task.allocationKey = allocationKey
 	task.nodeName = nodeID
 	if task.placeholder {
 		log.Log(log.ShimCacheTask).Info("placeholder is bound",
@@ -286,58 +289,50 @@ func (task *Task) handleSubmitTaskEvent() {
 	log.Log(log.ShimCacheTask).Debug("scheduling pod",
 		zap.String("podName", task.pod.Name))
 
-	// build preemption policy
-	preemptionPolicy := &si.PreemptionPolicy{
-		AllowPreemptSelf:  task.isPreemptSelfAllowed(),
-		AllowPreemptOther: task.isPreemptOtherAllowed(),
-	}
-
-	if utils.PodAlreadyBound(task.pod) {
-		// submit allocation
-		rr := common.CreateAllocationForTask(
-			task.applicationID,
-			task.taskID,
-			task.pod.Spec.NodeName,
-			task.resource,
-			task.placeholder,
-			task.taskGroupName,
-			task.pod,
-			task.originator,
-			preemptionPolicy)
-		log.Log(log.ShimCacheTask).Debug("send update request", zap.Stringer("request", rr))
-		if err := task.context.apiProvider.GetAPIs().SchedulerAPI.UpdateAllocation(rr); err != nil {
-			log.Log(log.ShimCacheTask).Debug("failed to send allocation to scheduler", zap.Error(err))
-			return
-		}
-	} else {
-		// submit allocation ask
-		rr := common.CreateAllocationRequestForTask(
-			task.applicationID,
-			task.taskID,
-			task.resource,
-			task.placeholder,
-			task.taskGroupName,
-			task.pod,
-			task.originator,
-			preemptionPolicy)
-		log.Log(log.ShimCacheTask).Debug("send update request", zap.Stringer("request", rr))
-		if err := task.context.apiProvider.GetAPIs().SchedulerAPI.UpdateAllocation(rr); err != nil {
-			log.Log(log.ShimCacheTask).Debug("failed to send scheduling request to scheduler", zap.Error(err))
-			return
-		}
+	// send update allocation event to core
+	task.updateAllocation()
 
+	if !utils.PodAlreadyBound(task.pod) {
+		// if this is a new request, add events to pod
 		events.GetRecorder().Eventf(task.pod.DeepCopy(), nil, v1.EventTypeNormal, "Scheduling", "Scheduling",
 			"%s is queued and waiting for allocation", task.alias)
 		// if this task belongs to a task group, that means the app has gang scheduling enabled
 		// in this case, post an event to indicate the task is being gang scheduled
 		if !task.placeholder && task.taskGroupName != "" {
 			events.GetRecorder().Eventf(task.pod.DeepCopy(), nil,
-				v1.EventTypeNormal, "GangScheduling", "GangScheduling",
+				v1.EventTypeNormal, "GangScheduling", "TaskGroupMatch",
 				"Pod belongs to the taskGroup %s, it will be scheduled as a gang member", task.taskGroupName)
 		}
 	}
 }
 
+// updateAllocation updates the core scheduler when task information changes.
+// This function must be called with the task lock held.
+func (task *Task) updateAllocation() {
+	// build preemption policy
+	preemptionPolicy := &si.PreemptionPolicy{
+		AllowPreemptSelf:  task.isPreemptSelfAllowed(),
+		AllowPreemptOther: task.isPreemptOtherAllowed(),
+	}
+
+	// submit allocation
+	rr := common.CreateAllocationForTask(
+		task.applicationID,
+		task.taskID,
+		task.pod.Spec.NodeName,
+		task.resource,
+		task.placeholder,
+		task.taskGroupName,
+		task.pod,
+		task.originator,
+		preemptionPolicy)
+	log.Log(log.ShimCacheTask).Debug("send update request", zap.Stringer("request", rr))
+	if err := task.context.apiProvider.GetAPIs().SchedulerAPI.UpdateAllocation(rr); err != nil {
+		log.Log(log.ShimCacheTask).Debug("failed to send allocation to scheduler", zap.Error(err))
+		return
+	}
+}
+
 // this is called after task reaches PENDING state,
 // submit the resource asks from this task to the scheduler core
 func (task *Task) postTaskPending() {
@@ -413,9 +408,9 @@ func (task *Task) postTaskAllocated() {
 // If we find the task is already in Completed state while handling TaskAllocated
 // event, we need to explicitly release this allocation because it is no
 // longer valid.
-func (task *Task) beforeTaskAllocated(eventSrc string, allocationID string, nodeID string) {
-	// task is allocated on a node with a allocationID set the details in the task here to allow referencing later.
-	task.allocationID = allocationID
+func (task *Task) beforeTaskAllocated(eventSrc string, allocationKey string, nodeID string) {
+	// task is allocated on a node with a allocationKey set the details in the task here to allow referencing later.
+	task.allocationKey = allocationKey
 	task.nodeName = nodeID
 	// If the task is Completed the pod was deleted on K8s but the core was not aware yet.
 	// Notify the core to release this allocation to avoid resource leak.
@@ -423,7 +418,7 @@ func (task *Task) beforeTaskAllocated(eventSrc string, allocationID string, node
 	if eventSrc == TaskStates().Completed {
 		log.Log(log.ShimCacheTask).Info("task is already completed, invalidate the allocation",
 			zap.String("currentTaskState", eventSrc),
-			zap.String("allocationID", allocationID),
+			zap.String("allocationKey", allocationKey),
 			zap.String("allocatedNode", nodeID))
 		task.releaseAllocation()
 	}
@@ -497,7 +492,7 @@ func (task *Task) beforeTaskCompleted() {
 		"Task %s is completed", task.alias)
 }
 
-// releaseAllocation sends the release request for the Allocation or the AllocationAsk to the core.
+// releaseAllocation sends the release request for the Allocation to the core.
 func (task *Task) releaseAllocation() {
 	// scheduler api might be nil in some tests
 	if task.context.apiProvider.GetAPIs().SchedulerAPI != nil {
@@ -505,33 +500,37 @@ func (task *Task) releaseAllocation() {
 			zap.String("applicationID", task.applicationID),
 			zap.String("taskID", task.taskID),
 			zap.String("taskAlias", task.alias),
-			zap.String("allocationID", task.allocationID),
+			zap.String("allocationKey", task.allocationKey),
 			zap.String("task", task.GetTaskState()),
 			zap.String("terminationType", task.terminationType))
 
-		// The message depends on current task state, generate requests accordingly.
-		// If allocated send an AllocationReleaseRequest,
-		// If not allocated yet send an AllocationAskReleaseRequest
+		// send an AllocationReleaseRequest
 		var releaseRequest *si.AllocationRequest
 		s := TaskStates()
-		switch task.GetTaskState() {
-		case s.New, s.Pending, s.Scheduling, s.Rejected:
-			releaseRequest = common.CreateReleaseRequestForTask(task.applicationID, task.taskID, task.allocationID, task.application.partition, task.terminationType)
-		default:
-			if task.allocationID == "" {
-				log.Log(log.ShimCacheTask).Warn("BUG: task allocation allocationID is empty on release",
+
+		// check if the task is in a state where it has not been allocated yet
+		if task.GetTaskState() != s.New && task.GetTaskState() != s.Pending &&
+			task.GetTaskState() != s.Scheduling && task.GetTaskState() != s.Rejected {
+			// task is in a state where it might have been allocated
+			if task.allocationKey == "" {
+				log.Log(log.ShimCacheTask).Warn("BUG: task allocationKey is empty on release",
 					zap.String("applicationID", task.applicationID),
 					zap.String("taskID", task.taskID),
 					zap.String("taskAlias", task.alias),
-					zap.String("task", task.GetTaskState()))
+					zap.String("taskState", task.GetTaskState()))
 			}
-			releaseRequest = common.CreateReleaseRequestForTask(
-				task.applicationID, task.taskID, task.allocationID, task.application.partition, task.terminationType)
 		}
 
+		// create the release request
+		releaseRequest = common.CreateReleaseRequestForTask(
+			task.applicationID,
+			task.taskID,
+			task.application.partition,
+			task.terminationType,
+		)
+
 		if releaseRequest.Releases != nil {
 			log.Log(log.ShimCacheTask).Info("releasing allocations",
-				zap.Int("numOfAsksToRelease", len(releaseRequest.Releases.AllocationAsksToRelease)),
 				zap.Int("numOfAllocationsToRelease", len(releaseRequest.Releases.AllocationsToRelease)))
 		}
 		if err := task.context.apiProvider.GetAPIs().SchedulerAPI.UpdateAllocation(releaseRequest); err != nil {
@@ -544,9 +543,36 @@ func (task *Task) releaseAllocation() {
 // this reduces the scheduling overhead by blocking such
 // request away from the core scheduler.
 func (task *Task) sanityCheckBeforeScheduling() error {
+	// After version 1.7.0, we should reject the task whose pod is unbound and has conflicting metadata.
+	if !utils.PodAlreadyBound(task.pod) {
+		if err := utils.CheckAppIdInPod(task.pod); err != nil {
+			log.Log(log.ShimCacheTask).Warn("Pod has inconsistent application metadata and may be rejected in a future YuniKorn release",
+				zap.String("appID", task.applicationID),
+				zap.String("podName", task.pod.Name),
+				zap.String("error", err.Error()))
+
+			events.GetRecorder().Eventf(task.pod.DeepCopy(),
+				nil, v1.EventTypeWarning, "Scheduling", "Scheduling", fmt.Sprintf("Pod has inconsistent application metadata and may be rejected in a future YuniKorn release: %s", err.Error()))
+		}
+		if err := utils.CheckQueueNameInPod(task.pod); err != nil {
+			log.Log(log.ShimCacheTask).Warn("Pod has inconsistent queue metadata and may be rejected in a future YuniKorn release",
+				zap.String("appID", task.applicationID),
+				zap.String("podName", task.pod.Name),
+				zap.String("error", err.Error()))
+
+			events.GetRecorder().Eventf(task.pod.DeepCopy(),
+				nil, v1.EventTypeWarning, "Scheduling", "Scheduling", fmt.Sprintf("Pod has inconsistent queue metadata and may be rejected in a future YuniKorn release: %s", err.Error()))
+		}
+	}
+	return task.checkPodPVCs()
+}
+
+func (task *Task) checkPodPVCs() error {
+	task.lock.RLock()
 	// Check PVCs used by the pod
 	namespace := task.pod.Namespace
 	manifest := &(task.pod.Spec)
+	task.lock.RUnlock()
 	for i := range manifest.Volumes {
 		volume := &manifest.Volumes[i]
 		if volume.PersistentVolumeClaim == nil {
@@ -588,10 +614,22 @@ func (task *Task) UpdatePodCondition(podCondition *v1.PodCondition) (bool, *v1.P
 	return false, pod
 }
 
-func (task *Task) setAllocationID(allocationID string) {
+func (task *Task) GetAllocationKey() string {
+	task.lock.RLock()
+	defer task.lock.RUnlock()
+	return task.allocationKey
+}
+
+func (task *Task) setAllocationKey(allocationKey string) {
 	task.lock.Lock()
 	defer task.lock.Unlock()
-	task.allocationID = allocationID
+	task.allocationKey = allocationKey
+}
+
+func (task *Task) FailWithEvent(errorMessage, actionReason string) {
+	task.lock.RLock()
+	defer task.lock.RUnlock()
+	task.failWithEvent(errorMessage, actionReason)
 }
 
 func (task *Task) failWithEvent(errorMessage, actionReason string) {
@@ -599,3 +637,19 @@ func (task *Task) failWithEvent(errorMessage, actionReason string) {
 	events.GetRecorder().Eventf(task.pod.DeepCopy(),
 		nil, v1.EventTypeWarning, actionReason, actionReason, errorMessage)
 }
+
+func (task *Task) SetTaskPod(pod *v1.Pod) {
+	task.lock.Lock()
+	defer task.lock.Unlock()
+
+	task.pod = pod
+	oldResource := task.resource
+	newResource := common.GetPodResource(pod)
+	if !common.Equals(oldResource, newResource) {
+		// pod resources have changed
+		task.resource = newResource
+
+		// update allocation in core
+		task.updateAllocation()
+	}
+}
