@@ -30,10 +30,10 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	k8sEvents "k8s.io/client-go/tools/events"
 
-	"github.com/apache/yunikorn-core/pkg/common"
 	"github.com/apache/yunikorn-k8shim/pkg/client"
 	"github.com/apache/yunikorn-k8shim/pkg/common/constants"
 	"github.com/apache/yunikorn-k8shim/pkg/common/events"
+	"github.com/apache/yunikorn-k8shim/pkg/common/utils"
 	"github.com/apache/yunikorn-k8shim/pkg/conf"
 	"github.com/apache/yunikorn-k8shim/pkg/locking"
 
@@ -148,6 +148,7 @@ func TestTaskIllegalEventHandling(t *testing.T) {
 	assert.Equal(t, task.GetTaskState(), TaskStates().Pending)
 }
 
+//nolint:funlen
 func TestReleaseTaskAllocation(t *testing.T) {
 	mockedSchedulerApi := newMockSchedulerAPI()
 	mockedContext := initContextForTest()
@@ -170,7 +171,7 @@ func TestReleaseTaskAllocation(t *testing.T) {
 		},
 		ObjectMeta: metav1.ObjectMeta{
 			Name: "pod-resource-test-00001",
-			UID:  "UID-00001",
+			UID:  "task01",
 		},
 		Spec: v1.PodSpec{
 			Containers: containers,
@@ -200,10 +201,14 @@ func TestReleaseTaskAllocation(t *testing.T) {
 	assert.NilError(t, err, "failed to handle AllocateTask event")
 	assert.Equal(t, task.GetTaskState(), TaskStates().Allocated)
 	// bind a task is a async process, wait for it to happen
-	err = common.WaitFor(100*time.Millisecond, 3*time.Second, func() bool {
-		return task.getTaskAllocationID() == string(pod.UID)
-	})
-	assert.NilError(t, err, "failed to wait for allocation allocationID being set for task")
+	err = utils.WaitForCondition(
+		func() bool {
+			return task.GetNodeName() == "node-1"
+		},
+		100*time.Millisecond,
+		3*time.Second,
+	)
+	assert.NilError(t, err, "failed to wait for allocation allocationKey being set for task")
 
 	// bound
 	event3 := NewBindTaskEvent(app.applicationID, task.taskID)
@@ -218,12 +223,7 @@ func TestReleaseTaskAllocation(t *testing.T) {
 		assert.Assert(t, request.Releases.AllocationsToRelease != nil)
 		assert.Equal(t, request.Releases.AllocationsToRelease[0].ApplicationID, app.applicationID)
 		assert.Equal(t, request.Releases.AllocationsToRelease[0].PartitionName, "default")
-		assert.Equal(t, request.Releases.AllocationsToRelease[0].AllocationID, "UID-00001")
-		assert.Assert(t, request.Releases.AllocationAsksToRelease != nil)
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].ApplicationID, app.applicationID)
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].AllocationKey, "task01")
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].PartitionName, "default")
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].TerminationType, si.TerminationType_UNKNOWN_TERMINATION_TYPE)
+		assert.Equal(t, request.Releases.AllocationsToRelease[0].AllocationKey, "task01")
 		return nil
 	})
 
@@ -235,36 +235,31 @@ func TestReleaseTaskAllocation(t *testing.T) {
 	// 2 updates call, 1 for submit, 1 for release
 	assert.Equal(t, mockedApiProvider.GetSchedulerAPIUpdateAllocationCount(), int32(2))
 
-	// New to Failed, no AllocationID is set (only ask is released)
+	// New to Failed, no AllocationKey is set (only ask is released)
 	task = NewTask("task01", app, mockedContext, pod)
 	mockedApiProvider.MockSchedulerAPIUpdateAllocationFn(func(request *si.AllocationRequest) error {
 		assert.Assert(t, request.Releases != nil)
-		assert.Assert(t, request.Releases.AllocationsToRelease == nil)
-		assert.Assert(t, request.Releases.AllocationAsksToRelease != nil)
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].ApplicationID, app.applicationID)
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].AllocationKey, "task01")
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].PartitionName, "default")
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].TerminationType, si.TerminationType_UNKNOWN_TERMINATION_TYPE)
+		assert.Assert(t, request.Releases.AllocationsToRelease != nil)
+		assert.Equal(t, request.Releases.AllocationsToRelease[0].ApplicationID, app.applicationID)
+		assert.Equal(t, request.Releases.AllocationsToRelease[0].AllocationKey, "task01")
+		assert.Equal(t, request.Releases.AllocationsToRelease[0].PartitionName, "default")
+		assert.Equal(t, request.Releases.AllocationsToRelease[0].TerminationType, si.TerminationType_STOPPED_BY_RM)
 		return nil
 	})
 	err = task.handle(NewFailTaskEvent(app.applicationID, "task01", "test failure"))
 	assert.NilError(t, err, "failed to handle FailTask event")
 
-	// Scheduling to Failed, AllocationID is set (ask+allocation are both released)
+	// Scheduling to Failed, AllocationKey is set (ask+allocation are both released)
 	task = NewTask("task01", app, mockedContext, pod)
-	task.setAllocationID("alloc-0")
+	task.setAllocationKey("task01")
 	task.sm.SetState(TaskStates().Scheduling)
 	mockedApiProvider.MockSchedulerAPIUpdateAllocationFn(func(request *si.AllocationRequest) error {
 		assert.Assert(t, request.Releases != nil)
 		assert.Assert(t, request.Releases.AllocationsToRelease != nil)
 		assert.Equal(t, request.Releases.AllocationsToRelease[0].ApplicationID, app.applicationID)
 		assert.Equal(t, request.Releases.AllocationsToRelease[0].PartitionName, "default")
-		assert.Equal(t, request.Releases.AllocationsToRelease[0].AllocationID, "alloc-0")
-		assert.Assert(t, request.Releases.AllocationAsksToRelease != nil)
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].ApplicationID, app.applicationID)
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].AllocationKey, "task01")
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].PartitionName, "default")
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].TerminationType, si.TerminationType_UNKNOWN_TERMINATION_TYPE)
+		assert.Equal(t, request.Releases.AllocationsToRelease[0].AllocationKey, "task01")
+		assert.Equal(t, request.Releases.AllocationsToRelease[0].TerminationType, si.TerminationType_STOPPED_BY_RM)
 		return nil
 	})
 	err = task.handle(NewFailTaskEvent(app.applicationID, "task01", "test failure"))
@@ -325,11 +320,10 @@ func TestReleaseTaskAsk(t *testing.T) {
 	// this is to verify we are sending correct info to the scheduler core
 	mockedApiProvider.MockSchedulerAPIUpdateAllocationFn(func(request *si.AllocationRequest) error {
 		assert.Assert(t, request.Releases != nil)
-		assert.Assert(t, request.Releases.AllocationsToRelease == nil)
-		assert.Assert(t, request.Releases.AllocationAsksToRelease != nil)
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].ApplicationID, app.applicationID)
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].PartitionName, "default")
-		assert.Equal(t, request.Releases.AllocationAsksToRelease[0].AllocationKey, task.taskID)
+		assert.Assert(t, request.Releases.AllocationsToRelease != nil)
+		assert.Equal(t, request.Releases.AllocationsToRelease[0].ApplicationID, app.applicationID)
+		assert.Equal(t, request.Releases.AllocationsToRelease[0].PartitionName, "default")
+		assert.Equal(t, request.Releases.AllocationsToRelease[0].AllocationKey, task.taskID)
 		return nil
 	})
 
@@ -487,9 +481,10 @@ func TestSetTaskGroup(t *testing.T) {
 	}
 	task := NewTask("task01", app, mockedContext, pod)
 	task.setTaskGroupName("test-group")
-	assert.Equal(t, task.getTaskGroupName(), "test-group")
+	assert.Equal(t, task.GetTaskGroupName(), "test-group")
 }
 
+//nolint:funlen
 func TestHandleSubmitTaskEvent(t *testing.T) {
 	mockedContext, mockedSchedulerAPI := initContextAndAPIProviderForTest()
 	var allocRequest *si.AllocationRequest
@@ -592,11 +587,11 @@ func TestHandleSubmitTaskEvent(t *testing.T) {
 	assert.Equal(t, task1.GetTaskState(), TaskStates().Scheduling)
 	assert.Equal(t, rt.time, int64(1))
 	assert.Assert(t, allocRequest != nil)
-	assert.Equal(t, len(allocRequest.Asks), 1)
-	assert.Equal(t, allocRequest.Asks[0].Priority, int32(1000))
-	assert.Assert(t, allocRequest.Asks[0].PreemptionPolicy != nil)
-	assert.Assert(t, allocRequest.Asks[0].PreemptionPolicy.AllowPreemptSelf)
-	assert.Assert(t, !allocRequest.Asks[0].PreemptionPolicy.AllowPreemptOther)
+	assert.Equal(t, len(allocRequest.Allocations), 1)
+	assert.Equal(t, allocRequest.Allocations[0].Priority, int32(1000))
+	assert.Assert(t, allocRequest.Allocations[0].PreemptionPolicy != nil)
+	assert.Assert(t, allocRequest.Allocations[0].PreemptionPolicy.AllowPreemptSelf)
+	assert.Assert(t, !allocRequest.Allocations[0].PreemptionPolicy.AllowPreemptOther)
 	allocRequest = nil
 	rt.time = 0
 	// pod with taskGroup name
@@ -606,11 +601,11 @@ func TestHandleSubmitTaskEvent(t *testing.T) {
 	assert.Equal(t, task2.GetTaskState(), TaskStates().Scheduling)
 	assert.Equal(t, rt.time, int64(2))
 	assert.Assert(t, allocRequest != nil)
-	assert.Equal(t, len(allocRequest.Asks), 1)
-	assert.Equal(t, allocRequest.Asks[0].Priority, int32(1001))
-	assert.Assert(t, allocRequest.Asks[0].PreemptionPolicy != nil)
-	assert.Assert(t, !allocRequest.Asks[0].PreemptionPolicy.AllowPreemptSelf)
-	assert.Assert(t, allocRequest.Asks[0].PreemptionPolicy.AllowPreemptOther)
+	assert.Equal(t, len(allocRequest.Allocations), 1)
+	assert.Equal(t, allocRequest.Allocations[0].Priority, int32(1001))
+	assert.Assert(t, allocRequest.Allocations[0].PreemptionPolicy != nil)
+	assert.Assert(t, !allocRequest.Allocations[0].PreemptionPolicy.AllowPreemptSelf)
+	assert.Assert(t, allocRequest.Allocations[0].PreemptionPolicy.AllowPreemptOther)
 
 	// Test over, set Recorder back fake type
 	events.SetRecorder(k8sEvents.NewFakeRecorder(1024))
@@ -618,10 +613,9 @@ func TestHandleSubmitTaskEvent(t *testing.T) {
 
 func TestSimultaneousTaskCompleteAndAllocate(t *testing.T) {
 	const (
-		podUID       = "UID-00001"
-		appID        = "app-test-001"
-		queueName    = "root.abc"
-		allocationID = "allocationid-xyz"
+		podUID    = "UID-00001"
+		appID     = "app-test-001"
+		queueName = "root.abc"
 	)
 	mockedContext := initContextForTest()
 	mockedAPIProvider, ok := mockedContext.apiProvider.(*client.MockedAPIProvider)
@@ -660,11 +654,9 @@ func TestSimultaneousTaskCompleteAndAllocate(t *testing.T) {
 	// because the task is in Scheduling state,
 	// here we expect to trigger a UpdateRequest that contains a releaseAllocationAsk request
 	mockedAPIProvider.MockSchedulerAPIUpdateAllocationFn(func(request *si.AllocationRequest) error {
-		assert.Equal(t, len(request.Releases.AllocationAsksToRelease), 1,
-			"allocationAskToRelease is not in the expected length")
-		assert.Equal(t, len(request.Releases.AllocationsToRelease), 0,
+		assert.Equal(t, len(request.Releases.AllocationsToRelease), 1,
 			"allocationsToRelease is not in the expected length")
-		askToRelease := request.Releases.AllocationAsksToRelease[0]
+		askToRelease := request.Releases.AllocationsToRelease[0]
 		assert.Equal(t, askToRelease.ApplicationID, appID)
 		assert.Equal(t, askToRelease.AllocationKey, podUID)
 		return nil
@@ -679,22 +671,19 @@ func TestSimultaneousTaskCompleteAndAllocate(t *testing.T) {
 	// can be released from the core to avoid resource leak
 	alloc := &si.Allocation{
 		AllocationKey: string(pod1.UID),
-		AllocationID:  allocationID,
 		NodeID:        "fake-node",
 		ApplicationID: appID,
 		PartitionName: "default",
 	}
 	mockedAPIProvider.MockSchedulerAPIUpdateAllocationFn(func(request *si.AllocationRequest) error {
-		assert.Equal(t, len(request.Releases.AllocationAsksToRelease), 1,
-			"allocationAskToRelease is not in the expected length")
 		assert.Equal(t, len(request.Releases.AllocationsToRelease), 1,
 			"allocationsToRelease is not in the expected length")
 		allocToRelease := request.Releases.AllocationsToRelease[0]
 		assert.Equal(t, allocToRelease.ApplicationID, alloc.ApplicationID)
-		assert.Equal(t, allocToRelease.AllocationID, alloc.AllocationID)
+		assert.Equal(t, allocToRelease.AllocationKey, alloc.AllocationKey)
 		return nil
 	})
-	ev1 := NewAllocateTaskEvent(app.GetApplicationID(), alloc.AllocationKey, alloc.AllocationID, alloc.NodeID)
+	ev1 := NewAllocateTaskEvent(app.GetApplicationID(), alloc.AllocationKey, alloc.AllocationKey, alloc.NodeID)
 	err = task1.handle(ev1)
 	assert.NilError(t, err, "failed to handle AllocateTask event")
 	assert.Equal(t, task1.GetTaskState(), TaskStates().Completed)
