@@ -19,6 +19,7 @@
 package client
 
 import (
+	"errors"
 	"time"
 
 	"go.uber.org/zap"
@@ -35,16 +36,24 @@ import (
 
 type Type int
 
-var informerTypes = [...]string{"Pod", "Node", "ConfigMap", "Storage", "PV", "PVC", "PriorityClass"}
+var informerTypes = [...]string{"Pod", "Node", "ConfigMap", "PV", "PVC", "Storage", "CSINode", "CSIDriver", "CSIStorageCapacity", "Namespace", "PriorityClass", "Service", "ReplicationController", "ReplicaSet", "StatefulSet"}
 
 const (
 	PodInformerHandlers Type = iota
 	NodeInformerHandlers
 	ConfigMapInformerHandlers
-	StorageInformerHandlers
 	PVInformerHandlers
 	PVCInformerHandlers
+	StorageInformerHandlers
+	CSINodeInformerHandlers
+	CSIDriverInformerHandlers
+	CSIStorageCapacityInformerHandlers
+	NamespaceInformerHandlers
 	PriorityClassInformerHandlers
+	ServiceInformerHandlers
+	ReplicationControllerInformerHandlers
+	ReplicaSetInformerHandlers
+	StatefulSetInformerHandlers
 )
 
 func (t Type) String() string {
@@ -53,7 +62,7 @@ func (t Type) String() string {
 
 type APIProvider interface {
 	GetAPIs() *Clients
-	AddEventHandler(handlers *ResourceEventHandlers)
+	AddEventHandler(handlers *ResourceEventHandlers) error
 	Start()
 	Stop()
 	WaitForSync()
@@ -84,15 +93,21 @@ func NewAPIFactory(scheduler api.SchedulerAPI, informerFactory informers.SharedI
 
 	// init informers
 	// volume informers are also used to get the Listers for the predicates
-	nodeInformer := informerFactory.Core().V1().Nodes()
 	podInformer := informerFactory.Core().V1().Pods()
+	nodeInformer := informerFactory.Core().V1().Nodes()
 	configMapInformer := informerFactory.Core().V1().ConfigMaps()
-	storageInformer := informerFactory.Storage().V1().StorageClasses()
-	csiNodeInformer := informerFactory.Storage().V1().CSINodes()
 	pvInformer := informerFactory.Core().V1().PersistentVolumes()
 	pvcInformer := informerFactory.Core().V1().PersistentVolumeClaims()
+	storageInformer := informerFactory.Storage().V1().StorageClasses()
+	csiNodeInformer := informerFactory.Storage().V1().CSINodes()
+	csiDriverInformer := informerFactory.Storage().V1().CSIDrivers()
+	csiStorageCapacityInformer := informerFactory.Storage().V1().CSIStorageCapacities()
 	namespaceInformer := informerFactory.Core().V1().Namespaces()
 	priorityClassInformer := informerFactory.Scheduling().V1().PriorityClasses()
+	serviceInformer := informerFactory.Core().V1().Services()
+	replicationControllerInformer := informerFactory.Core().V1().ReplicationControllers()
+	replicaSetInformer := informerFactory.Apps().V1().ReplicaSets()
+	statefulSetInformer := informerFactory.Apps().V1().StatefulSets()
 
 	var capacityCheck = volumebinding.CapacityCheck{
 		CSIDriverInformer:          informerFactory.Storage().V1().CSIDrivers(),
@@ -114,20 +129,25 @@ func NewAPIFactory(scheduler api.SchedulerAPI, informerFactory informers.SharedI
 
 	return &APIFactory{
 		clients: &Clients{
-			conf:                  configs,
-			KubeClient:            kubeClient,
-			SchedulerAPI:          scheduler,
-			InformerFactory:       informerFactory,
-			PodInformer:           podInformer,
-			NodeInformer:          nodeInformer,
-			ConfigMapInformer:     configMapInformer,
-			PVInformer:            pvInformer,
-			PVCInformer:           pvcInformer,
-			NamespaceInformer:     namespaceInformer,
-			StorageInformer:       storageInformer,
-			CSINodeInformer:       csiNodeInformer,
-			PriorityClassInformer: priorityClassInformer,
-			VolumeBinder:          volumeBinder,
+			KubeClient:                    kubeClient,
+			SchedulerAPI:                  scheduler,
+			InformerFactory:               informerFactory,
+			PodInformer:                   podInformer,
+			NodeInformer:                  nodeInformer,
+			ConfigMapInformer:             configMapInformer,
+			PVInformer:                    pvInformer,
+			PVCInformer:                   pvcInformer,
+			StorageInformer:               storageInformer,
+			CSINodeInformer:               csiNodeInformer,
+			CSIDriverInformer:             csiDriverInformer,
+			CSIStorageCapacityInformer:    csiStorageCapacityInformer,
+			NamespaceInformer:             namespaceInformer,
+			PriorityClassInformer:         priorityClassInformer,
+			ServiceInformer:               serviceInformer,
+			ReplicationControllerInformer: replicationControllerInformer,
+			ReplicaSetInformer:            replicaSetInformer,
+			StatefulSetInformer:           statefulSetInformer,
+			VolumeBinder:                  volumeBinder,
 		},
 		testMode: testMode,
 		stopChan: make(chan struct{}),
@@ -143,7 +163,7 @@ func (s *APIFactory) IsTestingMode() bool {
 	return s.testMode
 }
 
-func (s *APIFactory) AddEventHandler(handlers *ResourceEventHandlers) {
+func (s *APIFactory) AddEventHandler(handlers *ResourceEventHandlers) error {
 	s.lock.Lock()
 	defer s.lock.Unlock()
 	// register all handlers
@@ -166,34 +186,43 @@ func (s *APIFactory) AddEventHandler(handlers *ResourceEventHandlers) {
 	}
 
 	log.Log(log.ShimClient).Info("registering event handler", zap.Stringer("type", handlers.Type))
-	s.addEventHandlers(handlers.Type, h, 0)
+	if err := s.addEventHandlers(handlers.Type, h, 0); err != nil {
+		return errors.Join(errors.New("failed to initialize event handlers: "), err)
+	}
+	return nil
 }
 
 func (s *APIFactory) addEventHandlers(
-	handlerType Type, handler cache.ResourceEventHandler, resyncPeriod time.Duration) {
+	handlerType Type, handler cache.ResourceEventHandler, resyncPeriod time.Duration) error {
+	var err error
 	switch handlerType {
 	case PodInformerHandlers:
-		s.GetAPIs().PodInformer.Informer().
+		_, err = s.GetAPIs().PodInformer.Informer().
 			AddEventHandlerWithResyncPeriod(handler, resyncPeriod)
 	case NodeInformerHandlers:
-		s.GetAPIs().NodeInformer.Informer().
+		_, err = s.GetAPIs().NodeInformer.Informer().
 			AddEventHandlerWithResyncPeriod(handler, resyncPeriod)
 	case ConfigMapInformerHandlers:
-		s.GetAPIs().ConfigMapInformer.Informer().
+		_, err = s.GetAPIs().ConfigMapInformer.Informer().
 			AddEventHandlerWithResyncPeriod(handler, resyncPeriod)
 	case StorageInformerHandlers:
-		s.GetAPIs().StorageInformer.Informer().
+		_, err = s.GetAPIs().StorageInformer.Informer().
 			AddEventHandlerWithResyncPeriod(handler, resyncPeriod)
 	case PVInformerHandlers:
-		s.GetAPIs().PVInformer.Informer().
+		_, err = s.GetAPIs().PVInformer.Informer().
 			AddEventHandlerWithResyncPeriod(handler, resyncPeriod)
 	case PVCInformerHandlers:
-		s.GetAPIs().PVCInformer.Informer().
+		_, err = s.GetAPIs().PVCInformer.Informer().
 			AddEventHandlerWithResyncPeriod(handler, resyncPeriod)
 	case PriorityClassInformerHandlers:
-		s.GetAPIs().PriorityClassInformer.Informer().
+		_, err = s.GetAPIs().PriorityClassInformer.Informer().
 			AddEventHandlerWithResyncPeriod(handler, resyncPeriod)
 	}
+
+	if err != nil {
+		return errors.Join(errors.New("failed to add event handlers: "), err)
+	}
+	return nil
 }
 
 func (s *APIFactory) WaitForSync() {
