@@ -21,7 +21,9 @@ package utils
 import (
 	"bytes"
 	"compress/gzip"
+	"errors"
 	"fmt"
+	"reflect"
 	"strings"
 	"testing"
 	"time"
@@ -49,6 +51,21 @@ func TestConvert2Pod(t *testing.T) {
 	assert.Assert(t, pod != nil)
 }
 
+func TestConvert2ConfigMap(t *testing.T) {
+	configMap := &v1.ConfigMap{}
+	result := Convert2ConfigMap(configMap)
+	assert.Equal(t, result != nil, true)
+	assert.Equal(t, reflect.DeepEqual(result, configMap), true)
+
+	obj := struct{}{}
+	result = Convert2ConfigMap(obj)
+	assert.Equal(t, result == nil, true)
+
+	pod := &v1.Pod{}
+	result = Convert2ConfigMap(pod)
+	assert.Equal(t, result == nil, true)
+}
+
 func TestIsAssignedPod(t *testing.T) {
 	assigned := IsAssignedPod(&v1.Pod{
 		Spec: v1.PodSpec{
@@ -212,6 +229,15 @@ func TestGetNamespaceQuotaFromAnnotationUsingNewAnnotations(t *testing.T) {
 				},
 			},
 		}, nil},
+		{&v1.Namespace{
+			ObjectMeta: metav1.ObjectMeta{
+				Name:      "test",
+				Namespace: "test",
+				Annotations: map[string]string{
+					constants.DomainYuniKorn + "namespace.quota": "expecting JSON object",
+				},
+			},
+		}, nil},
 	}
 	for _, tc := range testCases {
 		t.Run(fmt.Sprintf("namespace: %v", tc.namespace), func(t *testing.T) {
@@ -306,6 +332,53 @@ func TestGetNamespaceGuaranteedFromAnnotation(t *testing.T) {
 	}
 }
 
+func TestGetNamespaceMaxAppsFromAnnotation(t *testing.T) {
+	testCases := []struct {
+		namespace      *v1.Namespace
+		expectedMaxApp string
+	}{
+		{&v1.Namespace{
+			ObjectMeta: metav1.ObjectMeta{
+				Name:      "test",
+				Namespace: "test",
+			},
+		}, ""},
+		{&v1.Namespace{
+			ObjectMeta: metav1.ObjectMeta{
+				Name:      "test",
+				Namespace: "test",
+				Annotations: map[string]string{
+					constants.NamespaceMaxApps: "5",
+				},
+			},
+		}, "5"},
+		{&v1.Namespace{
+			ObjectMeta: metav1.ObjectMeta{
+				Name:      "test",
+				Namespace: "test",
+				Annotations: map[string]string{
+					constants.NamespaceMaxApps: "-5",
+				},
+			},
+		}, ""},
+		{&v1.Namespace{
+			ObjectMeta: metav1.ObjectMeta{
+				Name:      "test",
+				Namespace: "test",
+				Annotations: map[string]string{
+					constants.NamespaceMaxApps: "error",
+				},
+			},
+		}, ""},
+	}
+	for _, tc := range testCases {
+		t.Run(fmt.Sprintf("namespace: %v", tc.namespace), func(t *testing.T) {
+			maxApp := GetNamespaceMaxAppsFromAnnotation(tc.namespace)
+			assert.Equal(t, maxApp, tc.expectedMaxApp)
+		})
+	}
+}
+
 func TestGetNamespaceQuotaFromAnnotationUsingNewAndOldAnnotations(t *testing.T) {
 	testCases := []struct {
 		namespace        *v1.Namespace
@@ -510,14 +583,15 @@ func TestPodUnderCondition(t *testing.T) {
 	assert.Equal(t, PodUnderCondition(pod, condition), false)
 }
 
+// nolint: funlen
 func TestGetApplicationIDFromPod(t *testing.T) {
 	defer SetPluginMode(false)
 	defer func() { conf.GetSchedulerConf().GenerateUniqueAppIds = false }()
 
-	appIDInLabel := "labelAppID"
+	appIDInCanonicalLabel := "CanonicalLabelAppID"
 	appIDInAnnotation := "annotationAppID"
-	appIDInSelector := "selectorAppID"
-	sparkIDInAnnotation := "sparkAnnotationAppID"
+	appIDInLabel := "labelAppID"
+	appIDInSelector := "sparkLabelAppID"
 	testCases := []struct {
 		name                    string
 		pod                     *v1.Pod
@@ -525,13 +599,55 @@ func TestGetApplicationIDFromPod(t *testing.T) {
 		expectedAppIDPluginMode string
 		generateUniqueAppIds    bool
 	}{
-		{"AppID defined in label", &v1.Pod{
+		{"AppID defined in canonical label", &v1.Pod{
+			ObjectMeta: metav1.ObjectMeta{
+				Labels: map[string]string{constants.CanonicalLabelApplicationID: appIDInCanonicalLabel},
+			},
+			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
+		}, appIDInCanonicalLabel, appIDInCanonicalLabel, false},
+		{"AppID defined in annotation", &v1.Pod{
+			ObjectMeta: metav1.ObjectMeta{
+				Annotations: map[string]string{constants.AnnotationApplicationID: appIDInAnnotation},
+			},
+			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
+		}, appIDInAnnotation, appIDInAnnotation, false},
+		{"AppID defined in legacy label", &v1.Pod{
 			ObjectMeta: metav1.ObjectMeta{
 				Labels: map[string]string{constants.LabelApplicationID: appIDInLabel},
 			},
 			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
 		}, appIDInLabel, appIDInLabel, false},
-		{"No AppID defined", &v1.Pod{
+		{"AppID defined in spark app selector label", &v1.Pod{
+			ObjectMeta: metav1.ObjectMeta{
+				Labels: map[string]string{constants.SparkLabelAppID: appIDInSelector},
+			},
+			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
+		}, appIDInSelector, appIDInSelector, false},
+		{"AppID defined in canonical label and annotation, canonical label win", &v1.Pod{
+			ObjectMeta: metav1.ObjectMeta{
+				Annotations: map[string]string{constants.AnnotationApplicationID: appIDInAnnotation},
+				Labels:      map[string]string{constants.CanonicalLabelApplicationID: appIDInCanonicalLabel},
+			},
+			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
+		}, appIDInCanonicalLabel, appIDInCanonicalLabel, false},
+		{"AppID defined in annotation and legacy label, annotation win", &v1.Pod{
+			ObjectMeta: metav1.ObjectMeta{
+				Annotations: map[string]string{constants.AnnotationApplicationID: appIDInAnnotation},
+				Labels:      map[string]string{constants.LabelApplicationID: appIDInLabel},
+			},
+			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
+		}, appIDInAnnotation, appIDInAnnotation, false},
+		{"Spark AppID defined in legacy label and spark app selector, legacy label win", &v1.Pod{
+			ObjectMeta: metav1.ObjectMeta{
+				Labels: map[string]string{
+					constants.LabelApplicationID: appIDInLabel,
+					constants.SparkLabelAppID:    appIDInSelector,
+				},
+			},
+			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
+		}, appIDInLabel, appIDInLabel, false},
+		{"No AppID defined", &v1.Pod{}, "", "", false},
+		{"No AppID defined but not generateUnique", &v1.Pod{
 			ObjectMeta: metav1.ObjectMeta{
 				Namespace: "testns",
 				UID:       "podUid",
@@ -545,7 +661,15 @@ func TestGetApplicationIDFromPod(t *testing.T) {
 			},
 			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
 		}, "testns-podUid", "", true},
-		{"Unique autogen token found with generateUnique", &v1.Pod{
+		{"Unique autogen token found with generateUnique in canonical AppId label", &v1.Pod{
+			ObjectMeta: metav1.ObjectMeta{
+				Namespace: "testns",
+				UID:       "podUid",
+				Labels:    map[string]string{constants.CanonicalLabelApplicationID: "testns-uniqueautogen"},
+			},
+			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
+		}, "testns-podUid", "testns-podUid", true},
+		{"Unique autogen token found with generateUnique in legacy AppId labels", &v1.Pod{
 			ObjectMeta: metav1.ObjectMeta{
 				Namespace: "testns",
 				UID:       "podUid",
@@ -553,18 +677,18 @@ func TestGetApplicationIDFromPod(t *testing.T) {
 			},
 			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
 		}, "testns-podUid", "testns-podUid", true},
-		{"Non-yunikorn schedulerName", &v1.Pod{
+		{"Non-yunikorn schedulerName with canonical AppId label", &v1.Pod{
+			ObjectMeta: metav1.ObjectMeta{
+				Labels: map[string]string{constants.CanonicalLabelApplicationID: appIDInCanonicalLabel},
+			},
+			Spec: v1.PodSpec{SchedulerName: "default"},
+		}, "", "", false},
+		{"Non-yunikorn schedulerName with legacy AppId label", &v1.Pod{
 			ObjectMeta: metav1.ObjectMeta{
 				Labels: map[string]string{constants.LabelApplicationID: appIDInLabel},
 			},
 			Spec: v1.PodSpec{SchedulerName: "default"},
 		}, "", "", false},
-		{"AppID defined in annotation", &v1.Pod{
-			ObjectMeta: metav1.ObjectMeta{
-				Annotations: map[string]string{constants.AnnotationApplicationID: appIDInAnnotation},
-			},
-			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
-		}, appIDInAnnotation, appIDInAnnotation, false},
 		{"AppID defined but ignore-application set", &v1.Pod{
 			ObjectMeta: metav1.ObjectMeta{
 				Annotations: map[string]string{
@@ -583,56 +707,226 @@ func TestGetApplicationIDFromPod(t *testing.T) {
 			},
 			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
 		}, appIDInAnnotation, appIDInAnnotation, false},
-		{"AppID defined in label and annotation", &v1.Pod{
-			ObjectMeta: metav1.ObjectMeta{
-				Annotations: map[string]string{constants.AnnotationApplicationID: appIDInAnnotation},
-				Labels:      map[string]string{constants.LabelApplicationID: appIDInLabel},
-			},
-			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
-		}, appIDInAnnotation, appIDInAnnotation, false},
-
-		{"Spark AppID defined in spark app selector", &v1.Pod{
-			ObjectMeta: metav1.ObjectMeta{
-				Labels: map[string]string{constants.SparkLabelAppID: appIDInSelector},
-			},
-			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
-		}, appIDInSelector, appIDInSelector, false},
-		{"Spark AppID defined in spark app selector and annotation", &v1.Pod{
-			ObjectMeta: metav1.ObjectMeta{
-				Labels:      map[string]string{constants.SparkLabelAppID: appIDInSelector},
-				Annotations: map[string]string{constants.AnnotationApplicationID: sparkIDInAnnotation},
-			},
-			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
-		}, sparkIDInAnnotation, sparkIDInAnnotation, false},
-		{"Spark AppID defined in spark app selector and annotation", &v1.Pod{
-			ObjectMeta: metav1.ObjectMeta{
-				Labels:      map[string]string{constants.SparkLabelAppID: appIDInSelector, constants.LabelApplicationID: appIDInLabel},
-				Annotations: map[string]string{constants.AnnotationApplicationID: sparkIDInAnnotation},
-			},
-			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
-		}, sparkIDInAnnotation, sparkIDInAnnotation, false},
-		{"No AppID defined", &v1.Pod{}, "", "", false},
-		{"Spark AppID defined in spark app selector and label", &v1.Pod{
-			ObjectMeta: metav1.ObjectMeta{
-				Labels: map[string]string{constants.SparkLabelAppID: appIDInSelector, constants.LabelApplicationID: appIDInLabel},
-			},
-			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
-		}, appIDInLabel, appIDInLabel, false},
 	}
 
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
 			conf.GetSchedulerConf().GenerateUniqueAppIds = tc.generateUniqueAppIds
 			SetPluginMode(false)
+			assert.Equal(t, IsPluginMode(), false)
 			appID := GetApplicationIDFromPod(tc.pod)
 			assert.Equal(t, appID, tc.expectedAppID, "Wrong appID (standard mode)")
 			SetPluginMode(true)
+			assert.Equal(t, IsPluginMode(), true)
 			appID2 := GetApplicationIDFromPod(tc.pod)
 			assert.Equal(t, appID2, tc.expectedAppIDPluginMode, "Wrong appID (plugin mode)")
 		})
 	}
 }
 
+func TestCheckAppIdInPod(t *testing.T) {
+	testCases := []struct {
+		name     string
+		pod      *v1.Pod
+		expected error
+	}{
+		{
+			name: "consistent app ID",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Labels: map[string]string{
+						constants.CanonicalLabelApplicationID: "app-123",
+						constants.SparkLabelAppID:             "app-123",
+						constants.LabelApplicationID:          "app-123",
+					},
+					Annotations: map[string]string{
+						constants.AnnotationApplicationID: "app-123",
+					},
+				},
+			},
+			expected: nil,
+		},
+		{
+			name: "inconsistent app ID in labels",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Labels: map[string]string{
+						constants.CanonicalLabelApplicationID: "app-123",
+						constants.SparkLabelAppID:             "app-456",
+					},
+				},
+			},
+			expected: errors.New("label spark-app-selector: \"app-456\" doesn't match label yunikorn.apache.org/app-id: \"app-123\""),
+		},
+		{
+			name: "inconsistent app ID between label and annotation",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Labels: map[string]string{
+						constants.CanonicalLabelApplicationID: "app-123",
+					},
+					Annotations: map[string]string{
+						constants.AnnotationApplicationID: "app-456",
+					},
+				},
+			},
+			expected: errors.New("annotation yunikorn.apache.org/app-id: \"app-456\" doesn't match label yunikorn.apache.org/app-id: \"app-123\""),
+		},
+	}
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			err := CheckAppIdInPod(tc.pod)
+			if tc.expected != nil {
+				assert.ErrorContains(t, err, tc.expected.Error())
+			} else {
+				assert.NilError(t, err)
+			}
+		})
+	}
+}
+
+func TestCheckQueueNameInPod(t *testing.T) {
+	testCases := []struct {
+		name     string
+		pod      *v1.Pod
+		expected error
+	}{
+		{
+			name: "consistent queue name",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Labels: map[string]string{
+						constants.CanonicalLabelQueueName: "root.a",
+						constants.LabelQueueName:          "root.a",
+					},
+					Annotations: map[string]string{
+						constants.AnnotationQueueName: "root.a",
+					},
+				},
+			},
+			expected: nil,
+		},
+		{
+			name: "inconsistent app ID in labels",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Labels: map[string]string{
+						constants.CanonicalLabelQueueName: "root.a",
+						constants.LabelQueueName:          "root.b",
+					},
+				},
+			},
+			expected: errors.New("label queue: \"root.b\" doesn't match label yunikorn.apache.org/queue: \"root.a\""),
+		},
+		{
+			name: "inconsistent app ID between label and annotation",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Labels: map[string]string{
+						constants.CanonicalLabelQueueName: "root.a",
+					},
+					Annotations: map[string]string{
+						constants.AnnotationQueueName: "root.b",
+					},
+				},
+			},
+			expected: errors.New("annotation yunikorn.apache.org/queue: \"root.b\" doesn't match label yunikorn.apache.org/queue: \"root.a\""),
+		},
+	}
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			err := CheckQueueNameInPod(tc.pod)
+			if tc.expected != nil {
+				assert.ErrorContains(t, err, tc.expected.Error())
+			} else {
+				assert.NilError(t, err)
+			}
+		})
+	}
+}
+
+func TestValidatePodLabelAnnotation(t *testing.T) {
+	labelKeys := []string{"labelKey1", "labelKey2"}
+	annotationKeys := []string{"annotationKey1", "annotationKey2"}
+
+	testCases := []struct {
+		name     string
+		pod      *v1.Pod
+		expected error
+	}{
+		{
+			name:     "empty pod",
+			pod:      &v1.Pod{},
+			expected: nil,
+		},
+		{
+			name: "pod with all values are consistent",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Labels: map[string]string{
+						"labelKey1": "value1",
+						"labelKey2": "value1",
+					},
+					Annotations: map[string]string{
+						"annotationKey1": "value1",
+						"annotationKey2": "value1",
+					},
+				},
+			},
+			expected: nil,
+		},
+		{
+			name: "pod with inconsistent value in labels",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Labels: map[string]string{
+						"labelKey1": "value1",
+						"labelKey2": "value2",
+					},
+				},
+			},
+			expected: errors.New("label labelKey2: \"value2\" doesn't match label labelKey1: \"value1\""),
+		},
+		{
+			name: "pod with inconsistent value between label and annotation",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Labels: map[string]string{
+						"labelKey1": "value1",
+					},
+					Annotations: map[string]string{
+						"annotationKey1": "value2",
+					},
+				},
+			},
+			expected: errors.New("annotation annotationKey1: \"value2\" doesn't match label labelKey1: \"value1\""),
+		},
+		{
+			name: "pod with inconsistent value in annotations",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Annotations: map[string]string{
+						"annotationKey1": "value1",
+						"annotationKey2": "value2",
+					},
+				},
+			},
+			expected: errors.New("annotation annotationKey2: \"value2\" doesn't match annotation annotationKey1: \"value1\""),
+		},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			err := ValidatePodLabelAnnotation(tc.pod, labelKeys, annotationKeys)
+			if tc.expected != nil {
+				assert.ErrorContains(t, err, tc.expected.Error())
+			} else {
+				assert.NilError(t, err)
+			}
+		})
+	}
+}
+
 func TestGenerateApplicationID(t *testing.T) {
 	assert.Equal(t, "yunikorn-this-is-a-namespace-autogen",
 		GenerateApplicationID("this-is-a-namespace", false, "pod-uid"))
@@ -817,6 +1111,7 @@ func TestGetUserFromPodAnnotation(t *testing.T) {
 }
 
 func TestGetQueueNameFromPod(t *testing.T) {
+	queueInCanonicalLabel := "sandboxCanonicalLabel"
 	queueInLabel := "sandboxLabel"
 	queueInAnnotation := "sandboxAnnotation"
 	testCases := []struct {
@@ -825,16 +1120,16 @@ func TestGetQueueNameFromPod(t *testing.T) {
 		expectedQueue string
 	}{
 		{
-			name: "With queue label",
+			name: "Queue defined in canonical label",
 			pod: &v1.Pod{
 				ObjectMeta: metav1.ObjectMeta{
-					Labels: map[string]string{constants.LabelQueueName: queueInLabel},
+					Labels: map[string]string{constants.CanonicalLabelQueueName: queueInCanonicalLabel},
 				},
 			},
-			expectedQueue: queueInLabel,
+			expectedQueue: queueInCanonicalLabel,
 		},
 		{
-			name: "With queue annotation",
+			name: "Queue defined in annotation",
 			pod: &v1.Pod{
 				ObjectMeta: metav1.ObjectMeta{
 					Annotations: map[string]string{constants.AnnotationQueueName: queueInAnnotation},
@@ -843,17 +1138,36 @@ func TestGetQueueNameFromPod(t *testing.T) {
 			expectedQueue: queueInAnnotation,
 		},
 		{
-			name: "With queue label and annotation",
+			name: "Queue defined in legacy label",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Labels: map[string]string{constants.LabelQueueName: queueInLabel},
+				},
+			},
+			expectedQueue: queueInLabel,
+		},
+		{
+			name: "Queue defined in canonical label and annotation, canonical label win",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Labels:      map[string]string{constants.CanonicalLabelQueueName: queueInCanonicalLabel},
+					Annotations: map[string]string{constants.AnnotationQueueName: queueInAnnotation},
+				},
+			},
+			expectedQueue: queueInCanonicalLabel,
+		},
+		{
+			name: "Queue defined in annotation and legacy label, annotation win",
 			pod: &v1.Pod{
 				ObjectMeta: metav1.ObjectMeta{
 					Labels:      map[string]string{constants.LabelQueueName: queueInLabel},
 					Annotations: map[string]string{constants.AnnotationQueueName: queueInAnnotation},
 				},
 			},
-			expectedQueue: queueInLabel,
+			expectedQueue: queueInAnnotation,
 		},
 		{
-			name: "Without queue label and annotation",
+			name: "No queue defined",
 			pod: &v1.Pod{
 				ObjectMeta: metav1.ObjectMeta{},
 			},
@@ -946,6 +1260,22 @@ func TestPodAlreadyBound(t *testing.T) {
 	}
 }
 
+func TestIsPodRunning(t *testing.T) {
+	pod := &v1.Pod{
+		Status: v1.PodStatus{
+			Phase: v1.PodRunning,
+		},
+	}
+	assert.Equal(t, IsPodRunning(pod), true)
+
+	pod = &v1.Pod{
+		Status: v1.PodStatus{
+			Phase: v1.PodFailed,
+		},
+	}
+	assert.Equal(t, IsPodRunning(pod), false)
+}
+
 func TestGetTaskGroupFromPodSpec(t *testing.T) {
 	pod := &v1.Pod{
 		TypeMeta: metav1.TypeMeta{
@@ -983,7 +1313,6 @@ func TestGetPlaceholderFlagFromPodSpec(t *testing.T) {
 		pod                     *v1.Pod
 		expectedPlaceholderFlag bool
 	}{
-
 		{"Setting by annotation", &v1.Pod{
 			TypeMeta: metav1.TypeMeta{
 				Kind:       "Pod",
@@ -997,20 +1326,7 @@ func TestGetPlaceholderFlagFromPodSpec(t *testing.T) {
 				},
 			},
 		}, true},
-		{"Setting by label", &v1.Pod{
-			TypeMeta: metav1.TypeMeta{
-				Kind:       "Pod",
-				APIVersion: "v1",
-			},
-			ObjectMeta: metav1.ObjectMeta{
-				Name: "pod-01",
-				UID:  "UID-01",
-				Labels: map[string]string{
-					constants.LabelPlaceholderFlag: "true",
-				},
-			},
-		}, true},
-		{"Setting both annotation and label, annotation has higher priority", &v1.Pod{
+		{"Setting by deprecated annotation", &v1.Pod{
 			TypeMeta: metav1.TypeMeta{
 				Kind:       "Pod",
 				APIVersion: "v1",
@@ -1019,12 +1335,41 @@ func TestGetPlaceholderFlagFromPodSpec(t *testing.T) {
 				Name: "pod-01",
 				UID:  "UID-01",
 				Annotations: map[string]string{
-					constants.AnnotationPlaceholderFlag: "true",
-					constants.LabelPlaceholderFlag:      "false",
+					constants.OldAnnotationPlaceholderFlag: "true", // nolint:staticcheck
 				},
 			},
 		}, true},
-		{"No setting both annotation and label", &v1.Pod{
+		{"Setting by deprecated label", &v1.Pod{
+			TypeMeta: metav1.TypeMeta{
+				Kind:       "Pod",
+				APIVersion: "v1",
+			},
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "pod-01",
+				UID:  "UID-01",
+				Labels: map[string]string{
+					constants.OldLabelPlaceholderFlag: "true", // nolint:staticcheck
+				},
+			},
+		}, true},
+		{"Set new placeholder annotation and old placeholder label/annotation together, new annotation has higher priority", &v1.Pod{
+			TypeMeta: metav1.TypeMeta{
+				Kind:       "Pod",
+				APIVersion: "v1",
+			},
+			ObjectMeta: metav1.ObjectMeta{
+				Name: "pod-01",
+				UID:  "UID-01",
+				Labels: map[string]string{
+					constants.OldLabelPlaceholderFlag: "false", // nolint:staticcheck
+				},
+				Annotations: map[string]string{
+					constants.AnnotationPlaceholderFlag:    "true",
+					constants.OldAnnotationPlaceholderFlag: "false", // nolint:staticcheck
+				},
+			},
+		}, true},
+		{"Pod without placeholder annotation", &v1.Pod{
 			TypeMeta: metav1.TypeMeta{
 				Kind:       "Pod",
 				APIVersion: "v1",
@@ -1044,20 +1389,25 @@ func TestGetPlaceholderFlagFromPodSpec(t *testing.T) {
 	}
 }
 
-func TestGetCoreSchedulerConfigFromConfigMapNil(t *testing.T) {
-	assert.Equal(t, "", GetCoreSchedulerConfigFromConfigMap(nil))
-}
-
-func TestGetCoreSchedulerConfigFromConfigMapEmpty(t *testing.T) {
-	cm := map[string]string{}
-	assert.Equal(t, "", GetCoreSchedulerConfigFromConfigMap(cm))
-}
-
 func TestGetCoreSchedulerConfigFromConfigMap(t *testing.T) {
+	// case: mapping
 	cm := map[string]string{
 		"queues.yaml": "test",
 	}
 	assert.Equal(t, "test", GetCoreSchedulerConfigFromConfigMap(cm))
+
+	// case: not mapping
+	cm = map[string]string{
+		"unknow.yaml": "test",
+	}
+	assert.Equal(t, "", GetCoreSchedulerConfigFromConfigMap(cm))
+
+	// case: nil
+	assert.Equal(t, "", GetCoreSchedulerConfigFromConfigMap(nil))
+
+	// case: empty
+	cm = map[string]string{}
+	assert.Equal(t, "", GetCoreSchedulerConfigFromConfigMap(cm))
 }
 
 func TestGzipCompressedConfigMap(t *testing.T) {
@@ -1125,3 +1475,30 @@ func TestConvert2PriorityClass(t *testing.T) {
 	assert.Assert(t, result != nil)
 	assert.Equal(t, result.PreemptionPolicy, &preemptLower)
 }
+
+func TestWaitForCondition(t *testing.T) {
+	target := false
+	eval := func() bool {
+		return target
+	}
+	tests := []struct {
+		input    bool
+		interval time.Duration
+		timeout  time.Duration
+		output   error
+	}{
+		{true, time.Duration(1) * time.Second, time.Duration(2) * time.Second, nil},
+		{false, time.Duration(1) * time.Second, time.Duration(2) * time.Second, ErrorTimeout},
+		{true, time.Duration(3) * time.Second, time.Duration(2) * time.Second, nil},
+		{false, time.Duration(3) * time.Second, time.Duration(2) * time.Second, ErrorTimeout},
+	}
+	for _, test := range tests {
+		target = test.input
+		get := WaitForCondition(eval, test.timeout, test.interval)
+		if test.output == nil {
+			assert.NilError(t, get)
+		} else {
+			assert.Equal(t, get.Error(), test.output.Error())
+		}
+	}
+}
