@@ -43,7 +43,7 @@ import (
 type PredicateManager interface {
 	EventsToRegister(queueingHintFn framework.QueueingHintFn) []framework.ClusterEventWithHint
 	Predicates(pod *v1.Pod, node *framework.NodeInfo, allocate bool) (plugin string, error error)
-	PreemptionPredicates(pod *v1.Pod, node *framework.NodeInfo, victims []*v1.Pod, startIndex int) (index int, ok bool)
+	PreemptionPredicates(pod *v1.Pod, node *framework.NodeInfo, victims []*v1.Pod, startIndex int) (index int)
 }
 
 var _ PredicateManager = &predicateManagerImpl{}
@@ -75,7 +75,11 @@ func pluginEvents(plugin framework.Plugin) []framework.ClusterEventWithHint {
 		// legacy plugins that don't register for EnqueueExtensions get a default list of events
 		return framework.UnrollWildCardResource()
 	}
-	return ext.EventsToRegister()
+	events, err := ext.EventsToRegister(context.Background())
+	if err != nil {
+		log.Log(log.ShimPredicates).Fatal("Failed to configure predicate plugin", zap.String("name", ext.Name()), zap.Error(err))
+	}
+	return events
 }
 
 func mergePluginEvents(actionMap map[framework.GVK]framework.ActionType, events []framework.ClusterEventWithHint) {
@@ -125,7 +129,7 @@ func (p *predicateManagerImpl) Predicates(pod *v1.Pod, node *framework.NodeInfo,
 	return p.predicatesReserve(pod, node)
 }
 
-func (p *predicateManagerImpl) PreemptionPredicates(pod *v1.Pod, node *framework.NodeInfo, victims []*v1.Pod, startIndex int) (index int, ok bool) {
+func (p *predicateManagerImpl) PreemptionPredicates(pod *v1.Pod, node *framework.NodeInfo, victims []*v1.Pod, startIndex int) int {
 	ctx := context.Background()
 	state := framework.NewCycleState()
 
@@ -138,7 +142,7 @@ func (p *predicateManagerImpl) PreemptionPredicates(pod *v1.Pod, node *framework
 			zap.String("plugin", plugin),
 			zap.String("message", s.Message()))
 
-		return -1, false
+		return -1
 	}
 
 	// clone node so that we can modify it here for predicate checks
@@ -154,7 +158,7 @@ func (p *predicateManagerImpl) PreemptionPredicates(pod *v1.Pod, node *framework
 		p.removePodFromNodeNoFail(preemptingNode, victims[i])
 		status, _ := p.runFilterPlugins(ctx, *p.allocationFilters, state, pod, preemptingNode, skip)
 		if status.IsSuccess() {
-			return i, true
+			return i
 		}
 	}
 
@@ -162,7 +166,7 @@ func (p *predicateManagerImpl) PreemptionPredicates(pod *v1.Pod, node *framework
 	log.Log(log.ShimPredicates).Debug("Filter checks failed during preemption check, no fit",
 		zap.String("podUID", string(pod.UID)),
 		zap.String("nodeID", node.Node().Name))
-	return -1, false
+	return -1
 }
 
 func (p *predicateManagerImpl) removePodFromNodeNoFail(node *framework.NodeInfo, pod *v1.Pod) {
@@ -178,19 +182,19 @@ func (p *predicateManagerImpl) removePodFromNodeNoFail(node *framework.NodeInfo,
 	}
 }
 
-func (p *predicateManagerImpl) predicatesReserve(pod *v1.Pod, node *framework.NodeInfo) (plugin string, error error) {
+func (p *predicateManagerImpl) predicatesReserve(pod *v1.Pod, node *framework.NodeInfo) (string, error) {
 	ctx := context.Background()
 	state := framework.NewCycleState()
 	return p.podFitsNode(ctx, state, *p.reservationPreFilters, *p.reservationFilters, pod, node)
 }
 
-func (p *predicateManagerImpl) predicatesAllocate(pod *v1.Pod, node *framework.NodeInfo) (plugin string, error error) {
+func (p *predicateManagerImpl) predicatesAllocate(pod *v1.Pod, node *framework.NodeInfo) (string, error) {
 	ctx := context.Background()
 	state := framework.NewCycleState()
 	return p.podFitsNode(ctx, state, *p.allocationPreFilters, *p.allocationFilters, pod, node)
 }
 
-func (p *predicateManagerImpl) podFitsNode(ctx context.Context, state *framework.CycleState, preFilters []framework.PreFilterPlugin, filters []framework.FilterPlugin, pod *v1.Pod, node *framework.NodeInfo) (plugin string, error error) {
+func (p *predicateManagerImpl) podFitsNode(ctx context.Context, state *framework.CycleState, preFilters []framework.PreFilterPlugin, filters []framework.FilterPlugin, pod *v1.Pod, node *framework.NodeInfo) (string, error) {
 	// Run "prefilter" plugins.
 	status, plugin, skip := p.runPreFilterPlugins(ctx, state, preFilters, pod, node)
 	if !status.IsSuccess() && !status.IsSkip() {
@@ -205,27 +209,26 @@ func (p *predicateManagerImpl) podFitsNode(ctx context.Context, state *framework
 	return "", nil
 }
 
-func (p *predicateManagerImpl) runPreFilterPlugins(ctx context.Context, state *framework.CycleState, plugins []framework.PreFilterPlugin, pod *v1.Pod, node *framework.NodeInfo) (status *framework.Status, plugin string, skip map[string]interface{}) {
-	var mergedNodes *framework.PreFilterResult = nil
-	skip = nil
+func (p *predicateManagerImpl) runPreFilterPlugins(ctx context.Context, state *framework.CycleState, plugins []framework.PreFilterPlugin, pod *v1.Pod, node *framework.NodeInfo) (*framework.Status, string, map[string]bool) {
+	var mergedNodes *framework.PreFilterResult
+	skip := make(map[string]bool)
 	for _, pl := range plugins {
+		plugin := pl.Name()
 		nodes, status := p.runPreFilterPlugin(ctx, pl, state, pod)
 		if status.IsSkip() {
-			if skip == nil {
-				skip = make(map[string]interface{})
-			}
-			skip[pl.Name()] = nil
+			skip[plugin] = true
 		} else if !status.IsSuccess() {
 			if status.IsRejected() {
-				return status, plugin, skip
+				return status, "", skip
 			}
 			err := errors.New(status.Message())
 			log.Log(log.ShimPredicates).Error("failed running PreFilter plugin",
-				zap.String("pluginName", pl.Name()),
+				zap.String("pluginName", plugin),
 				zap.String("pod", fmt.Sprintf("%s/%s", pod.Namespace, pod.Name)),
 				zap.Error(err))
-			return framework.AsStatus(fmt.Errorf("running PreFilter plugin %q: %w", pl.Name(), err)), plugin, skip
+			return framework.AsStatus(errors.Join(fmt.Errorf("running PreFilter plugin %q: ", plugin), err)), plugin, skip
 		}
+		// Merge is nil safe and returns a new PreFilterResult result if mergedNodes was nil
 		mergedNodes = mergedNodes.Merge(nodes)
 		if !mergedNodes.AllNodes() && !mergedNodes.NodeNames.Has(node.Node().Name) {
 			return framework.NewStatus(framework.UnschedulableAndUnresolvable, "node not eligible"), plugin, skip
@@ -239,27 +242,24 @@ func (p *predicateManagerImpl) runPreFilterPlugin(ctx context.Context, pl framew
 	return pl.PreFilter(ctx, state, pod)
 }
 
-func (p *predicateManagerImpl) runFilterPlugins(ctx context.Context, plugins []framework.FilterPlugin, state *framework.CycleState, pod *v1.Pod, nodeInfo *framework.NodeInfo, skip map[string]interface{}) (status *framework.Status, plugin string) {
-	plugin = ""
+func (p *predicateManagerImpl) runFilterPlugins(ctx context.Context, plugins []framework.FilterPlugin, state *framework.CycleState, pod *v1.Pod, nodeInfo *framework.NodeInfo, skip map[string]bool) (*framework.Status, string) {
 	for _, pl := range plugins {
+		plugin := pl.Name()
 		// skip plugin if prefilter returned skip
-		if _, ok := skip[pl.Name()]; ok {
+		if skip[plugin] {
 			continue
 		}
 		status := p.runFilterPlugin(ctx, pl, state, pod, nodeInfo)
 		if !status.IsSuccess() {
-			if plugin == "" {
-				plugin = pl.Name()
-			}
 			if !status.IsRejected() {
 				// Filter plugins are not supposed to return any status other than
 				// Success or Unschedulable.
-				status = framework.NewStatus(framework.Error, fmt.Sprintf("running %q filter plugin for pod %q: %v", pl.Name(), pod.Name, status.Message()))
+				status = framework.NewStatus(framework.Error, fmt.Sprintf("running %q filter plugin for pod %q: %v", plugin, pod.Name, status.Message()))
 				log.Log(log.ShimPredicates).Error("failed running Filter plugin",
-					zap.String("pluginName", pl.Name()),
+					zap.String("pluginName", plugin),
 					zap.String("pod", fmt.Sprintf("%s/%s", pod.Namespace, pod.Name)),
 					zap.String("message", status.Message()))
-				return status, pl.Name()
+				return status, plugin
 			}
 			return status, plugin
 		}
@@ -273,11 +273,12 @@ func (p *predicateManagerImpl) runFilterPlugin(ctx context.Context, pl framework
 
 func NewPredicateManager(handle framework.Handle) PredicateManager {
 	/*
-		Default K8S plugins as of 1.27 that implement PreFilter:
+		Default K8S plugins as of 1.31 that implement PreFilter:
 			NodeAffinity
 			NodePorts
 			Fit
 			VolumeRestrictions
+			CSILimits
 			VolumeBinding
 			VolumeZone
 			PodTopologySpread
@@ -291,6 +292,7 @@ func NewPredicateManager(handle framework.Handle) PredicateManager {
 		names.PodTopologySpread: true,
 		names.InterPodAffinity:  true,
 		// Fit : skip because during reservation, node resources are not enough
+		// CSILimits
 		// VolumeRestrictions
 		// VolumeBinding
 		// VolumeZone
@@ -302,7 +304,7 @@ func NewPredicateManager(handle framework.Handle) PredicateManager {
 	}
 
 	/*
-		Default K8S plugins as of 1.27 that implement Filter:
+		Default K8S plugins as of 1.31 that implement Filter:
 		    NodeUnschedulable
 			NodeName
 			TaintToleration
@@ -310,10 +312,7 @@ func NewPredicateManager(handle framework.Handle) PredicateManager {
 			NodePorts
 			Fit
 			VolumeRestrictions
-			EBSLimits [nonCSILimits]
-			GCEPDLimits [nonCSILimits]
 			CSILimits
-			AzureDiskLimits [nonCSILimits]
 			VolumeBinding
 			VolumeZone
 			PodTopologySpread
@@ -331,10 +330,7 @@ func NewPredicateManager(handle framework.Handle) PredicateManager {
 		names.InterPodAffinity:  true,
 		// Fit : skip because during reservation, node resources are not enough
 		// VolumeRestrictions
-		// EBSLimits [nonCSILimits]
-		// GCEPDLimits [nonCSILimits]
 		// CSILimits
-		// AzureDiskLimits [nonCSILimits]
 		// VolumeBinding
 		// VolumeZone
 	}
