@@ -1099,7 +1099,7 @@ func makeResources(milliCPU, memory, pods, extendedA, storage, hugePageA int64)
 func newPodWithPort(hostPorts ...int) *v1.Pod {
 	var networkPorts []v1.ContainerPort
 	for _, port := range hostPorts {
-		networkPorts = append(networkPorts, v1.ContainerPort{HostPort: int32(port)})
+		networkPorts = append(networkPorts, v1.ContainerPort{HostPort: int32(port)}) // nolint: gosec
 	}
 	return &v1.Pod{
 		Spec: v1.PodSpec{
