@@ -19,9 +19,11 @@
 package simple_preemptor_test
 
 import (
+	"path/filepath"
 	"testing"
 
-	"github.com/onsi/ginkgo"
+	"github.com/onsi/ginkgo/v2"
+	"github.com/onsi/ginkgo/v2/reporters"
 	"github.com/onsi/gomega"
 
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/configmanager"
@@ -32,8 +34,17 @@ func init() {
 }
 
 func TestSimplePreemptor(t *testing.T) {
+	ginkgo.ReportAfterSuite("TestSimplePreemptor", func(report ginkgo.Report) {
+		err := reporters.GenerateJUnitReportWithConfig(
+			report,
+			filepath.Join(configmanager.YuniKornTestConfig.LogDir, "TEST-simple_preemptor_junit.xml"),
+			reporters.JunitReportConfig{OmitSpecLabels: true},
+		)
+		Ω(err).NotTo(HaveOccurred())
+	})
 	gomega.RegisterFailHandler(ginkgo.Fail)
-	ginkgo.RunSpecs(t, "SimplePreemptor Suite")
+	ginkgo.RunSpecs(t, "TestSimplePreemptor", ginkgo.Label("TestSimplePreemptor"))
 }
 
 var Ω = gomega.Ω
+var HaveOccurred = gomega.HaveOccurred
