@@ -26,11 +26,11 @@ export class AppInfo {
   constructor(
     public applicationId: string,
     public usedResource: string,
+    public pendingResource: string,
     public maxUsedResource: string,
-    public partition: string,
-    public queueName: string,
     public submissionTime: number,
     public finishedTime: null | number,
+    public stateLog: Array<StateLog>,
     public lastStateChangeTime: null | number,
     public applicationState: string,
     public allocations: AllocationInfo[] | null
@@ -41,6 +41,14 @@ export class AppInfo {
     return moment(millisecs).format('YYYY/MM/DD HH:mm:ss');
   }
 
+  get formattedlastStateChangeTime() {
+    if(this.lastStateChangeTime==null){
+      return 'n/a'
+    }
+    const millisecs = Math.round(this.lastStateChangeTime! / (1000 * 1000));
+    return moment(millisecs).format('YYYY/MM/DD HH:mm:ss');
+  }
+
   get formattedFinishedTime() {
     if (this.finishedTime) {
       const millisecs = Math.round(this.finishedTime / (1000 * 1000));
@@ -53,4 +61,21 @@ export class AppInfo {
   setAllocations(allocs: AllocationInfo[]) {
     this.allocations = allocs;
   }
+
+  setLastStateChangeTime() {
+    let time=0
+    this.stateLog.forEach(log => {
+      if (log.time>time){
+        time=log.time
+      }
+    });
+    this.lastStateChangeTime=time
+  }
+}
+
+export class StateLog{
+  constructor(
+    public time: number,
+    public applicationState: string
+  ) {}
 }
