@@ -19,11 +19,31 @@
 export class ChartDataItem {
   name: string;
   value: number;
-  color: string;
+  color?: string;
+  description?: string;
 
-  constructor(name: string, value: number, color: string) {
+  constructor(name: string, value: number, color?: string, description?: string) {
     this.name = name;
     this.value = value;
     this.color = color;
+    this.description = description;
+  }
+}
+
+export class BarChartDataSet {
+  label: string;
+  data: number[];
+  avgUtilizationRate: number;
+  backgroundColor: string;
+  borderWidth: number;
+  description: string[];
+
+  constructor(label: string, data: number[], avgUtilizationRate: number, backgroundColor: string, borderWidth: number, description: string[]) {
+    this.label = label;
+    this.data = data;
+    this.avgUtilizationRate = avgUtilizationRate;
+    this.backgroundColor = backgroundColor;
+    this.borderWidth = borderWidth;
+    this.description = description;
   }
 }
