@@ -24,5 +24,13 @@ export class NodeUtilization {
       numOfNodes: number;
       nodeNames: null | string[];
     }[],
-  ) {}
+  ) { }
+}
+
+export class NodeUtilizationsInfo {
+  constructor(
+    public clusterId: string,
+    public partition: string,
+    public utilizations: NodeUtilization[],
+  ) { }
 }
