@@ -18,18 +18,17 @@
 
 import {HttpClientTestingModule} from '@angular/common/http/testing';
 import {TestBed} from '@angular/core/testing';
-import {configureTestSuite} from 'ng-bullet';
 
 import {EnvconfigService} from './envconfig.service';
 
 describe('EnvconfigService', () => {
   let service: EnvconfigService;
 
-  configureTestSuite(() => {
+  beforeAll(() => {
     TestBed.configureTestingModule({
       imports: [HttpClientTestingModule],
       providers: [EnvconfigService],
-    });
+    }).compileComponents();
   });
 
   beforeEach(() => {
